@@ -142,58 +142,79 @@ public class HomeFramePane extends JRootPane implements View {
       homePane.setJMenuBar(null);
     }
     
-    // Add listeners to model and frame    
-    addListeners(this.home, this.application, this.controller.getHomeController(), homeFrame);
-    
+    // Add listeners to model and component   
+    addModelListeners(this.home, this.application, homeFrame);
+    addComponentListener(this.home, this.application, this.controller.getHomeController(), homeFrame);
+    homeFrame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
     homeFrame.setVisible(true);
-    // Request the frame to go to front again because closing waiting dialog meanwhile  
-    // could put in front the already opened frame  
     EventQueue.invokeLater(new Runnable() {
         public void run() {
+          // Add listeners to frame once it's visible to avoid some undesired events during first showing    
+          addWindowListeners(home, application, controller.getHomeController(), homeFrame);
+          // Request the frame to go to front again because closing waiting dialog meanwhile  
+          // could put in front the already opened frame  
           homeFrame.toFront();
         }
       });
   }
   
   /**
-   * Adds listeners to <code>frame</code> and model objects.
+   * Adds a component listener to <code>frame</code>.
    */
-  private void addListeners(final Home home,
-                            final HomeApplication application,
-                            final HomeController controller,
-                            final JFrame frame) {
+  private void addComponentListener(final Home home,
+                                    final HomeApplication application,
+                                    final HomeController controller,
+                                    final JFrame frame) {
     // Add a listener that keeps track of window location and size
-    frame.addComponentListener(new ComponentAdapter() {
+    final ComponentAdapter componentListener = new ComponentAdapter() {
         @Override
         public void componentResized(ComponentEvent ev) {
           // Store new size only if frame isn't maximized
           if ((frame.getExtendedState() & JFrame.MAXIMIZED_BOTH) != JFrame.MAXIMIZED_BOTH) {
-            controller.setVisualProperty(FRAME_WIDTH_VISUAL_PROPERTY, frame.getWidth());
-            controller.setVisualProperty(FRAME_HEIGHT_VISUAL_PROPERTY, frame.getHeight());
+            controller.setHomeProperty(FRAME_WIDTH_VISUAL_PROPERTY, String.valueOf(frame.getWidth()));
+            controller.setHomeProperty(FRAME_HEIGHT_VISUAL_PROPERTY, String.valueOf(frame.getHeight()));
           }
           Dimension userScreenSize = getUserScreenSize();
-          controller.setVisualProperty(SCREEN_WIDTH_VISUAL_PROPERTY, userScreenSize.width);
-          controller.setVisualProperty(SCREEN_HEIGHT_VISUAL_PROPERTY, userScreenSize.height);
+          controller.setHomeProperty(SCREEN_WIDTH_VISUAL_PROPERTY, String.valueOf(userScreenSize.width));
+          controller.setHomeProperty(SCREEN_HEIGHT_VISUAL_PROPERTY, String.valueOf(userScreenSize.height));
         }
         
         @Override
         public void componentMoved(ComponentEvent ev) {
           // Store new location only if frame isn't maximized
           if ((frame.getExtendedState() & JFrame.MAXIMIZED_BOTH) != JFrame.MAXIMIZED_BOTH) {
-            controller.setVisualProperty(FRAME_X_VISUAL_PROPERTY, frame.getX());
-            controller.setVisualProperty(FRAME_Y_VISUAL_PROPERTY, frame.getY());
+            controller.setHomeProperty(FRAME_X_VISUAL_PROPERTY, String.valueOf(frame.getX()));
+            controller.setHomeProperty(FRAME_Y_VISUAL_PROPERTY, String.valueOf(frame.getY()));
           }
         }
+      };
+    frame.addComponentListener(componentListener);
+    application.addHomesListener(new CollectionListener<Home>() {
+        public void collectionChanged(CollectionEvent<Home> ev) {
+          if (ev.getItem() == home
+              && ev.getType() == CollectionEvent.Type.DELETE) {
+            application.removeHomesListener(this);
+            frame.removeComponentListener(componentListener);
+          }
+        };
       });
+  }
+    
+  /**
+   * Adds window listeners to <code>frame</code>.
+   */
+  private void addWindowListeners(final Home home,
+                                  final HomeApplication application,
+                                  final HomeController controller,
+                                  final JFrame frame) {
     // Control frame closing and activation 
-    frame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
-    WindowAdapter windowListener = new WindowAdapter () {
+    final WindowAdapter windowListener = new WindowAdapter () {
         private Component mostRecentFocusOwner;
 
         @Override
         public void windowStateChanged(WindowEvent ev) {
-          controller.setVisualProperty(FRAME_MAXIMIZED_VISUAL_PROPERTY, 
-              (frame.getExtendedState() & JFrame.MAXIMIZED_BOTH) == JFrame.MAXIMIZED_BOTH);
+          controller.setHomeProperty(FRAME_MAXIMIZED_VISUAL_PROPERTY, 
+              String.valueOf((frame.getExtendedState() & JFrame.MAXIMIZED_BOTH) == JFrame.MAXIMIZED_BOTH));
         }
         
         @Override
@@ -229,9 +250,6 @@ public class HomeFramePane extends JRootPane implements View {
       };
     frame.addWindowListener(windowListener);    
     frame.addWindowStateListener(windowListener);    
-    // Add a listener to preferences to apply component orientation to frame matching current language
-    application.getUserPreferences().addPropertyChangeListener(UserPreferences.Property.LANGUAGE, 
-        new LanguageChangeListener(frame, this));
     // Dispose window when a home is deleted 
     application.addHomesListener(new CollectionListener<Home>() {
         public void collectionChanged(CollectionEvent<Home> ev) {
@@ -239,9 +257,22 @@ public class HomeFramePane extends JRootPane implements View {
               && ev.getType() == CollectionEvent.Type.DELETE) {
             application.removeHomesListener(this);
             frame.dispose();
+            frame.removeWindowListener(windowListener);
+            frame.removeWindowStateListener(windowListener);
           }
         };
       });
+  }
+    
+  /**
+   * Adds listeners to model objects.
+   */
+  private void addModelListeners(final Home home,
+                                 final HomeApplication application,
+                                 final JFrame frame) {
+    // Add a listener to preferences to apply component orientation to frame matching current language
+    application.getUserPreferences().addPropertyChangeListener(UserPreferences.Property.LANGUAGE, 
+        new LanguageChangeListener(frame, this));
     // Update title when the name or the modified state of home changes
     PropertyChangeListener frameTitleChangeListener = new PropertyChangeListener () {
         public void propertyChange(PropertyChangeEvent ev) {
@@ -285,23 +316,23 @@ public class HomeFramePane extends JRootPane implements View {
    * Computes <code>frame</code> size and location to fit into screen.
    */
   private void computeFrameBounds(Home home, final JFrame frame) {
-    Integer x = (Integer)home.getVisualProperty(FRAME_X_VISUAL_PROPERTY);
-    Integer y = (Integer)home.getVisualProperty(FRAME_Y_VISUAL_PROPERTY);
-    Integer width = (Integer)home.getVisualProperty(FRAME_WIDTH_VISUAL_PROPERTY);
-    Integer height = (Integer)home.getVisualProperty(FRAME_HEIGHT_VISUAL_PROPERTY);
-    Boolean maximized = (Boolean)home.getVisualProperty(FRAME_MAXIMIZED_VISUAL_PROPERTY);
-    Integer screenWidth = (Integer)home.getVisualProperty(SCREEN_WIDTH_VISUAL_PROPERTY);
-    Integer screenHeight = (Integer)home.getVisualProperty(SCREEN_HEIGHT_VISUAL_PROPERTY);
+    Number x = home.getNumericProperty(FRAME_X_VISUAL_PROPERTY);
+    Number y = home.getNumericProperty(FRAME_Y_VISUAL_PROPERTY);
+    Number width = home.getNumericProperty(FRAME_WIDTH_VISUAL_PROPERTY);
+    Number height = home.getNumericProperty(FRAME_HEIGHT_VISUAL_PROPERTY);
+    boolean maximized = Boolean.parseBoolean(home.getProperty(FRAME_MAXIMIZED_VISUAL_PROPERTY));
+    Number screenWidth = home.getNumericProperty(SCREEN_WIDTH_VISUAL_PROPERTY);
+    Number screenHeight = home.getNumericProperty(SCREEN_HEIGHT_VISUAL_PROPERTY);
     
     Dimension screenSize = getUserScreenSize();
     // If home frame bounds exist and screen resolution didn't reduce 
     if (x != null && y != null 
         && width != null && height != null 
         && screenWidth != null && screenHeight != null
-        && screenWidth <= screenSize.width
-        && screenHeight <= screenSize.height) {
-      final Rectangle frameBounds = new Rectangle(x, y, width, height);
-      if (maximized != null && maximized) {
+        && screenWidth.intValue() <= screenSize.width
+        && screenHeight.intValue() <= screenSize.height) {
+      final Rectangle frameBounds = new Rectangle(x.intValue(), y.intValue(), width.intValue(), height.intValue());
+      if (maximized) {
         if (OperatingSystem.isMacOSX() 
             && OperatingSystem.isJavaVersionGreaterOrEqual("1.7")) {
           // Display the frame at its maximum size because calling setExtendedState to maximize 
@@ -320,7 +351,7 @@ public class HomeFramePane extends JRootPane implements View {
           frame.setExtendedState(JFrame.MAXIMIZED_BOTH);
         }
         // Add a listener that will set the normal size when the frame leaves the maximized state
-        frame.addWindowStateListener(new WindowAdapter() {
+        WindowAdapter windowStateListener = new WindowAdapter() {
             @Override
             public void windowStateChanged(WindowEvent ev) {
               if ((ev.getOldState() == JFrame.MAXIMIZED_BOTH 
@@ -344,7 +375,15 @@ public class HomeFramePane extends JRootPane implements View {
                 frame.removeWindowStateListener(this);
               }
             }
-          });
+            
+            @Override
+            public void windowClosed(WindowEvent ev) {
+              frame.removeWindowListener(this);
+              frame.removeWindowStateListener(this);
+            }
+          };
+        frame.addWindowStateListener(windowStateListener);
+        frame.addWindowListener(windowStateListener);
       } else {
         // Reuse home bounds
         frame.setBounds(frameBounds);
@@ -356,8 +395,8 @@ public class HomeFramePane extends JRootPane implements View {
       frame.setSize(Math.min(screenSize.width * 4 / 5, frame.getWidth()), 
               Math.min(screenSize.height * 4 / 5, frame.getHeight()));
       if (OperatingSystem.isMacOSX() 
-          && OperatingSystem.isJavaVersionGreaterOrEqual("1.7")) {
-        // JFrame#setLocationByPlatform does nothing under Java 7
+          && OperatingSystem.isJavaVersionBetween("1.7", "1.9")) {
+        // JFrame#setLocationByPlatform does nothing under Java 7/8
         int minX = Integer.MAX_VALUE;
         int minY = Integer.MAX_VALUE;
         int maxX = Integer.MIN_VALUE;
@@ -382,7 +421,7 @@ public class HomeFramePane extends JRootPane implements View {
         } else {
           y = maxY + 23;
         }
-        frame.setLocation(x, y);
+        frame.setLocation(x.intValue(), y.intValue());
       }
     }
   }
