@@ -35,6 +35,7 @@ import java.awt.event.ActionEvent;
 import java.awt.event.MouseEvent;
 import java.awt.event.WindowAdapter;
 import java.awt.event.WindowEvent;
+import java.awt.event.WindowStateListener;
 import java.io.IOException;
 import java.lang.reflect.InvocationTargetException;
 import java.util.List;
@@ -105,20 +106,9 @@ class MacOSXConfiguration {
     final HomeController defaultController = 
         homeApplication.createHomeFrameController(homeApplication.createHome()).getHomeController();
     final HomePane defaultHomeView = (HomePane)defaultController.getView();
-    for (HomePane.ActionType action : HomePane.ActionType.values()) {
-      switch (action) {
-        case ABOUT :
-        case NEW_HOME :
-        case OPEN :
-        case DELETE_RECENT_HOMES :
-        case HELP :
-          break;
-        default :
-          defaultHomeView.setEnabled(action, false);
-      }
-    }
-
+    setDefaultActionsEnabled(defaultHomeView, false);
     final JMenuBar defaultMenuBar = defaultHomeView.getJMenuBar();
+    
     JFrame frame = null;
     try {
       if (OperatingSystem.isJavaVersionBetween("1.7", "1.7.0_60")) {
@@ -126,7 +116,7 @@ class MacOSXConfiguration {
         frame = createDummyFrameWithDefaultMenuBar(homeApplication, defaultHomeView, defaultMenuBar);
       } else if (UIManager.getLookAndFeel().getClass().getName().equals(UIManager.getSystemLookAndFeelClassName())) {
         macosxApplication.setDefaultMenuBar(defaultMenuBar);
-        addWindowMenu(null, defaultMenuBar, homeApplication, true);
+        addWindowMenu(null, defaultMenuBar, homeApplication, defaultHomeView, true);
       }
     } catch (NoSuchMethodError ex) {
       // Create default frame if setDefaultMenuBar isn't available
@@ -139,13 +129,38 @@ class MacOSXConfiguration {
     macosxApplication.addApplicationListener(new ApplicationAdapter() {      
       @Override
       public void handleQuit(ApplicationEvent ev) { 
-        handleAction(new Runnable() {
-            public void run() {
-              getHomeController().exit();
-            }
-          });
-        if (homeApplication.getHomes().isEmpty()) {
-          System.exit(0);
+        Home modifiedHome = null;
+        int modifiedHomesCount = 0;
+        for (Home home : homeApplication.getHomes()) {
+          if (home.isModified()) {
+            modifiedHome = home;
+            modifiedHomesCount++;
+          }
+        }
+        
+        if (modifiedHomesCount == 1) {
+          // If only one home is modified, close it and exit if it was successfully closed
+          homeApplication.getHomeFrame(modifiedHome).toFront();
+          homeApplication.getHomeFrameController(modifiedHome).getHomeController().close(
+              new Runnable() {
+                public void run() {
+                  for (Home home : homeApplication.getHomes()) {
+                    if (home.isModified()) {
+                      return;
+                    }
+                  }
+                  System.exit(0);
+                }
+              });
+        } else {
+          handleAction(new Runnable() {
+              public void run() {
+                getHomeController().exit();
+              }
+            });
+          if (homeApplication.getHomes().isEmpty()) {
+            System.exit(0);
+          }
         }
       }
       
@@ -301,7 +316,7 @@ class MacOSXConfiguration {
           }
           // Add Mac OS X Window menu on new homes
           MacOSXConfiguration.addWindowMenu(
-              homeFrame, homeFrame.getJMenuBar(), homeApplication, false);
+              homeFrame, homeFrame.getJMenuBar(), homeApplication, defaultHomeView, false);
           
           if (OperatingSystem.isJavaVersionBetween("1.7", "1.7.0_60")) {
             // Help system to understand it should display the main menu of one of the remaining windows when a window is closed
@@ -318,6 +333,17 @@ class MacOSXConfiguration {
                 }
               });
           }
+          homeFrame.addWindowStateListener(new WindowStateListener() {
+              public void windowStateChanged(WindowEvent ev) {
+                // Enable default actions if needed
+                enableDefaultActions(homeApplication, defaultHomeView);
+              }
+            });
+          // Don't enable actions in default menu bar (the menu bar might be displayed when file dialogs are displayed)
+          setDefaultActionsEnabled(defaultHomeView, false);
+        } else if (ev.getType() == CollectionEvent.Type.DELETE) {
+          // Enable default actions if needed
+          enableDefaultActions(homeApplication, defaultHomeView);
         }
       };
     });
@@ -335,6 +361,39 @@ class MacOSXConfiguration {
     }
   }
 
+  /**
+   * Enables default menu bar actions if no window is at screen.
+   */
+  private static void enableDefaultActions(SweetHome3D homeApplication, HomePane defaultHomeView) {
+    for (Home home : homeApplication.getHomes()) {
+      if ((homeApplication.getHomeFrame(home).getState() & JFrame.ICONIFIED) == 0) {
+        setDefaultActionsEnabled(defaultHomeView, false);
+        return;
+      }
+    }
+    // If all homes are iconified, enable actions in default menu bar
+    setDefaultActionsEnabled(defaultHomeView, true);
+  }
+  
+  /**
+   * Enables / disables default actions in the given view.
+   */
+  private static void setDefaultActionsEnabled(HomePane homeView, boolean enabled) {
+    for (HomePane.ActionType action : HomePane.ActionType.values()) {
+      switch (action) {
+        case ABOUT :
+        case NEW_HOME :
+        case OPEN :
+        case DELETE_RECENT_HOMES :
+        case HELP :
+          homeView.setEnabled(action, enabled);
+          break;
+        default :
+          homeView.setEnabled(action, false);
+      }
+    }
+  }
+
   /**
    * Returns a dummy frame used to display the default menu bar.
    */
@@ -352,7 +411,7 @@ class MacOSXConfiguration {
           frame.setVisible(true);
           frame.setJMenuBar(defaultMenuBar);
           frame.setContentPane(defaultHomeView);
-          addWindowMenu(frame, defaultMenuBar, homeApplication, true);
+          addWindowMenu(frame, defaultMenuBar, homeApplication, defaultHomeView, true);
         }
       });
     homeApplication.addHomesListener(new CollectionListener<Home>() {
@@ -381,6 +440,7 @@ class MacOSXConfiguration {
   private static void addWindowMenu(final JFrame frame, 
                                     final JMenuBar menuBar, 
                                     final SweetHome3D homeApplication,
+                                    final HomePane defaultHomeView, 
                                     boolean defaultFrame) {
     UserPreferences preferences = homeApplication.getUserPreferences();
     final JMenu windowMenu = new JMenu(
