@@ -19,10 +19,20 @@
  */
 package com.eteks.sweethome3d;
 
+import java.awt.Component;
 import java.awt.EventQueue;
+import java.awt.Graphics;
+import java.awt.Graphics2D;
+import java.awt.Insets;
 import java.awt.KeyboardFocusManager;
+import java.awt.Polygon;
+import java.awt.RenderingHints;
+import java.awt.Shape;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
+import java.awt.geom.Area;
+import java.awt.geom.Ellipse2D;
+import java.awt.geom.Rectangle2D;
 import java.beans.PropertyChangeEvent;
 import java.beans.PropertyChangeListener;
 import java.io.BufferedReader;
@@ -59,9 +69,12 @@ import javax.jnlp.UnavailableServiceException;
 import javax.swing.JComponent;
 import javax.swing.JFrame;
 import javax.swing.JOptionPane;
+import javax.swing.JSplitPane;
 import javax.swing.SwingUtilities;
 import javax.swing.Timer;
 import javax.swing.UIManager;
+import javax.swing.border.Border;
+import javax.swing.plaf.basic.BasicSplitPaneDivider;
 
 import com.eteks.sweethome3d.io.AutoRecoveryManager;
 import com.eteks.sweethome3d.io.FileUserPreferences;
@@ -165,7 +178,7 @@ public class SweetHome3D extends HomeApplication {
   public HomeRecorder getHomeRecorder() {
     // Initialize homeRecorder lazily
     if (this.homeRecorder == null) {
-      this.homeRecorder = new HomeFileRecorder(0, false, getUserPreferences(), false);
+      this.homeRecorder = new HomeFileRecorder(0, false, getUserPreferences(), false, true);
     }
     return this.homeRecorder;
   }
@@ -175,7 +188,7 @@ public class SweetHome3D extends HomeApplication {
     if (type == HomeRecorder.Type.COMPRESSED) {
       // Initialize compressedHomeRecorder lazily
       if (this.compressedHomeRecorder == null) {
-        this.compressedHomeRecorder = new HomeFileRecorder(9, false, getUserPreferences(), false);
+        this.compressedHomeRecorder = new HomeFileRecorder(9, false, getUserPreferences(), false, true);
       }
       return this.compressedHomeRecorder;
     } else {
@@ -214,8 +227,9 @@ public class SweetHome3D extends HomeApplication {
       this.userPreferences = new FileUserPreferences(preferencesFolder, applicationFolders, eventQueueExecutor) {
           @Override
           public List<Library> getLibraries() {
-            if (pluginManager != null) {
-              List<Library> pluginLibraries = pluginManager.getPluginLibraries();
+            if (userPreferences != null // Don't go further if preferences are not ready
+                && getPluginManager() != null) {
+              List<Library> pluginLibraries = getPluginManager().getPluginLibraries();
               if (!pluginLibraries.isEmpty()) {
                 // Add plug-ins to the list returned by user preferences
                 ArrayList<Library> libraries = new ArrayList<Library>(super.getLibraries());
@@ -228,14 +242,17 @@ public class SweetHome3D extends HomeApplication {
           
           @Override
           public void deleteLibraries(List<Library> libraries) throws RecorderException {
-            super.deleteLibraries(libraries);
-            List<Library> plugins = new ArrayList<Library>();
-            for (Library library : libraries) {
-              if (PluginManager.PLUGIN_LIBRARY_TYPE.equals(library.getType())) {
-                plugins.add(library);
+            if (userPreferences != null // Don't go further if preferences are not ready
+                && getPluginManager() != null) {
+              super.deleteLibraries(libraries);
+              List<Library> plugins = new ArrayList<Library>();
+              for (Library library : libraries) {
+                if (PluginManager.PLUGIN_LIBRARY_TYPE.equals(library.getType())) {
+                  plugins.add(library);
+                }
               }
+              getPluginManager().deletePlugins(plugins);
             }
-            pluginManager.deletePlugins(plugins);            
           }
         };
       this.checkUpdatesNeeded = this.userPreferences.isCheckUpdatesEnabled();
@@ -346,7 +363,7 @@ public class SweetHome3D extends HomeApplication {
 
   /**
    * Sweet Home 3D entry point.
-   * @param args may contain one .sh3d, .sh3f or .sh3p file to open or install,
+   * @param args may contain one .sh3d, .sh3x, .sh3f, .sh3t, .sh3l or .sh3p file to open or install,
    *          following a <code>-open</code> option.
    */
   public static void main(final String [] args) {
@@ -502,7 +519,7 @@ public class SweetHome3D extends HomeApplication {
       // Force the use of Quartz under Mac OS X for better Java 2D rendering performance
       System.setProperty("apple.awt.graphics.UseQuartz", "true");
       if (System.getProperty("com.eteks.sweethome3d.dragAndDropWithoutTransferHandler") == null
-          && OperatingSystem.isJavaVersionGreaterOrEqual("1.7")) {
+          && OperatingSystem.isJavaVersionBetween("1.7", "1.8.0_40")) {
         System.setProperty("com.eteks.sweethome3d.dragAndDropWithoutTransferHandler", "true");
       }
     }
@@ -523,6 +540,37 @@ public class SweetHome3D extends HomeApplication {
       if (OperatingSystem.isMacOSXLeopardOrSuperior()) {
         UIManager.put("TitledBorder.border", UIManager.getBorder("TitledBorder.aquaVariant"));
       }
+      if (OperatingSystem.isMacOSXYosemiteOrSuperior()) {
+        UIManager.put("SplitPaneDivider.border", new Border() {
+            public void paintBorder(Component c, Graphics g, int x, int y, int width, int height) {
+              // Set a border that fills the divider with the expected background color (instead of white)
+              // except at the place where the buttons are already drawn 
+              ((Graphics2D)g).setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
+              g.setColor(UIManager.getColor("SplitPane.background"));
+              Shape clip = g.getClip();
+              Area clipArea = new Area(new Rectangle2D.Float(x - 0.5f, y - 0.5f, width + 1f, height + 1f));
+              clipArea.subtract(new Area(new Ellipse2D.Float(x + width / 2f - 3.4f, y + height / 2f - 3.2f, 6.8f, 6.8f)));
+              JSplitPane splitPane = ((BasicSplitPaneDivider)c).getBasicSplitPaneUI().getSplitPane();
+              if (splitPane.getOrientation() == JSplitPane.VERTICAL_SPLIT) {
+                clipArea.subtract(new Area(new Polygon(new int [] {x, x + 4, x + 8}, 
+                    new int [] {y + height / 2 + 3, y + height / 2 - 3, y + height / 2 + 3}, 3)));
+                clipArea.subtract(new Area(new Polygon(new int [] {x + 12, x + 15, x + 19}, 
+                    new int [] {y + height / 2 - 2, y + height / 2 + 3, y + height / 2 - 2}, 3)));
+              }
+              g.setClip(clipArea);
+              g.fillRect(x, y + height / 2 - 5, x + width, 11);
+              g.setClip(clip);
+            }
+            
+            public boolean isBorderOpaque() {
+              return true;
+            }
+            
+            public Insets getBorderInsets(Component c) {
+              return new Insets(0, 0, 0, 0);
+            }
+          });
+      }
       SwingTools.updateSwingResourceLanguage(getUserPreferences());
     } catch (Exception ex) {
       // Too bad keep current look and feel
@@ -541,7 +589,19 @@ public class SweetHome3D extends HomeApplication {
             if (ev.getItem().getName() != null 
                 && home.getName() == null
                 && !home.isRecovered()) {
-              controller.close();
+              if (OperatingSystem.isMacOSXLionOrSuperior()
+                  && OperatingSystem.isJavaVersionGreaterOrEqual("1.7")
+                  && MacOSXConfiguration.isWindowFullScreen(getHomeFrame(home))) {
+                // Delay home disposal to avoid Java 3D fatal error
+                new Timer(3000, new ActionListener() {
+                    public void actionPerformed(ActionEvent ev) {
+                      ((Timer)ev.getSource()).stop();
+                      controller.close();
+                    }
+                  }).start();
+              } else {
+                controller.close();
+              }
             }
             removeHomesListener(this);
           } else if (ev.getItem() == home && ev.getType() == CollectionEvent.Type.DELETE) {
