@@ -184,7 +184,8 @@ public final class ViewerHelper {
     // Force offscreen in 3D view under Plugin 2 / Java 6 / Mac OS X
     if (OperatingSystem.isMacOSX()            
         && applet.getAppletContext() != null
-        && applet.getAppletContext().getClass().getName().startsWith("sun.plugin2.applet.Plugin2Manager")) {
+        && applet.getAppletContext().getClass().getName().startsWith("sun.plugin2.applet.Plugin2Manager")
+        && OperatingSystem.isJavaVersionBetween("1.6", "1.7")) {
       System.setProperty("com.eteks.sweethome3d.j3d.useOffScreen3DView", "true");
     }
 
