@@ -31,6 +31,7 @@ import java.net.MalformedURLException;
 import java.net.URL;
 import java.net.URLConnection;
 import java.util.ArrayList;
+import java.util.ConcurrentModificationException;
 import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.List;
@@ -179,7 +180,6 @@ public class DAELoader extends LoaderBase implements Loader {
     try {
       SceneBase scene = new SceneBase();
       SAXParserFactory factory = SAXParserFactory.newInstance();
-      factory.setValidating(false);
       SAXParser saxParser = factory.newSAXParser();
       saxParser.parse(in, new DAEHandler(scene, baseUrl));
       return scene;
@@ -265,7 +265,7 @@ public class DAELoader extends LoaderBase implements Loader {
 
     @Override
     public void startElement(String uri, String localName, String name, Attributes attributes) throws SAXException {
-      buffer.setLength(0);
+      this.buffer.setLength(0);
       String parent = this.parentElements.isEmpty() ? null : this.parentElements.peek();
       if (parent == null && !"COLLADA".equals(name)) {
         throw new SAXException("Expected COLLADA element");
@@ -636,9 +636,19 @@ public class DAELoader extends LoaderBase implements Loader {
       if ("init_from".equals(name)) {
         InputStream in = null;
         try {
-          URL textureImageUrl = new URL(this.baseUrl, getCharacters());
+          // Don't interpret possible # sign in URLs as anchor separator
+          URL textureImageUrl = new URL(this.baseUrl, getCharacters().replace("#", "%23"));
           in = openStream(textureImageUrl);
-          BufferedImage textureImage = ImageIO.read(in);
+          BufferedImage textureImage;
+          try {
+            textureImage = ImageIO.read(in);
+          } catch (ConcurrentModificationException ex) {
+            // Try to read the image once more, 
+            // see unfixed Java bug http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6986863
+            in.close();
+            in = openStream(textureImageUrl);
+            textureImage = ImageIO.read(in);
+          }
           if (textureImage != null) {
             TextureLoader textureLoader = new TextureLoader(textureImage);
             Texture texture = textureLoader.getTexture();
@@ -824,13 +834,15 @@ public class DAELoader extends LoaderBase implements Loader {
             this.polygonsHoles.get(this.polygonsPrimitives.size() - 1).add(integers);
           } 
         }
-      } else if ("triangles".equals(name)
-                 || "trifans".equals(name)
-                 || "tristrips".equals(name)
-                 || "polylist".equals(name)
-                 || "polygons".equals(name)
-                 || "lines".equals(name)
-                 || "linestrips".equals(name)) {
+      } else if (("triangles".equals(name)
+                  || "trifans".equals(name)
+                  || "tristrips".equals(name)
+                  || "polylist".equals(name)
+                  || "polygons".equals(name)
+                  || "lines".equals(name)
+                  || "linestrips".equals(name))
+                 // Ignore geometries with missing vertices
+                 && this.geometryVertices != null) {
         Geometry geometry;
         if ("lines".equals(name)
             || "linestrips".equals(name)) {
@@ -838,9 +850,11 @@ public class DAELoader extends LoaderBase implements Loader {
         } else {
           geometry = getFacesGeometry(name);
         }
-        this.geometries.get(this.geometryId).add(geometry);
-        if (this.geometryAppearance != null) {
-          this.geometryAppearances.put(geometry, this.geometryAppearance);
+        if (geometry != null) {
+          this.geometries.get(this.geometryId).add(geometry);
+          if (this.geometryAppearance != null) {
+            this.geometryAppearances.put(geometry, this.geometryAppearance);
+          }
         }
         this.geometryAppearance = null;
         this.geometryVertices = null;
@@ -947,28 +961,33 @@ public class DAELoader extends LoaderBase implements Loader {
       }
       
       int [] coordinatesIndices = getIndices(this.geometryVertexOffset);
-      IndexedGeometryArray geometry;
-      if ("lines".equals(name)) {
-        geometry = new IndexedLineArray(this.geometryVertices.length / 3, format, coordinatesIndices.length);
-      } else { // linestrips
-        int [] stripCounts = new int [this.facesAndLinesPrimitives.size()];
-        for (int i = 0; i < stripCounts.length; i++) {
-          stripCounts [i] = this.facesAndLinesPrimitives.get(i).length / this.inputCount;
+      if (coordinatesIndices.length != 0) {
+        IndexedGeometryArray geometry;
+        if ("lines".equals(name)) {
+          geometry = new IndexedLineArray(this.geometryVertices.length / 3, format, coordinatesIndices.length);
+        } else { // linestrips
+          int [] stripCounts = new int [this.facesAndLinesPrimitives.size()];
+          for (int i = 0; i < stripCounts.length; i++) {
+            stripCounts [i] = this.facesAndLinesPrimitives.get(i).length / this.inputCount;
+          }
+          geometry = new IndexedLineStripArray(this.geometryVertices.length / 3, format, coordinatesIndices.length, stripCounts);
         }
-        geometry = new IndexedLineStripArray(this.geometryVertices.length / 3, format, coordinatesIndices.length, stripCounts);
+        
+        geometry.setCoordinates(0, this.geometryVertices);
+        geometry.setCoordinateIndices(0, coordinatesIndices);
+        if (this.geometryNormals != null) {
+          geometry.setNormals(0, this.geometryNormals);
+          geometry.setNormalIndices(0, getIndices(this.geometryNormalOffset));
+        }
+        if (this.geometryTextureCoordinates != null) {
+          geometry.setTextureCoordinates(0, 0, this.geometryTextureCoordinates);
+          geometry.setTextureCoordinateIndices(0, 0, getIndices(this.geometryTextureCoordinatesOffset));
+        }
+        return geometry;
+      } else {
+        // Ignore lines with an empty index set
+        return null;
       }
-      
-      geometry.setCoordinates(0, this.geometryVertices);
-      geometry.setCoordinateIndices(0, coordinatesIndices);
-      if (this.geometryNormals != null) {
-        geometry.setNormals(0, this.geometryNormals);
-        geometry.setNormalIndices(0, getIndices(this.geometryNormalOffset));
-      }
-      if (this.geometryTextureCoordinates != null) {
-        geometry.setTextureCoordinates(0, 0, this.geometryTextureCoordinates);
-        geometry.setTextureCoordinateIndices(0, 0, getIndices(this.geometryTextureCoordinatesOffset));
-      }
-      return geometry;
     }
 
     /**
