@@ -33,7 +33,6 @@ import java.util.TreeMap;
 import javax.media.j3d.Appearance;
 import javax.media.j3d.Shape3D;
 import javax.media.j3d.Texture;
-import javax.media.j3d.TextureAttributes;
 import javax.media.j3d.TransparencyAttributes;
 import javax.vecmath.Point3f;
 import javax.vecmath.TexCoord2f;
@@ -75,9 +74,7 @@ public class Ground3D extends Object3DBranch {
     Appearance groundAppearance = new Appearance();
     groundAppearance.setCapability(Appearance.ALLOW_MATERIAL_WRITE);
     groundAppearance.setCapability(Appearance.ALLOW_TEXTURE_WRITE);
-    TextureAttributes textureAttributes = new TextureAttributes();
-    textureAttributes.setTextureMode(TextureAttributes.MODULATE);
-    groundAppearance.setTextureAttributes(textureAttributes);
+    groundAppearance.setCapability(Appearance.ALLOW_TEXTURE_ATTRIBUTES_WRITE);
     groundAppearance.setCapability(Appearance.ALLOW_TRANSPARENCY_ATTRIBUTES_READ);
     TransparencyAttributes transparencyAttributes = new TransparencyAttributes();
     transparencyAttributes.setCapability(TransparencyAttributes.ALLOW_MODE_WRITE);
@@ -121,8 +118,9 @@ public class Ground3D extends Object3DBranch {
       groundAppearance.getTransparencyAttributes().setTransparencyMode(TransparencyAttributes.NONE);      
     } else {
       groundAppearance.setMaterial(getMaterial(DEFAULT_COLOR, DEFAULT_COLOR, 0));
-      final TextureManager imageManager = TextureManager.getInstance();
-      imageManager.loadTexture(groundTexture.getImage(), groundTexture.getAngle(), waitTextureLoadingEnd,
+      groundAppearance.setTextureAttributes(getTextureAttributes(groundTexture));
+      final TextureManager textureManager = TextureManager.getInstance();
+      textureManager.loadTexture(groundTexture.getImage(), waitTextureLoadingEnd,
           new TextureManager.TextureObserver() {
               public void textureUpdated(Texture texture) {
                 groundAppearance.setTexture(getHomeTextureClone(texture, home));
@@ -146,13 +144,14 @@ public class Ground3D extends Object3DBranch {
     Map<Level, Area> roomAreas = new TreeMap<Level, Area>(levelComparator);
     for (Room room : home.getRooms()) {
       Level roomLevel = room.getLevel();
-      if (room.isFloorVisible()) {
+      if ((roomLevel == null || roomLevel.isViewable())
+          && room.isFloorVisible()) {
         float [][] roomPoints = room.getPoints();
         if (roomPoints.length > 2) {
           Area roomArea = null;
           if (roomLevel == null
               || (roomLevel.getElevation() <= 0
-                  && roomLevel.isVisible())) {
+                  && roomLevel.isViewableAndVisible())) {
             roomArea = new Area(getShape(roomPoints));
             areaRemovedFromGround.add(roomArea);
             updateUndergroundAreas(roomAreas, room.getLevel(), roomPoints, roomArea);
@@ -164,15 +163,20 @@ public class Ground3D extends Object3DBranch {
     
     // Search all items at negative levels that could dig the ground 
     for (HomePieceOfFurniture piece : home.getFurniture()) {
-      if (piece.getStaircaseCutOutShape() == null) {
-        updateUndergroundAreas(undergroundAreas, piece.getLevel(), piece.getPoints(), null);
-      } else {
-        updateUndergroundAreas(undergroundAreas, piece.getLevel(), null, ModelManager.getInstance().getAreaOnFloor(piece));
+      if (piece.getGroundElevation() < 0
+          && (piece.getLevel() == null || piece.getLevel().isViewable())) {
+        if (piece.getStaircaseCutOutShape() == null) {
+          updateUndergroundAreas(undergroundAreas, piece.getLevel(), piece.getPoints(), null);
+        } else {
+          updateUndergroundAreas(undergroundAreas, piece.getLevel(), null, ModelManager.getInstance().getAreaOnFloor(piece));
+        }
       }
     }
     Map<Level, Area> wallAreas = new HashMap<Level, Area>();
     for (Wall wall : home.getWalls()) {
-      updateUndergroundAreas(wallAreas, wall.getLevel(), wall.getPoints(), null);
+      if (wall.getLevel() == null || wall.getLevel().isViewable()) {
+        updateUndergroundAreas(wallAreas, wall.getLevel(), wall.getPoints(), null);
+      }
     }
     // Consider that walls around a closed area define a hole 
     for (Map.Entry<Level, Area> wallAreaEntry : wallAreas.entrySet()) {
@@ -188,6 +192,7 @@ public class Ground3D extends Object3DBranch {
     for (Map.Entry<Level, Area> undergroundAreaEntry : undergroundAreas.entrySet()) {
       Level level = undergroundAreaEntry.getKey();
       Area area = undergroundAreaEntry.getValue();
+      Area areaAtStart = (Area)area.clone();
       undergroundSideAreas.put(level, (Area)area.clone());
       upperLevelAreas.put(level, new Area());
       // Remove lower levels areas from the area at the current level
@@ -205,13 +210,18 @@ public class Ground3D extends Object3DBranch {
       // Add underground area to ground area at ground level
       for (float [][] points : getAreaPoints(area)) {
         if (new Room(points).isClockwise()) {
-          upperLevelAreas.get(level).add(new Area(getShape(points)));
+          // Hole surrounded by a union of rooms that form a polygon  
+          Area coveredHole = new Area(getShape(points));
+          // Compute the missing hole area in the level area before other sublevels were subtracted from it 
+          coveredHole.exclusiveOr(areaAtStart);
+          coveredHole.subtract(areaAtStart);
+          upperLevelAreas.get(level).add(coveredHole);
         } else {
           areaRemovedFromGround.add(new Area(getShape(points)));
         }
       }
     }
-    // Remove room areas 
+    // Remove room areas because they are displayed by Room3D instances
     for (Map.Entry<Level, Area> undergroundAreaEntry : undergroundAreas.entrySet()) {
       Level level = undergroundAreaEntry.getKey();
       Area area = undergroundAreaEntry.getValue();
@@ -274,7 +284,7 @@ public class Ground3D extends Object3DBranch {
     List<float [][]> areaPoints = new ArrayList<float [][]>();
     List<float []>   areaPartPoints  = new ArrayList<float[]>();
     float [] previousRoomPoint = null;
-    for (PathIterator it = area.getPathIterator(null, 1); !it.isDone(); ) {
+    for (PathIterator it = area.getPathIterator(null, 1); !it.isDone(); it.next()) {
       float [] roomPoint = new float[2];
       if (it.currentSegment(roomPoint) == PathIterator.SEG_CLOSE) {
         if (areaPartPoints.get(0) [0] == previousRoomPoint [0] 
@@ -293,8 +303,7 @@ public class Ground3D extends Object3DBranch {
           areaPartPoints.add(roomPoint);
         }
         previousRoomPoint = roomPoint;
-      }
-      it.next();
+      }      
     }
     return areaPoints;
   }
@@ -342,8 +351,8 @@ public class Ground3D extends Object3DBranch {
       float textureWidth;
       float textureHeight;
       if (groundTexture != null) {
-        textureWidth = TextureManager.getInstance().getRotatedTextureWidth(groundTexture);
-        textureHeight = TextureManager.getInstance().getRotatedTextureHeight(groundTexture);
+        textureWidth = groundTexture.getWidth();
+        textureHeight = groundTexture.getHeight();
       } else {
         textureWidth = 0;
         textureHeight = 0;
@@ -387,8 +396,8 @@ public class Ground3D extends Object3DBranch {
     float textureWidth;
     float textureHeight;
     if (groundTexture != null) {
-      textureWidth = TextureManager.getInstance().getRotatedTextureWidth(groundTexture);
-      textureHeight = TextureManager.getInstance().getRotatedTextureHeight(groundTexture);
+      textureWidth = groundTexture.getWidth();
+      textureHeight = groundTexture.getHeight();
     } else {
       textureWidth = 0;
       textureHeight = 0;
