@@ -85,13 +85,11 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
       new PolygonAttributes(PolygonAttributes.POLYGON_FILL, PolygonAttributes.CULL_NONE, 0, false);
   private static final PolygonAttributes      NORMAL_FLIPPED_TEXTURED_SHAPE_POLYGON_ATTRIBUTES = 
       new PolygonAttributes(PolygonAttributes.POLYGON_FILL, PolygonAttributes.CULL_NONE, 0, true);
-  private static final TextureAttributes      MODULATE_TEXTURE_ATTRIBUTES = new TextureAttributes();
   private static final Bounds                 DEFAULT_INFLUENCING_BOUNDS = new BoundingSphere(new Point3d(), 1E7);
 
   private final Home home;
   
   static {
-    MODULATE_TEXTURE_ATTRIBUTES.setTextureMode(TextureAttributes.MODULATE);
     DEFAULT_TEXTURED_SHAPE_POLYGON_ATTRIBUTES.setCapability(PolygonAttributes.ALLOW_CULL_FACE_READ);
     NORMAL_FLIPPED_TEXTURED_SHAPE_POLYGON_ATTRIBUTES.setCapability(PolygonAttributes.ALLOW_CULL_FACE_READ);
   }
@@ -119,13 +117,7 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
     setCapability(BranchGroup.ALLOW_CHILDREN_READ);
     setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
     
-    if (piece instanceof HomeFurnitureGroup) {
-      for (HomePieceOfFurniture groupPiece : ((HomeFurnitureGroup)piece).getFurniture()) {
-        addChild(new HomePieceOfFurniture3D(groupPiece, home, ignoreDrawingMode, waitModelAndTextureLoadingEnd));
-      }
-    } else {
-      createPieceOfFurnitureNode(piece, ignoreDrawingMode, waitModelAndTextureLoadingEnd);
-    }
+    createPieceOfFurnitureNode(piece, ignoreDrawingMode, waitModelAndTextureLoadingEnd);
   }
 
   /**
@@ -204,21 +196,16 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
         });
   }
 
+  /**
+   * Updates this branch from the home piece it manages.
+   */
   @Override
   public void update() {
-    HomePieceOfFurniture piece = (HomePieceOfFurniture)getUserData();
-    if (piece instanceof HomeFurnitureGroup) {
-      Enumeration<?> enumeration = getAllChildren(); 
-      while (enumeration.hasMoreElements()) {
-        ((HomePieceOfFurniture3D)enumeration.nextElement()).update();
-      }
-    } else {
-      updatePieceOfFurnitureTransform();
-      updatePieceOfFurnitureModelMirrored();
-      updatePieceOfFurnitureColorAndTexture(false);      
-      updateLight();
-      updatePieceOfFurnitureVisibility();      
-    }
+    updatePieceOfFurnitureTransform();
+    updatePieceOfFurnitureModelMirrored();
+    updatePieceOfFurnitureColorAndTexture(false);      
+    updateLight();
+    updatePieceOfFurnitureVisibility();      
   }
 
   /**
@@ -266,7 +253,7 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
       boolean enabled = this.home.getEnvironment().getSubpartSizeUnderLight() > 0 
           && piece.isVisible() 
           && (piece.getLevel() == null
-            || piece.getLevel().isVisible());
+            || piece.getLevel().isViewableAndVisible());
       HomeLight light = (HomeLight)piece;
       LightSource [] lightSources = light.getLightSources();
       if (numChildren() > 2) {
@@ -356,7 +343,7 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
     HomePieceOfFurniture piece = (HomePieceOfFurniture)getUserData();
     Node outlineModelNode = getOutlineModelNode();
     HomeEnvironment.DrawingMode drawingMode;
-    if (outlineModelNode != null) {
+    if (this.home != null && outlineModelNode != null) {
       drawingMode = this.home.getEnvironment().getDrawingMode(); 
     } else {
       drawingMode = null; 
@@ -364,16 +351,21 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
     // Update visibility of filled model shapes
     boolean visible = piece.isVisible() 
         && (piece.getLevel() == null
-            || piece.getLevel().isVisible()); 
+            || piece.getLevel().isViewableAndVisible()); 
+    HomeMaterial [] materials = piece.getColor() == null && piece.getTexture() == null
+        ? piece.getModelMaterials()
+        : null;
     setVisible(getFilledModelNode(), visible
         && (drawingMode == null
             || drawingMode == HomeEnvironment.DrawingMode.FILL 
-            || drawingMode == HomeEnvironment.DrawingMode.FILL_AND_OUTLINE));
+            || drawingMode == HomeEnvironment.DrawingMode.FILL_AND_OUTLINE),
+        materials);
     if (outlineModelNode != null) {
       // Update visibility of outline model shapes
       setVisible(outlineModelNode, visible
           && (drawingMode == HomeEnvironment.DrawingMode.OUTLINE
-              || drawingMode == HomeEnvironment.DrawingMode.FILL_AND_OUTLINE));
+              || drawingMode == HomeEnvironment.DrawingMode.FILL_AND_OUTLINE),
+          materials);
     }
   }
 
@@ -393,16 +385,16 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
                                                TransformGroup normalization,
                                                boolean ignoreDrawingMode,
                                                boolean waitTextureLoadingEnd) {    
-    BranchGroup modelBranch = new BranchGroup();
     normalization.addChild(modelNode);
-    normalization.setCapability(ALLOW_CHILDREN_READ);
+    setModelCapabilities(normalization);
     // Add model node to branch group
+    BranchGroup modelBranch = new BranchGroup();
+    modelBranch.setCapability(BranchGroup.ALLOW_CHILDREN_READ);
     modelBranch.addChild(normalization);
     if (!ignoreDrawingMode) {
-      // Add outline model node 
+      // Add outline model node
       modelBranch.addChild(createOutlineModelNode(normalization));
     }
-    setModelCapabilities(modelBranch);
 
     TransformGroup transformGroup = (TransformGroup)getChild(0);
     // Remove previous nodes    
@@ -591,28 +583,25 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
             || shininess != null;
         boolean windowPane = shapeName != null
             && shapeName.startsWith(ModelManager.WINDOW_PANE_SHAPE_PREFIX);
-        if (!windowPane && appearanceModified            
-            || windowPane && materialModified) {
+        float materialShininess = 0;
+        if (appearanceModified) {
           // Use appearance user data to store shape default material 
-          // (global color or texture change doesn't have effect on window panes)
           defaultMaterialAndTexture = (DefaultMaterialAndTexture)appearance.getUserData();
           if (defaultMaterialAndTexture == null) {
             defaultMaterialAndTexture = new DefaultMaterialAndTexture(appearance);
             appearance.setUserData(defaultMaterialAndTexture);
           }
-        }
-        float materialShininess = 0;
-        if (appearanceModified) {
+          
           materialShininess = shininess != null
               ? shininess.floatValue()
-              : (appearance.getMaterial() != null
-                  ? appearance.getMaterial().getShininess() / 128f
+              : (defaultMaterialAndTexture.getMaterial() != null
+                  ? defaultMaterialAndTexture.getMaterial().getShininess() / 128f
                   : 0);
         }
         if (colorModified) {
           // Change color only of shapes that are not window panes
           if (windowPane) {
-            restoreDefaultMaterialAndTexture(appearance, null);
+            restoreDefaultMaterialAndTexture(appearance, materialShininess);
           } else {
             // Change material if no default texture is displayed on the shape
             // (textures always keep the colors of their image file)
@@ -626,26 +615,30 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
         } else if (textureModified) {            
           // Change texture only of shapes that are not window panes
           if (windowPane) {
-            restoreDefaultMaterialAndTexture(appearance, null);
+            restoreDefaultMaterialAndTexture(appearance, materialShininess);
           } else {
             // Change material to white then texture
-            appearance.setTexCoordGeneration(getTextureCoordinates(appearance, texture, pieceSize, modelBounds));
+            appearance.setTexCoordGeneration(getTextureCoordinates(texture, pieceSize, modelBounds));
             appearance.setMaterial(getMaterial(DEFAULT_COLOR, DEFAULT_AMBIENT_COLOR, materialShininess));
-            appearance.setTextureAttributes(MODULATE_TEXTURE_ATTRIBUTES);
-            TextureManager.getInstance().loadTexture(texture.getImage(), texture.getAngle(), 
+            appearance.setTextureAttributes(getTextureAttributes(texture));
+            TextureManager.getInstance().loadTexture(texture.getImage(),  
                 waitTextureLoadingEnd, getTextureObserver(appearance));
           }
         } else if (materialModified) {
           boolean materialFound = false;
           // Apply color, texture and shininess of the material named as appearance name
           for (HomeMaterial material : materials) {
-            if (material != null 
-                && material.getName().equals(appearance.getName())) {
+            if (material != null
+                && (material.getKey() != null
+                        && material.getKey().equals(appearance.getName())
+                    || material.getKey() == null
+                        && material.getName().equals(appearance.getName()))) {
               if (material.getShininess() != null) {
                 materialShininess = material.getShininess();
               }
               color = material.getColor();                
-              if (color != null) {
+              if (color != null 
+                  && (color.intValue() & 0xFF000000) != 0) {
                 appearance.setMaterial(getMaterial(color, color, materialShininess));
                 appearance.setTexture(null);
                 appearance.setTransparencyAttributes(defaultMaterialAndTexture.getTransparencyAttributes());
@@ -654,12 +647,12 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
                 if (isTexturesCoordinatesDefined(shape)) {
                   restoreDefaultTextureCoordinatesGeneration(appearance);
                 } else {
-                  appearance.setTexCoordGeneration(getTextureCoordinates(appearance, material.getTexture(), pieceSize, modelBounds));
+                  appearance.setTexCoordGeneration(getTextureCoordinates(material.getTexture(), pieceSize, modelBounds));
                 }
                 appearance.setMaterial(getMaterial(DEFAULT_COLOR, DEFAULT_AMBIENT_COLOR, materialShininess));
-                appearance.setTextureAttributes(MODULATE_TEXTURE_ATTRIBUTES);
                 HomeTexture materialTexture = material.getTexture();
-                TextureManager.getInstance().loadTexture(materialTexture.getImage(), materialTexture.getAngle(), 
+                appearance.setTextureAttributes(getTextureAttributes(materialTexture));
+                TextureManager.getInstance().loadTexture(materialTexture.getImage(),  
                     waitTextureLoadingEnd, getTextureObserver(appearance));
               } else {
                 restoreDefaultMaterialAndTexture(appearance, material.getShininess());
@@ -700,6 +693,7 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
             DefaultMaterialAndTexture defaultMaterialAndTexture = (DefaultMaterialAndTexture)appearance.getUserData();
             if (defaultMaterialAndTexture != null) {
               appearance.setTransparencyAttributes(defaultMaterialAndTexture.getTransparencyAttributes());
+              appearance.setPolygonAttributes(defaultMaterialAndTexture.getPolygonAttributes());
             }
           }
           Texture homeTexture = getHomeTextureClone(texture, home);
@@ -713,15 +707,15 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
   /**
    * Returns a texture coordinates generator that wraps the given texture on front face.
    */
-  private TexCoordGeneration getTextureCoordinates(Appearance appearance, HomeTexture texture, 
-                                                   Vector3f pieceSize, BoundingBox modelBounds) {
+  private TexCoordGeneration getTextureCoordinates(HomeTexture texture, Vector3f pieceSize, 
+                                                   BoundingBox modelBounds) {
     Point3d lower = new Point3d();
     modelBounds.getLower(lower);
     Point3d upper = new Point3d();
     modelBounds.getUpper(upper);
     float minimumSize = ModelManager.getInstance().getMinimumSize();
-    float textureWidth = TextureManager.getInstance().getRotatedTextureWidth(texture);
-    float textureHeight = TextureManager.getInstance().getRotatedTextureHeight(texture);
+    float textureWidth = texture.getWidth();
+    float textureHeight = texture.getHeight();
     float sx = pieceSize.x / (float)Math.max(upper.x - lower.x, minimumSize) / textureWidth;
     float sw = texture.isLeftToRightOriented()  
         ? (float)-lower.x * sx  
@@ -784,15 +778,15 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
   /**
    * Sets the visible attribute of the <code>Shape3D</code> children nodes of <code>node</code>.
    */
-  private void setVisible(Node node, boolean visible) {
+  private void setVisible(Node node, boolean visible, HomeMaterial [] materials) {
     if (node instanceof Group) {
       // Set visibility of all children
       Enumeration<?> enumeration = ((Group)node).getAllChildren(); 
       while (enumeration.hasMoreElements()) {
-        setVisible((Node)enumeration.nextElement(), visible);
+        setVisible((Node)enumeration.nextElement(), visible, materials);
       }
     } else if (node instanceof Link) {
-      setVisible(((Link)node).getSharedGroup(), visible);
+      setVisible(((Link)node).getSharedGroup(), visible, materials);
     } else if (node instanceof Shape3D) {
       final Shape3D shape = (Shape3D)node;
       Appearance appearance = shape.getAppearance();
@@ -817,6 +811,20 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
         // Don't display light sources shapes of unselected lights
         visible = false;
       }
+      
+      if (visible
+          && materials != null) {
+        // Check whether the material color used by this shape isn't invisible 
+        for (HomeMaterial material : materials) {
+          if (material != null 
+              && material.getName().equals(appearance.getName())) {
+            Integer color = material.getColor();                
+            visible = color == null
+                || (color.intValue() & 0xFF000000) != 0;
+            break;
+          }
+        }
+      }  
       // Change visibility
       renderingAttributes.setVisible(visible);
     } 
@@ -964,10 +972,10 @@ public class HomePieceOfFurniture3D extends Object3DBranch {
         && geometry instanceof GeometryArray) {
       geometry.setCapability(GeometryArray.ALLOW_FORMAT_READ);
       geometry.setCapability(GeometryArray.ALLOW_COUNT_READ);
+      geometry.setCapability(GeometryArray.ALLOW_REF_DATA_READ);
       geometry.setCapability(GeometryArray.ALLOW_COORDINATE_READ);
       geometry.setCapability(GeometryArray.ALLOW_NORMAL_READ);
       geometry.setCapability(GeometryArray.ALLOW_TEXCOORD_READ);
-      geometry.setCapability(GeometryArray.ALLOW_REF_DATA_READ);
     }
   }
 
