@@ -21,7 +21,6 @@ package com.eteks.sweethome3d.j3d;
 
 import java.awt.image.BufferedImage;
 import java.io.BufferedInputStream;
-import java.io.ByteArrayInputStream;
 import java.io.ByteArrayOutputStream;
 import java.io.EOFException;
 import java.io.File;
@@ -38,6 +37,7 @@ import java.net.URLConnection;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Comparator;
+import java.util.ConcurrentModificationException;
 import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.LinkedHashMap;
@@ -356,8 +356,8 @@ public class Max3DSLoader extends LoaderBase implements Loader {
 
   /**
    * Sets whether this loader should try or avoid accessing to URLs with cache.
-   * @param useCaches <id>Boolean.TRUE</id>, <id>Boolean.FALSE</id>, or 
-   *    <id>null</id> then caches will be used according to the value 
+   * @param useCaches <code>Boolean.TRUE</code>, <code>Boolean.FALSE</code>, or 
+   *    <code>null</code> then caches will be used according to the value 
    *    returned by {@link URLConnection#getDefaultUseCaches()}.
    */
   public void setUseCaches(Boolean useCaches) {
@@ -583,9 +583,9 @@ public class Max3DSLoader extends LoaderBase implements Loader {
             }
             
             // Add vertex index to the list of shared vertices 
-            Mesh3DSSharedVertex sharedVertice = new Mesh3DSSharedVertex(i, normal);
-            sharedVertice.setNextVertex(sharedVertices [vertexIndex]);
-            sharedVertices [vertexIndex] = sharedVertice;
+            Mesh3DSSharedVertex sharedVertex = new Mesh3DSSharedVertex(i, normal);
+            sharedVertex.setNextVertex(sharedVertices [vertexIndex]);
+            sharedVertices [vertexIndex] = sharedVertex;
             defaultNormals [k] = normal;
           }
         }
@@ -773,7 +773,7 @@ public class Max3DSLoader extends LoaderBase implements Loader {
         case MESH_VERSION : 
           in.readLittleEndianInt();
           break;
-        case MASTER_SCALE: 
+        case MASTER_SCALE : 
           this.masterScale = in.readLittleEndianFloat();
           break;
         case NAMED_OBJECT : 
@@ -799,14 +799,14 @@ public class Max3DSLoader extends LoaderBase implements Loader {
         case V_GRADIENT :
         case USE_BIT_MAP :
         case USE_SOLID_BGND :
-        case USE_V_GRADIENT:
+        case USE_V_GRADIENT :
         case FOG :
         case LAYER_FOG :
         case DISTANCE_CUE :
         case USE_FOG :
         case USE_LAYER_FOG :
         case USE_DISTANCE_CUE : 
-        default:
+        default :
           in.readUntilChunkEnd();
           break;
       }
@@ -833,7 +833,7 @@ public class Max3DSLoader extends LoaderBase implements Loader {
         case OBJECT_DOESNT_RCVSHADOW :
         case OBJECT_FAST :
         case OBJECT_FROZEN :
-        default:
+        default :
           in.readUntilChunkEnd();
           break;
       }
@@ -893,7 +893,7 @@ public class Max3DSLoader extends LoaderBase implements Loader {
                 }
                 break;
               case MESH_BOXMAP :
-              default:
+              default :
                 in.readUntilChunkEnd();
                 break;
             }
@@ -987,7 +987,7 @@ public class Max3DSLoader extends LoaderBase implements Loader {
                 // Ignore next frames
                 in.readUntilChunkEnd();
                 break;
-              default:
+              default :
                 in.readUntilChunkEnd();
                 break;
             }
@@ -1027,7 +1027,7 @@ public class Max3DSLoader extends LoaderBase implements Loader {
           }
           currentTransformGroup.setTransform(transform);
           break;
-        default:
+        default :
           in.readUntilChunkEnd();
           break;
       }
@@ -1125,7 +1125,7 @@ public class Max3DSLoader extends LoaderBase implements Loader {
         case MATERIAL_REFBLUR : 
         case MATERIAL_USE_REFBLUR :
         case MATERIAL_SHADING : 
-        case MATERIAL_DECAL:
+        case MATERIAL_DECAL :
         case MATERIAL_ADDITIVE :
         case MATERIAL_FACEMAP :
         case MATERIAL_PHONGSOFT :
@@ -1148,7 +1148,7 @@ public class Max3DSLoader extends LoaderBase implements Loader {
         case MATERIAL_REFLMAP :
         case MATERIAL_REFLMASK :
         case MATERIAL_ACUBIC :
-        default:
+        default :
           in.readUntilChunkEnd();
           break;
       }
@@ -1190,7 +1190,7 @@ public class Max3DSLoader extends LoaderBase implements Loader {
             color = readColor;
           }
           break;
-        default:
+        default :
           in.readUntilChunkEnd();
           break;
       }
@@ -1213,7 +1213,7 @@ public class Max3DSLoader extends LoaderBase implements Loader {
         case PERCENTAGE_INT :
           percentage = in.readLittleEndianShort() / 100.f;
           break;
-        default:
+        default :
           in.readUntilChunkEnd();
           break;
       }
@@ -1238,8 +1238,8 @@ public class Max3DSLoader extends LoaderBase implements Loader {
           break;
         case PERCENTAGE_INT :
         case MATERIAL_MAP_TILING :
-        case MATERIAL_MAP_TEXBLUR: 
-        case MATERIAL_MAP_USCALE:
+        case MATERIAL_MAP_TEXBLUR : 
+        case MATERIAL_MAP_USCALE :
         case MATERIAL_MAP_VSCALE :
         case MATERIAL_MAP_UOFFSET :
         case MATERIAL_MAP_VOFFSET :
@@ -1249,7 +1249,7 @@ public class Max3DSLoader extends LoaderBase implements Loader {
         case MATERIAL_MAP_RCOL :
         case MATERIAL_MAP_GCOL :
         case MATERIAL_MAP_BCOL :
-        default:
+        default :
           in.readUntilChunkEnd();
           break;
       }
@@ -1350,7 +1350,16 @@ public class Max3DSLoader extends LoaderBase implements Loader {
           ? new URL(baseUrl, fileName.replace("%", "%25").replace("#", "%23"))
           : new File(fileName).toURI().toURL();
       imageStream = openStream(textureImageUrl, useCaches);
-      BufferedImage textureImage = ImageIO.read(imageStream);          
+      BufferedImage textureImage;          
+      try {
+        textureImage = ImageIO.read(imageStream);
+      } catch (ConcurrentModificationException ex) {
+        // Try to read the image once more, 
+        // see unfixed Java bug http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6986863
+        imageStream.close();
+        imageStream = openStream(textureImageUrl, useCaches);
+        textureImage = ImageIO.read(imageStream);
+      }
       if (textureImage != null) {
         TextureLoader textureLoader = new TextureLoader(textureImage);
         Texture texture = textureLoader.getTexture();
