@@ -104,6 +104,7 @@ import com.eteks.sweethome3d.model.HomePieceOfFurniture;
 import com.eteks.sweethome3d.model.HomeTexture;
 import com.eteks.sweethome3d.model.Room;
 import com.eteks.sweethome3d.tools.OperatingSystem;
+import com.eteks.sweethome3d.tools.SimpleURLContent;
 import com.eteks.sweethome3d.tools.TemporaryURLContent;
 import com.eteks.sweethome3d.tools.URLContent;
 import com.sun.j3d.loaders.IncorrectFormatException;
@@ -217,7 +218,7 @@ public class ModelManager {
   }
 
   /**
-   * Shutdowns the multithreaded service that load textures. 
+   * Shutdowns the multithreaded service that load models and clears loaded models cache. 
    */
   public void clear() {
     if (this.modelsLoader != null) {
@@ -227,6 +228,7 @@ public class ModelManager {
     synchronized (this.loadedModelNodes) {
       this.loadedModelNodes.clear();
     }
+    this.loadingModelObservers.clear();
   }
   
   /**
@@ -241,6 +243,7 @@ public class ModelManager {
    * This method computes the exact box that contains all the shapes,
    * contrary to <code>node.getBounds()</code> that returns a bounding 
    * sphere for a scene.
+   * @param node     the root of a model 
    */
   public Vector3f getSize(Node node) {
     return getSize(node, new Transform3D());
@@ -251,6 +254,9 @@ public class ModelManager {
    * This method computes the exact box that contains all the shapes,
    * contrary to <code>node.getBounds()</code> that returns a bounding 
    * sphere for a scene.
+   * @param node     the root of a model 
+   * @param transformation the transformation applied to the model  
+   *                 or <code>null</code> if no transformation should be applied to node.
    */
   public Vector3f getSize(Node node, Transform3D transformation) {
     BoundingBox bounds = getBounds(node, transformation);
@@ -268,6 +274,7 @@ public class ModelManager {
    * This method computes the exact box that contains all the shapes,
    * contrary to <code>node.getBounds()</code> that returns a bounding 
    * sphere for a scene.
+   * @param node  the root of a model 
    */
   public BoundingBox getBounds(Node node) {
     return getBounds(node, new Transform3D());
@@ -277,6 +284,9 @@ public class ModelManager {
    * Returns the bounds of the 3D shapes of <code>node</code> with an additional <code>transformation</code>.
    * This method computes the exact box that contains all the shapes, contrary to <code>node.getBounds()</code> 
    * that returns a bounding sphere for a scene.
+   * @param node     the root of a model 
+   * @param transformation the transformation applied to the model  
+   *                 or <code>null</code> if no transformation should be applied to node.
    */
   public BoundingBox getBounds(Node node, Transform3D transformation) {
     BoundingBox objectBounds = new BoundingBox(
@@ -576,16 +586,22 @@ public class ModelManager {
               }
               EventQueue.invokeLater(new Runnable() {
                   public void run() {
-                    for (final ModelObserver observer : loadingModelObservers.remove(content)) {
-                      observer.modelUpdated((BranchGroup)cloneNode(loadedModel));
+                    List<ModelObserver> observers = loadingModelObservers.remove(content);
+                    if (observers != null) {
+                      for (final ModelObserver observer : observers) {
+                        observer.modelUpdated((BranchGroup)cloneNode(loadedModel));
+                      }
                     }
                   }
                 });
             } catch (final IOException ex) {
               EventQueue.invokeLater(new Runnable() {
                   public void run() {
-                    for (final ModelObserver observer : loadingModelObservers.remove(content)) {
-                      observer.modelError(ex);
+                    List<ModelObserver> observers = loadingModelObservers.remove(content);
+                    if (observers != null) {
+                      for (final ModelObserver observer : observers) {
+                        observer.modelError(ex);
+                      }
                     }
                   }
                 });
@@ -931,7 +947,7 @@ public class ModelManager {
   public void checkAppearancesName(Node node) {
     // Search appearances used by node shapes keeping their enumeration order 
     Set<Appearance> appearances = new LinkedHashSet<Appearance>(); 
-    searchMaterials(node, appearances);
+    searchAppearances(node, appearances);
     int i = 0;
     for (Appearance appearance : appearances) {
       try {
@@ -951,7 +967,7 @@ public class ModelManager {
   public HomeMaterial [] getMaterials(Node node) {
     // Search appearances used by node shapes 
     Set<Appearance> appearances = new HashSet<Appearance>(); 
-    searchMaterials(node, appearances);
+    searchAppearances(node, appearances);
     Set<HomeMaterial> materials = new TreeSet<HomeMaterial>(new Comparator<HomeMaterial>() {
         public int compare(HomeMaterial m1, HomeMaterial m2) {
           String name1 = m1.getName();
@@ -976,7 +992,8 @@ public class ModelManager {
       if (material != null) {
         Color3f diffuseColor = new Color3f();
         material.getDiffuseColor(diffuseColor);
-        color = ((int)(diffuseColor.x * 255) << 16) 
+        color = 0xFF000000 
+            | ((int)(diffuseColor.x * 255) << 16) 
             | ((int)(diffuseColor.y * 255) << 8)
             | (int)(diffuseColor.z * 255); 
         shininess = material.getShininess() / 128;          
@@ -986,8 +1003,15 @@ public class ModelManager {
       if (appearanceTexture != null) {
         URL textureImageUrl = (URL)appearanceTexture.getUserData();
         if (textureImageUrl != null) {
-          Content textureImage = new URLContent(textureImageUrl);
-          texture = new HomeTexture(new CatalogTexture(null, textureImage, -1, -1));
+          Content textureImage = new SimpleURLContent(textureImageUrl);
+          // Extract image name
+          String textureImageName = textureImageUrl.getFile();
+          textureImageName = textureImageName.substring(textureImageName.lastIndexOf('/') + 1);
+          int lastPoint = textureImageName.lastIndexOf('.');
+          if (lastPoint != -1) {
+            textureImageName = textureImageName.substring(0, lastPoint);
+          }
+          texture = new HomeTexture(new CatalogTexture(textureImageName, textureImage, -1, -1));
         }
       }
       try {
@@ -1000,15 +1024,15 @@ public class ModelManager {
     return materials.toArray(new HomeMaterial [materials.size()]);
   }
 
-  private void searchMaterials(Node node, Set<Appearance> appearances) {
+  private void searchAppearances(Node node, Set<Appearance> appearances) {
     if (node instanceof Group) {
       // Enumerate children
       Enumeration<?> enumeration = ((Group)node).getAllChildren(); 
       while (enumeration.hasMoreElements()) {
-        searchMaterials((Node)enumeration.nextElement(), appearances);
+        searchAppearances((Node)enumeration.nextElement(), appearances);
       }
     } else if (node instanceof Link) {
-      searchMaterials(((Link)node).getSharedGroup(), appearances);
+      searchAppearances(((Link)node).getSharedGroup(), appearances);
     } else if (node instanceof Shape3D) {
       Appearance appearance = ((Shape3D)node).getAppearance();
       if (appearance != null) {
@@ -1026,7 +1050,7 @@ public class ModelManager {
   Area getFrontArea(String cutOutShape, Node node) {
     Area frontArea;
     if (cutOutShape != null) {
-      frontArea = new Area(parseShape(cutOutShape));
+      frontArea = new Area(getShape(cutOutShape));
       frontArea.transform(AffineTransform.getScaleInstance(1, -1));
       frontArea.transform(AffineTransform.getTranslateInstance(-.5, .5));
     } else {
@@ -1631,7 +1655,7 @@ public class ModelManager {
     if (staircase.getStaircaseCutOutShape() == null) {
       throw new IllegalArgumentException("No cut out shape associated to piece");
     }
-    Shape shape = parseShape(staircase.getStaircaseCutOutShape());
+    Shape shape = getShape(staircase.getStaircaseCutOutShape());
     Area staircaseArea = new Area(shape);
     if (staircase.isModelMirrored()) {
       staircaseArea = getMirroredArea(staircaseArea);
@@ -1679,8 +1703,8 @@ public class ModelManager {
   /**
    * Returns the AWT shape matching the given <a href="http://www.w3.org/TR/SVG/paths.html">SVG path shape</a>.  
    */
-  private Shape parseShape(String svgPathShape) {
-    Shape shape = parsedShapes.get(svgPathShape);
+  public Shape getShape(String svgPathShape) {
+    Shape shape = this.parsedShapes.get(svgPathShape);
     if (shape == null) {
       try {
         shape = SVGPathSupport.parsePathShape(svgPathShape);
@@ -1688,7 +1712,7 @@ public class ModelManager {
         // Fallback to default square shape if batik classes aren't in classpath
         shape = new Rectangle2D.Float(0, 0, 1, 1);
       }
-      parsedShapes.put(svgPathShape, shape);
+      this.parsedShapes.put(svgPathShape, shape);
     }
     return shape;
   }
