@@ -36,6 +36,7 @@ import java.util.Enumeration;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
+import java.util.Locale;
 import java.util.Map;
 import java.util.ResourceBundle;
 import java.util.Set;
@@ -68,6 +69,7 @@ import javax.media.j3d.RenderingAttributes;
 import javax.media.j3d.Shape3D;
 import javax.media.j3d.TexCoordGeneration;
 import javax.media.j3d.Texture;
+import javax.media.j3d.TextureAttributes;
 import javax.media.j3d.Transform3D;
 import javax.media.j3d.TransformGroup;
 import javax.media.j3d.TransparencyAttributes;
@@ -121,6 +123,8 @@ import com.eteks.sweethome3d.viewcontroller.Object3DFactory;
 public class PhotoRenderer {
   public enum Quality {LOW, HIGH}
   
+  private final Home home;
+  private final Object3DFactory object3dFactory;
   private final Quality quality;
   private final Compass compass;
   private final int homeLightColor;
@@ -128,8 +132,9 @@ public class PhotoRenderer {
   private final SunflowAPI sunflow;
   private boolean useSunSky;
   private boolean useSunskyLight;
-  private String sunSkyLightName;
-  private String sunLightName;
+  private String  sunSkyLightName;
+  private String  sunLightName;
+  private final Map<Selectable, String []>         homeItemsNames     = new HashMap<Selectable, String []>();
   private final Map<TransparentTextureKey, String> textureImagesCache = new HashMap<TransparentTextureKey, String>();
   private Thread renderingThread;
 
@@ -146,59 +151,57 @@ public class PhotoRenderer {
    * @throws IOException if texture image files required in the scene couldn't be created. 
    */
   public PhotoRenderer(Home home, Quality quality) throws IOException {
-    this(home, 
-        new Object3DBranchFactory() {
-          public Object createObject3D(Home home, Selectable item, boolean waitForLoading) {
-            if (item instanceof Room) {
-              // Never display ceiling with top camera
-              return new Room3D((Room)item, home, !(home.getCamera() instanceof ObserverCamera), true, waitForLoading);
-            } else {
-              return super.createObject3D(home, item, waitForLoading);
-            }  
-          }
-        }, quality);
+    this(home, new PhotoObject3DFactory(), quality);
   }
   
   /**
    * Creates an instance ready to render the scene matching the given <code>home</code>.
    * @param home the home to render
-   * @param object3DFactory a factory able to create 3D objects from <code>home</code> items.
+   * @param object3dFactory a factory able to create 3D objects from <code>home</code> items.
    *            The {@link Object3DFactory#createObject3D(Home, Selectable, boolean) createObject3D} of 
    *            this factory is expected to return an instance of {@link Node} in current implementation.
    * @throws IOException if texture image files required in the scene couldn't be created. 
    */
   public PhotoRenderer(Home home,
-                       Object3DFactory object3DFactory,
+                       Object3DFactory object3dFactory,
                        Quality quality) throws IOException {
+    this.home = home;
     this.compass = home.getCompass();
     this.quality = quality;
     this.sunflow = new SunflowAPI();
     
     this.useSunskyLight = !(home.getCamera() instanceof ObserverCamera);
-    // SunFlow produce too much white spots when silk shader is used with sun sky light
-    // so use this shader only when observer is used 
-    boolean silk = !this.useSunskyLight && quality == Quality.HIGH;
-    String shininessShader = getRenderingParameterValue("shininessShader");
-    if ("glossy".equals(shininessShader)) {
-      silk = false;
-    } else if ("silk".equals(shininessShader)) {
-      silk = true;
-    }  
+    boolean silk = isSilkShaderUsed(quality);  
+    
+    if (object3dFactory == null) {
+      object3dFactory = new PhotoObject3DFactory();
+    }
+    this.object3dFactory = object3dFactory;
     
-    // Export to SunFlow the Java 3D shapes and appearance of the ground, the walls, the furniture and the rooms
     HomeEnvironment homeEnvironment = home.getEnvironment();
     float subpartSize = homeEnvironment.getSubpartSizeUnderLight();
     // Dividing walls and rooms surface in subparts is useless
-    homeEnvironment.setSubpartSizeUnderLight(0); 
-    for (Wall wall : home.getWalls()) {
-      exportNode((Node)object3DFactory.createObject3D(home, wall, true), true, silk);
+    homeEnvironment.setSubpartSizeUnderLight(0);
+
+    // Export to SunFlow the Java 3D shapes and appearance of the ground, the walls, the furniture and the rooms
+    for (Selectable item : home.getSelectableViewableItems()) {
+      if (item instanceof HomeFurnitureGroup) {
+        for (HomePieceOfFurniture piece : ((HomeFurnitureGroup)item).getAllFurniture()) {
+          if (!(piece instanceof HomeFurnitureGroup)) {
+            Node node = (Node)object3dFactory.createObject3D(home, piece, true);
+            if (node != null) {
+              this.homeItemsNames.put(piece, exportNode(node, false, silk));
+            }
+          }
+        }
+      } else {
+        Node node = (Node)object3dFactory.createObject3D(home, item, true);
+        if (node != null) {
+          String [] itemNames = exportNode(node, item instanceof Wall || item instanceof Room, silk);
+          this.homeItemsNames.put(item, itemNames);
+        }
+      }
     }
-    for (HomePieceOfFurniture piece : home.getFurniture()) {
-      exportNode((Node)object3DFactory.createObject3D(home, piece, true), false, silk);
-    }
-    for (Room room : home.getRooms()) {
-      exportNode((Node)object3DFactory.createObject3D(home, room, true), true, silk);
-    } 
     // Create a 3D ground large enough to join the sky at the horizon  
     Ground3D ground = new Ground3D(home, -1E7f / 2, -1E7f / 2, 1E7f, 1E7f, true);
     Transform3D translation = new Transform3D();
@@ -244,7 +247,7 @@ public class PhotoRenderer {
       for (Room room : home.getRooms()) {
         Level roomLevel = room.getLevel();
         if (room.isCeilingVisible() 
-            && (roomLevel == null || roomLevel.isVisible())) {
+            && (roomLevel == null || roomLevel.isViewableAndVisible())) {
           float xCenter = room.getXCenter();
           float yCenter = room.getYCenter();
           
@@ -258,7 +261,8 @@ public class PhotoRenderer {
           if (roomLevel == null || levels.indexOf(roomLevel) == levels.size() - 1) {
             // Search the height of the wall closest to the point xCenter, yCenter
             for (Wall wall : home.getWalls()) {
-              if (wall.isAtLevel(roomLevel)) {
+              if ((wall.getLevel() == null || wall.getLevel().isViewable())
+                  && wall.isAtLevel(roomLevel)) {
                 float wallElevation = wall.getLevel() == null ? 0 : wall.getLevel().getElevation();
                 Float wallHeightAtStart = wall.getHeight();
                 float [][] points = wall.getPoints();
@@ -302,7 +306,7 @@ public class PhotoRenderer {
       if (light.isVisible()
           && lightPower > 0f
           && (level == null
-              || level.isVisible())) {
+              || level.isViewableAndVisible())) {
         float angle = light.getAngle();
         float cos = (float)Math.cos(angle);
         float sin = (float)Math.sin(angle);
@@ -345,21 +349,55 @@ public class PhotoRenderer {
     }
 
     // Use a spiral computing
-    this.sunflow.parameter("bucket.size", 32);
+    this.sunflow.parameter("bucket.size", 64);
     this.sunflow.parameter("bucket.order", "spiral");
     this.sunflow.options(SunflowAPI.DEFAULT_OPTIONS);
   }
 
   /**
    * Renders home in <code>image</code> at the given <code>camera</code> location and image size.
-   * The rendered objects of the home are the ones given in constructor, meaning any change made in 
-   * home since the instantiation of this renderer won't be updated. 
+   * The rendered objects of the home are the same ones since last call to render or construction. 
    */
   public void render(final BufferedImage image, 
                      Camera camera, 
                      final ImageObserver observer) {
+    try {
+      render(image, camera, null, observer);
+    } catch (IOException ex) {
+      // Exception can't happen, since there's no updated item
+    }
+  }
+  
+  /**
+   * Renders home in <code>image</code> at the given <code>camera</code> location and image size.
+   * The home objects listed in <code>updatedItems</code> will be updated in the renderer, 
+   * allowing animations or modifications of their appearance. 
+   */
+  public void render(final BufferedImage image, 
+                     Camera camera,
+                     List<? extends Selectable> updatedItems, 
+                     final ImageObserver observer) throws IOException {
     this.renderingThread = Thread.currentThread();
 
+    if (updatedItems != null) {
+      boolean silk = isSilkShaderUsed(this.quality);  
+      for (Selectable item : updatedItems) {
+        // Remove from SunFlow updated objects
+        String [] itemNames = this.homeItemsNames.get(item);
+        if (itemNames != null) {
+          for (String name : itemNames) {
+            this.sunflow.remove(name);
+          }
+        }
+        
+        Node node = (Node)this.object3dFactory.createObject3D(home, item, true);
+        if (node != null) {
+          itemNames = exportNode(node, item instanceof Wall || item instanceof Room, silk);
+          this.homeItemsNames.put(item, itemNames);
+        }
+      }
+    }
+    
     if (this.sunSkyLightName != null) {
       this.sunflow.remove(this.sunSkyLightName);
     }
@@ -465,9 +503,9 @@ public class PhotoRenderer {
       Vector3 up = new Vector3(0, 1, 0);              
       transform = Matrix4.lookAt(eye, target, up);
     } else {
-      // Compute matrix directly when the camera points is at top
+      // Compute matrix directly when the camera points to top or bottom
       transform = new Matrix4((float)-Math.cos(yaw), (float)-Math.sin(yaw), 0, camera.getX(), 
-          0, 0, 1, camera.getZ(), 
+          0, 0, (float)Math.signum(Math.sin(pitch)), camera.getZ(), 
           (float)-Math.sin(yaw), (float)Math.cos(yaw), 0, camera.getY());
     }
     this.sunflow.parameter("transform", transform);
@@ -482,10 +520,12 @@ public class PhotoRenderer {
     
     int antiAliasingMin = Integer.parseInt(getRenderingParameterValue("antiAliasing.min"));
     int antiAliasingMax = Integer.parseInt(getRenderingParameterValue("antiAliasing.max"));
-    this.sunflow.parameter("filter", antiAliasingMax > 0  ? "blackman-harris"  : "box");
+    String filter = getRenderingParameterValue("filter");
+    this.sunflow.parameter("filter", filter);
     this.sunflow.parameter("aa.min", antiAliasingMin);
     this.sunflow.parameter("aa.max", antiAliasingMax); 
-    this.sunflow.parameter("sampler", "bucket"); // ipr, fast or bucket 
+    String samplerAlgorithm = getRenderingParameterValue("samplerAlgorithm");
+    this.sunflow.parameter("sampler", samplerAlgorithm); // ipr, fast or bucket 
 
     // Render image with default camera
     this.sunflow.parameter("camera", CAMERA_NAME);
@@ -523,7 +563,7 @@ public class PhotoRenderer {
   private String getRenderingParameterValue(String parameterName) {
     // Try to retrieve overridden parameter value from System property
     // (for example: System property com.eteks.sweethome3d.j3d.PhotoRenderer.lowQuality.antiAliasing.min)
-    String prefixedParameter = this.quality.name().toLowerCase() + "Quality." + parameterName;
+    String prefixedParameter = this.quality.name().toLowerCase(Locale.ENGLISH) + "Quality." + parameterName;
     String baseName = PhotoRenderer.class.getName();
     String value = System.getProperty(baseName + '.' + prefixedParameter);
     if (value != null) {
@@ -565,10 +605,29 @@ public class PhotoRenderer {
   }
 
   /**
-   * Exports the given Java 3D <code>node</code> and its children with SunFlow API.  
+   * Returns <code>true</code> if silk shader should be used.
    */
-  private void exportNode(Node node, boolean ignoreTransparency, boolean silk) throws IOException {
-    exportNode(node, ignoreTransparency, silk, new Transform3D());
+  private boolean isSilkShaderUsed(Quality quality) {
+    // SunFlow produce too much white spots when silk shader is used with sun sky light
+    // so use this shader only when observer is used 
+    boolean silk = !this.useSunskyLight && quality == Quality.HIGH;
+    String shininessShader = getRenderingParameterValue("shininessShader");
+    if ("glossy".equals(shininessShader)) {
+      silk = false;
+    } else if ("silk".equals(shininessShader)) {
+      silk = true;
+    }
+    return silk;
+  }
+
+  /**
+   * Exports the given Java 3D <code>node</code> and its children with SunFlow API,
+   * then returns the SunFlow names that match this node.   
+   */
+  private String [] exportNode(Node node, boolean ignoreTransparency, boolean silk) throws IOException {
+    List<String> nodeNames = new ArrayList<String>();
+    exportNode(node, ignoreTransparency, silk, nodeNames, new Transform3D());
+    return nodeNames.toArray(new String [nodeNames.size()]);
   }
 
   /**
@@ -577,6 +636,7 @@ public class PhotoRenderer {
   private void exportNode(Node node, 
                           boolean ignoreTransparency,
                           boolean silk,
+                          List<String> nodeNames,
                           Transform3D parentTransformations) throws IOException {
     if (node instanceof Group) {
       if (node instanceof TransformGroup) {
@@ -588,10 +648,10 @@ public class PhotoRenderer {
       // Export all children
       Enumeration<?> enumeration = ((Group)node).getAllChildren(); 
       while (enumeration.hasMoreElements()) {
-        exportNode((Node)enumeration.nextElement(), ignoreTransparency, silk, parentTransformations);
+        exportNode((Node)enumeration.nextElement(), ignoreTransparency, silk, nodeNames, parentTransformations);
       }
     } else if (node instanceof Link) {
-      exportNode(((Link)node).getSharedGroup(), ignoreTransparency, silk, parentTransformations);
+      exportNode(((Link)node).getSharedGroup(), ignoreTransparency, silk, nodeNames, parentTransformations);
     } else if (node instanceof Shape3D) {
       Shape3D shape = (Shape3D)node;
       Appearance appearance = shape.getAppearance();
@@ -610,6 +670,7 @@ public class PhotoRenderer {
   
         String appearanceName = null;
         TexCoordGeneration texCoordGeneration = null;
+        Transform3D textureTransform = new Transform3D();
         int cullFace = PolygonAttributes.CULL_BACK;
         boolean backFaceNormalFlip = false;
         if (appearance != null) {
@@ -619,10 +680,15 @@ public class PhotoRenderer {
             backFaceNormalFlip = polygonAttributes.getBackFaceNormalFlip();
           }
           texCoordGeneration = appearance.getTexCoordGeneration();
+          TextureAttributes textureAttributes = appearance.getTextureAttributes();
+          if (textureAttributes != null) {
+            textureAttributes.getTextureTransform(textureTransform);
+          }
           appearanceName = "shader" + uuid;
           boolean mirror = shapeName != null
               && shapeName.startsWith(ModelManager.MIRROR_SHAPE_PREFIX);
           exportAppearance(appearance, appearanceName, mirror, ignoreTransparency, silk);
+          nodeNames.add(appearanceName);
         }
 
         // Export object geometries
@@ -630,13 +696,16 @@ public class PhotoRenderer {
           String objectNameBase = "object" + uuid + "-" + i;
           // Always ignore normals on walls
           String [] objectsName = exportNodeGeometry(shape.getGeometry(i), parentTransformations, texCoordGeneration, 
-              cullFace, backFaceNormalFlip, objectNameBase);
+              textureTransform, cullFace, backFaceNormalFlip, objectNameBase);
           if (objectsName != null) {
             for (String objectName : objectsName) {
               if (appearanceName != null) {
                 this.sunflow.parameter("shaders", new String [] {appearanceName});
               }
-              this.sunflow.instance(objectName + ".instance", objectName);
+              String instanceName = objectName + ".instance";
+              this.sunflow.instance(instanceName, objectName);
+              nodeNames.add(instanceName);
+              nodeNames.add(objectName);
             }
           }
         }
@@ -650,6 +719,7 @@ public class PhotoRenderer {
   private String [] exportNodeGeometry(Geometry geometry, 
                                        Transform3D parentTransformations, 
                                        TexCoordGeneration texCoordGeneration, 
+                                       Transform3D textureTransform, 
                                        int cullFace, 
                                        boolean backFaceNormalFlip,
                                        String objectNameBase) {
@@ -757,14 +827,14 @@ public class PhotoRenderer {
                       index < n; index++, i += vertexSize) {
                   TexCoord2f textureCoordinates = generateTextureCoordinates(
                       vertexData [i], vertexData [i + 1], vertexData [i + 2], planeS, planeT);
-                  exportTextureCoordinates(textureCoordinates, index, uvs);
+                  exportTextureCoordinates(textureCoordinates, textureTransform, index, uvs);
                 }
               }
             } else if (uvs != null) {
               for (int index = 0, i = 0, n = geometryArray.getVertexCount(); 
                     index < n; index++, i += vertexSize) {
                 TexCoord2f textureCoordinates = new TexCoord2f(vertexData [i], vertexData [i + 1]);
-                exportTextureCoordinates(textureCoordinates, index, uvs);
+                exportTextureCoordinates(textureCoordinates, textureTransform, index, uvs);
               }
             }
           } else {
@@ -788,14 +858,14 @@ public class PhotoRenderer {
                 for (int index = 0, i = 0, n = geometryArray.getVertexCount(); index < n; index++, i += 3) {
                   TexCoord2f textureCoordinates = generateTextureCoordinates(
                       vertexCoordinates [i], vertexCoordinates [i + 1], vertexCoordinates [i + 2], planeS, planeT);
-                  exportTextureCoordinates(textureCoordinates, index, uvs);
+                  exportTextureCoordinates(textureCoordinates, textureTransform, index, uvs);
                 }
               }
             } else if (uvs != null) {
               float [] textureCoordinatesArray = geometryArray.getTexCoordRefFloat(0);
               for (int index = 0, i = 0, n = geometryArray.getVertexCount(); index < n; index++, i += 2) {
                 TexCoord2f textureCoordinates = new TexCoord2f(textureCoordinatesArray [i], textureCoordinatesArray [i + 1]);
-                exportTextureCoordinates(textureCoordinates, index, uvs);
+                exportTextureCoordinates(textureCoordinates, textureTransform, index, uvs);
               }
             }
           }
@@ -822,14 +892,14 @@ public class PhotoRenderer {
                 geometryArray.getCoordinate(index, vertex);
                 TexCoord2f textureCoordinates = generateTextureCoordinates(
                     vertex.x, vertex.y, vertex.z, planeS, planeT);
-                exportTextureCoordinates(textureCoordinates, index, uvs);
+                exportTextureCoordinates(textureCoordinates, textureTransform, index, uvs);
               }
             }
           } else if (uvs != null) {
             for (int index = 0, n = geometryArray.getVertexCount(); index < n; index++) {
               TexCoord2f textureCoordinates = new TexCoord2f();
               geometryArray.getTextureCoordinate(0, index, textureCoordinates);
-              exportTextureCoordinates(textureCoordinates, index, uvs);
+              exportTextureCoordinates(textureCoordinates, textureTransform, index, uvs);
             }
           }
         }
@@ -1127,13 +1197,21 @@ public class PhotoRenderer {
   }
 
   /**
-   * Stores <code>textureCoordinates</code> in <code>uvs</code>.  
+   * Stores <code>textureCoordinates</code> in <code>uvs</code>. 
    */
-  private void exportTextureCoordinates(TexCoord2f textureCoordinates, int index,
-                                        float [] uvs) {
+  private void exportTextureCoordinates(TexCoord2f textureCoordinates, 
+                                        Transform3D textureTransform,
+                                        int index, float [] uvs) {
     index *= 2;
-    uvs [index++] = textureCoordinates.x;
-    uvs [index] = textureCoordinates.y;
+    if (textureTransform.getBestType() != Transform3D.IDENTITY) {
+      Point3f transformedCoordinates = new Point3f(textureCoordinates.x, textureCoordinates.y, 0);
+      textureTransform.transform(transformedCoordinates);
+      uvs [index++] = transformedCoordinates.x;
+      uvs [index] = transformedCoordinates.y;
+    } else {
+      uvs [index++] = textureCoordinates.x;
+      uvs [index] = textureCoordinates.y;
+    }
   }
 
   /**
@@ -1366,6 +1444,20 @@ public class PhotoRenderer {
     }
   }
 
+  /**
+   * Default factory for photo creation with no ceiling for rooms when top camera is used.
+   */
+  private static class PhotoObject3DFactory extends Object3DBranchFactory {
+    public Object createObject3D(Home home, Selectable item, boolean waitForLoading) {
+      if (item instanceof Room) {
+        // Never display ceiling with top camera
+        return new Room3D((Room)item, home, !(home.getCamera() instanceof ObserverCamera), waitForLoading);
+      } else {
+        return super.createObject3D(home, item, waitForLoading);
+      }  
+    }
+  }
+
   /**
    * A SunFlow display that updates an existing image.
    * Implementation mostly copied from org.sunflow.system.ImagePanel.
