@@ -25,6 +25,7 @@ import java.awt.geom.Rectangle2D;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collections;
+import java.util.Comparator;
 import java.util.List;
 
 import javax.media.j3d.Appearance;
@@ -34,7 +35,6 @@ import javax.media.j3d.Node;
 import javax.media.j3d.RenderingAttributes;
 import javax.media.j3d.Shape3D;
 import javax.media.j3d.Texture;
-import javax.media.j3d.TextureAttributes;
 import javax.media.j3d.TransparencyAttributes;
 import javax.vecmath.Point3f;
 import javax.vecmath.TexCoord2f;
@@ -53,12 +53,6 @@ import com.sun.j3d.utils.geometry.NormalGenerator;
  * Root of room branch.
  */
 public class Room3D extends Object3DBranch {
-  private static final TextureAttributes MODULATE_TEXTURE_ATTRIBUTES = new TextureAttributes();
-  
-  static {
-    MODULATE_TEXTURE_ATTRIBUTES.setTextureMode(TextureAttributes.MODULATE);
-  }
-  
   private static final int FLOOR_PART  = 0;
   private static final int CEILING_PART = 1;
   
@@ -68,15 +62,14 @@ public class Room3D extends Object3DBranch {
    * Creates the 3D room matching the given home <code>room</code>.
    */
   public Room3D(Room room, Home home) {
-    this(room, home, false, false, false);
+    this(room, home, false, false);
   }
 
   /**
    * Creates the 3D room matching the given home <code>room</code>.
    */
   public Room3D(Room room, Home home,
-                boolean ignoreCeilingPart, 
-                boolean ignoreInvisiblePart,
+                boolean ignoreCeilingPart,
                 boolean waitTextureLoadingEnd) {
     setUserData(room);
     this.home = home;
@@ -93,15 +86,9 @@ public class Room3D extends Object3DBranch {
     updateRoomGeometry();
     updateRoomAppearance(waitTextureLoadingEnd);
     
-    if (ignoreCeilingPart
-        || (ignoreInvisiblePart
-            && !room.isCeilingVisible())) {
+    if (ignoreCeilingPart) {
       removeChild(CEILING_PART);
     }
-    if (ignoreInvisiblePart
-        && !room.isFloorVisible()) {
-      removeChild(FLOOR_PART);
-    }
   }
 
   /**
@@ -130,8 +117,7 @@ public class Room3D extends Object3DBranch {
     roomAppearance.setMaterial(DEFAULT_MATERIAL);      
     roomAppearance.setCapability(Appearance.ALLOW_TEXTURE_WRITE);
     roomAppearance.setCapability(Appearance.ALLOW_TEXTURE_READ);
-    // Mix texture and room color
-    roomAppearance.setTextureAttributes(MODULATE_TEXTURE_ATTRIBUTES);
+    roomAppearance.setCapability(Appearance.ALLOW_TEXTURE_ATTRIBUTES_WRITE);
     
     return roomShape;
   }
@@ -154,7 +140,7 @@ public class Room3D extends Object3DBranch {
     Shape3D roomShape = (Shape3D)getChild(roomPart);
     int currentGeometriesCount = roomShape.numGeometries();
     Room room = (Room)getUserData();
-    if (room.getLevel() == null || room.getLevel().isVisible()) {
+    if (room.getLevel() == null || room.getLevel().isViewableAndVisible()) {
       for (Geometry roomGeometry : createRoomGeometries(roomPart, texture)) {
         roomShape.addGeometry(roomGeometry);
       }
@@ -198,33 +184,48 @@ public class Room3D extends Object3DBranch {
 
       // Find rooms at the same elevation 
       // and room ceilings at same elevation as the floor bottom  
-      List<Room> roomsAtSameElevation = new ArrayList<Room>();
+      final List<Room> roomsAtSameElevation = new ArrayList<Room>();
       List<Room> ceilingsAtSameFloorBottomElevation = new ArrayList<Room>();
       for (Room homeRoom : this.home.getRooms()) {
         Level homeRoomLevel = homeRoom.getLevel();
-        if (room == homeRoom // Store also the room itself to know its order among rooms at same elevation
-            || roomLevel == homeRoomLevel
-                && (roomPart == FLOOR_PART && homeRoom.isFloorVisible()
-                    || roomPart == CEILING_PART && homeRoom.isCeilingVisible()) 
-            || roomLevel != null 
-                && homeRoomLevel != null 
-                && (roomPart == FLOOR_PART 
-                        && homeRoom.isFloorVisible()
-                        && Math.abs(roomElevation - homeRoomLevel.getElevation()) < 1E-4
-                    || roomPart == CEILING_PART 
-                        && homeRoom.isCeilingVisible()
-                        && !lastLevel
-                        && !isLastLevel(homeRoomLevel, levels)
-                        && Math.abs(roomElevation + roomLevel.getHeight() - (homeRoomLevel.getElevation() + homeRoomLevel.getHeight())) < 1E-4)) {         
-          roomsAtSameElevation.add(homeRoom);
-        } else if (floorBottomVisible 
-                    && homeRoomLevel != null 
-                    && homeRoom.isCeilingVisible() 
-                    && !isLastLevel(homeRoomLevel, levels) 
-                    && Math.abs(floorBottomElevation - (homeRoomLevel.getElevation() + homeRoomLevel.getHeight())) < 1E-4) {
-          ceilingsAtSameFloorBottomElevation.add(homeRoom);
+        if (homeRoomLevel == null || homeRoomLevel.isViewableAndVisible()) {
+          if (room == homeRoom // Store also the room itself to know its order among rooms at same elevation
+              || roomLevel == homeRoomLevel
+                  && (roomPart == FLOOR_PART && homeRoom.isFloorVisible()
+                      || roomPart == CEILING_PART && homeRoom.isCeilingVisible()) 
+              || roomLevel != null 
+                  && homeRoomLevel != null 
+                  && (roomPart == FLOOR_PART 
+                          && homeRoom.isFloorVisible()
+                          && Math.abs(roomElevation - homeRoomLevel.getElevation()) < 1E-4
+                      || roomPart == CEILING_PART 
+                          && homeRoom.isCeilingVisible()
+                          && !lastLevel
+                          && !isLastLevel(homeRoomLevel, levels)
+                          && Math.abs(roomElevation + roomLevel.getHeight() - (homeRoomLevel.getElevation() + homeRoomLevel.getHeight())) < 1E-4)) {         
+            roomsAtSameElevation.add(homeRoom);
+          } else if (floorBottomVisible 
+                      && homeRoomLevel != null 
+                      && homeRoom.isCeilingVisible() 
+                      && !isLastLevel(homeRoomLevel, levels) 
+                      && Math.abs(floorBottomElevation - (homeRoomLevel.getElevation() + homeRoomLevel.getHeight())) < 1E-4) {
+            ceilingsAtSameFloorBottomElevation.add(homeRoom);
+          }
         }
       }
+      if (roomLevel != null) {
+        // Update order to ensure that rooms are sorted first in level order
+        Collections.sort(roomsAtSameElevation, new Comparator<Room>() {
+            public int compare(Room room1, Room room2) {
+              int comparison = Float.compare(room1.getLevel().getElevation(), room2.getLevel().getElevation());
+              if (comparison != 0) {
+                return comparison;
+              } else {
+                return room1.getLevel().getElevationIndex() - room2.getLevel().getElevationIndex();
+              }
+            }
+          });
+      }
       
       List<HomePieceOfFurniture> visibleStaircases;
       if (roomLevel == null
@@ -444,8 +445,8 @@ public class Room3D extends Object3DBranch {
     geometryInfo.setStripCounts(stripCounts);
     
     if (texture != null) {
-      float textureWidth = TextureManager.getInstance().getRotatedTextureWidth(texture);
-      float textureHeight = TextureManager.getInstance().getRotatedTextureHeight(texture);
+      float textureWidth = texture.getWidth();
+      float textureHeight = texture.getHeight();
       TexCoord2f [] textureCoords = new TexCoord2f [vertexCount];
       i = 0;
       // Compute room texture coordinates
@@ -514,32 +515,36 @@ public class Room3D extends Object3DBranch {
     geometryInfo.setCoordinates(coords);
     
     if (texture != null) {
-      float textureWidth = TextureManager.getInstance().getRotatedTextureWidth(texture);
-      float textureHeight = TextureManager.getInstance().getRotatedTextureHeight(texture);
+      float textureWidth = texture.getWidth();
+      float textureHeight = texture.getHeight();
       TexCoord2f [] textureCoords = new TexCoord2f [vertexCount];
       i = 0;
       // Compute room border texture coordinates
       for (float [][] geometryPoints : geometryRooms) {
         for (int j = 0; j < geometryPoints.length; j++) {
-          textureCoords [i++] = new TexCoord2f(geometryPoints [j][0] / textureWidth, -geometryPoints [j][1] / textureHeight);
-          textureCoords [i++] = new TexCoord2f(geometryPoints [j][0] / textureWidth, -(geometryPoints [j][1] - roomLevel.getFloorThickness()) / textureHeight);
+          textureCoords [i++] = new TexCoord2f(0, roomLevel.getFloorThickness() / textureHeight);
+          textureCoords [i++] = new TexCoord2f(0, 0);
           int nextPoint = j < geometryPoints.length - 1  
               ? j + 1
               : 0;
-          textureCoords [i++] = new TexCoord2f(geometryPoints [nextPoint][0] / textureWidth, -(geometryPoints [nextPoint][1] - roomLevel.getFloorThickness()) / textureHeight);
-          textureCoords [i++] = new TexCoord2f(geometryPoints [nextPoint][0] / textureWidth, -geometryPoints [nextPoint][1] / textureHeight);
+          float textureCoord = (float)(Point2D.distance(geometryPoints [j][0], geometryPoints [j][1], 
+              geometryPoints [nextPoint][0], geometryPoints [nextPoint][1]) / textureWidth);
+          textureCoords [i++] = new TexCoord2f(textureCoord, 0);
+          textureCoords [i++] = new TexCoord2f(textureCoord, roomLevel.getFloorThickness() / textureHeight);
         }
       }
       // Compute holes borders texture coordinates
       for (float [][] geometryHole : geometryHoles) {
         for (int j = 0; j < geometryHole.length; j++) {
-          textureCoords [i++] = new TexCoord2f(geometryHole [j][0] / textureWidth, -geometryHole [j][1] / textureHeight);
+          textureCoords [i++] = new TexCoord2f(0, 0);
           int nextPoint = j < geometryHole.length - 1  
               ? j + 1
               : 0;
-          textureCoords [i++] = new TexCoord2f(geometryHole [nextPoint][0] / textureWidth, -geometryHole [nextPoint][1] / textureHeight);
-          textureCoords [i++] = new TexCoord2f(geometryHole [nextPoint][0] / textureWidth, -(geometryHole [nextPoint][1] - roomLevel.getFloorThickness()) / textureHeight);
-          textureCoords [i++] = new TexCoord2f(geometryHole [j][0] / textureWidth, -(geometryHole [j][1] - roomLevel.getFloorThickness()) / textureHeight);
+          float textureCoord = (float)(Point2D.distance(geometryHole [j][0], geometryHole [j][1], 
+              geometryHole [nextPoint][0], geometryHole [nextPoint][1]) / textureWidth);
+          textureCoords [i++] = new TexCoord2f(textureCoord, 0);
+          textureCoords [i++] = new TexCoord2f(textureCoord, roomLevel.getFloorThickness() / textureHeight);
+          textureCoords [i++] = new TexCoord2f(0, roomLevel.getFloorThickness() / textureHeight);
         }
       }
       geometryInfo.setTextureCoordinateParams(1, 2);
@@ -567,7 +572,9 @@ public class Room3D extends Object3DBranch {
                                                           boolean firstLevel) {
     List<HomePieceOfFurniture> visibleStaircases = new ArrayList<HomePieceOfFurniture>(furniture.size());
     for (HomePieceOfFurniture piece : furniture) {
-      if (piece.isVisible()) {
+      if (piece.isVisible()
+          && (piece.getLevel() == null
+              || piece.getLevel().isViewableAndVisible())) {
         if (piece instanceof HomeFurnitureGroup) {
           visibleStaircases.addAll(getVisibleStaircases(((HomeFurnitureGroup)piece).getFurniture(), roomPart, roomLevel, firstLevel));
         } else if (piece.getStaircaseCutOutShape() != null
@@ -614,7 +621,8 @@ public class Room3D extends Object3DBranch {
       float [][] closestWallPoints = null;
       int closestIndex = -1;
       for (Wall wall : this.home.getWalls()) {
-        if (wall.isAtLevel(roomLevel)) {
+        if ((wall.getLevel() == null || wall.getLevel().isViewable())
+            && wall.isAtLevel(roomLevel)) {
           float [][] points = wall.getPoints();
           for (int i = 0; i < points.length; i++) {
             double distanceToWallPoint = Point2D.distanceSq(points [i][0], points [i][1], x, y);
@@ -639,6 +647,7 @@ public class Room3D extends Object3DBranch {
           if (closestWall.isTrapezoidal()) {
             Float arcExtent = closestWall.getArcExtent();
             if (arcExtent == null
+                || arcExtent.floatValue() == 0
                 || closestIndex == closestWallPoints.length / 2 
                 || closestIndex == closestWallPoints.length / 2 - 1) {
               roomHeight += closestWall.getHeightAtEnd();
@@ -706,8 +715,9 @@ public class Room3D extends Object3DBranch {
     } else {
       // Update material and texture of room part
       roomPartAppearance.setMaterial(getMaterial(DEFAULT_COLOR, DEFAULT_AMBIENT_COLOR, shininess));
+      roomPartAppearance.setTextureAttributes(getTextureAttributes(roomPartTexture));
       final TextureManager textureManager = TextureManager.getInstance();
-      textureManager.loadTexture(roomPartTexture.getImage(), roomPartTexture.getAngle(), waitTextureLoadingEnd,
+      textureManager.loadTexture(roomPartTexture.getImage(), waitTextureLoadingEnd,
           new TextureManager.TextureObserver() {
               public void textureUpdated(Texture texture) {
                 texture = getHomeTextureClone(texture, home);
