@@ -32,6 +32,7 @@ import java.io.InputStream;
 import java.lang.ref.WeakReference;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.ConcurrentModificationException;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
@@ -98,6 +99,7 @@ public class TextureManager {
       this.contentTextures.clear();
       this.textures.clear();
     }
+    this.loadingTextureObservers.clear();
   }
   
   /**
@@ -209,8 +211,11 @@ public class TextureManager {
                 EventQueue.invokeLater(new Runnable() {
                     public void run() {
                       // Notify loaded texture to observer
-                      for (TextureObserver observer : loadingTextureObservers.remove(contentKey)) {
-                        observer.textureUpdated(texture);
+                      List<TextureObserver> observers = loadingTextureObservers.remove(contentKey);
+                      if (observers != null) {
+                        for (TextureObserver observer : observers) {
+                          observer.textureUpdated(texture);
+                        }
                       }
                     }
                   });
@@ -239,12 +244,22 @@ public class TextureManager {
     try {
       // Read the image 
       InputStream contentStream = content.openStream();
-      BufferedImage image = ImageIO.read(contentStream);
+      BufferedImage image;          
+      try {
+        image = ImageIO.read(contentStream);
+      } catch (ConcurrentModificationException ex) {
+        // Try to read the image once more, 
+        // see unfixed Java bug http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6986863
+        contentStream.close();
+        contentStream = content.openStream();
+       image = ImageIO.read(contentStream);
+      }
       if (angle != 0) {
         double cos = Math.cos(angle);
         double sin = Math.sin(angle);
         BufferedImage rotatedImage = new BufferedImage((int)Math.round(Math.abs(image.getWidth() * cos) + Math.abs(image.getHeight() * sin)), 
-            (int)Math.round(Math.abs(image.getWidth() * sin) + Math.abs(image.getHeight() * cos)), BufferedImage.TYPE_INT_ARGB);
+            (int)Math.round(Math.abs(image.getWidth() * sin) + Math.abs(image.getHeight() * cos)), 
+            image.getTransparency() == BufferedImage.TRANSLUCENT ? BufferedImage.TYPE_INT_ARGB : BufferedImage.TYPE_INT_RGB);
         Graphics2D g2D = (Graphics2D)rotatedImage.getGraphics();
         g2D.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
         g2D.setPaint(new TexturePaint(image, 
