@@ -29,7 +29,6 @@ import java.beans.PropertyChangeListener;
 import java.beans.PropertyChangeSupport;
 import java.io.IOException;
 import java.io.ObjectInputStream;
-import java.io.Serializable;
 import java.math.BigDecimal;
 import java.math.RoundingMode;
 import java.text.Collator;
@@ -42,7 +41,7 @@ import java.util.Map;
  * A piece of furniture in {@linkplain Home home}.
  * @author Emmanuel Puybaret
  */
-public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Selectable, Elevatable {
+public class HomePieceOfFurniture extends HomeObject implements PieceOfFurniture, Selectable, Elevatable {
   private static final long serialVersionUID = 1L;
   
   private static final double TWICE_PI = 2 * Math.PI;
@@ -69,7 +68,9 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
     SORTABLE_PROPERTY_COMPARATORS = new HashMap<SortableProperty, Comparator<HomePieceOfFurniture>>();
     SORTABLE_PROPERTY_COMPARATORS.put(SortableProperty.CATALOG_ID, new Comparator<HomePieceOfFurniture>() {
         public int compare(HomePieceOfFurniture piece1, HomePieceOfFurniture piece2) {
-          if (piece1.catalogId == null) {
+          if (piece1.catalogId == piece2.catalogId) {
+            return 0;
+          } else if (piece1.catalogId == null) {
             return -1;
           } else if (piece2.catalogId == null) {
             return 1; 
@@ -80,7 +81,9 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
       });
     SORTABLE_PROPERTY_COMPARATORS.put(SortableProperty.NAME, new Comparator<HomePieceOfFurniture>() {
         public int compare(HomePieceOfFurniture piece1, HomePieceOfFurniture piece2) {
-          if (piece1.name == null) {
+          if (piece1.name == piece2.name) {
+            return 0;
+          } else if (piece1.name == null) {
             return -1;
           } else if (piece2.name == null) {
             return 1; 
@@ -116,7 +119,9 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
       });
     SORTABLE_PROPERTY_COMPARATORS.put(SortableProperty.COLOR, new Comparator<HomePieceOfFurniture>() {
         public int compare(HomePieceOfFurniture piece1, HomePieceOfFurniture piece2) {
-          if (piece1.color == null) {
+          if (piece1.color == piece2.color) {
+            return 0; 
+          } else if (piece1.color == null) {
             return -1;
           } else if (piece2.color == null) {
             return 1; 
@@ -127,7 +132,9 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
       });
     SORTABLE_PROPERTY_COMPARATORS.put(SortableProperty.TEXTURE, new Comparator<HomePieceOfFurniture>() {
         public int compare(HomePieceOfFurniture piece1, HomePieceOfFurniture piece2) {
-          if (piece1.texture == null) {
+          if (piece1.texture == piece2.texture) {
+            return 0; 
+          } else if (piece1.texture == null) {
             return -1;
           } else if (piece2.texture == null) {
             return 1; 
@@ -199,7 +206,9 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
   }
   
   private static int compare(BigDecimal value1, BigDecimal value2) {
-    if (value1 == null) {
+    if (value1 == value2) {
+      return 0;
+    } else if (value1 == null) {
       return -1;
     } else if (value2 == null) {
       return 1; 
@@ -209,7 +218,9 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
   }
   
   private static int compare(Level level1, Level level2) {
-    if (level1 == null) {
+    if (level1 == level2) {
+      return 0;
+    } else if (level1 == null) {
       return -1;
     } else if (level2 == null) {
       return 1; 
@@ -257,7 +268,7 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
   private float                  angle;
   private boolean                modelMirrored;
   private Level                  level;
-
+  
   private transient PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);
   private transient Shape shapeCache;
 
@@ -297,6 +308,7 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
       this.nameVisible = homePiece.isNameVisible();
       this.nameXOffset = homePiece.getNameXOffset();
       this.nameYOffset = homePiece.getNameYOffset();
+      this.nameAngle = homePiece.getNameAngle();
       this.nameStyle = homePiece.getNameStyle();
       this.visible = homePiece.isVisible();
       this.angle = homePiece.getAngle();
@@ -304,6 +316,7 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
       this.y = homePiece.getY();
       this.modelMirrored = homePiece.isModelMirrored();
       this.texture = homePiece.getTexture();
+      this.shininess = homePiece.getShininess();
       this.modelMaterials = homePiece.getModelMaterials();
     } else {
       if (piece instanceof CatalogPieceOfFurniture) {
@@ -327,6 +340,9 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
     this.texturable = true;
     this.propertyChangeSupport = new PropertyChangeSupport(this);
     in.defaultReadObject();
+    
+    // Ensure angle is always positive and between 0 and 2 PI
+    this.angle = (float)((this.angle % TWICE_PI + TWICE_PI) % TWICE_PI);
   }
 
   /**
@@ -540,7 +556,6 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
       if (height != this.height) {
         float oldHeight = this.height;
         this.height = height;
-        this.shapeCache = null;
         this.propertyChangeSupport.firePropertyChange(Property.HEIGHT.name(), oldHeight, height);
       }
     } else {
@@ -925,14 +940,14 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
   }
 
   /**
-   * Returns the angle in radians of this piece of furniture. 
+   * Returns the angle in radians of this piece of furniture around vertical axis. 
    */
   public float getAngle() {
     return this.angle;
   }
 
   /**
-   * Sets the angle of this piece. Once this piece is updated, 
+   * Sets the angle of this piece around vertical axis. Once this piece is updated, 
    * listeners added to this piece will receive a change notification.
    */
   public void setAngle(float angle) {
@@ -1028,14 +1043,24 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
   }
   
   /**
-   * Returns <code>true</code> if this piece is visible at the given level.
+   * Returns <code>true</code> if this piece is at the given <code>level</code> 
+   * or at a level with the same elevation and a smaller elevation index
+   * or if the elevation of its highest point is higher than <code>level</code> elevation.
    * @since 3.4
    */
   public boolean isAtLevel(Level level) {
-    return this.level == level 
-        || this.level != null && level != null
-            && this.level.getElevation() < level.getElevation()
-            && isTopAtLevel(level);
+    if (this.level == level) {
+      return true;
+    } else if (this.level != null && level != null) {
+      float pieceLevelElevation = this.level.getElevation();
+      float levelElevation = level.getElevation();
+      return pieceLevelElevation == levelElevation
+             && this.level.getElevationIndex() < level.getElevationIndex()
+          || pieceLevelElevation < levelElevation
+             && isTopAtLevel(level);
+    } else {
+      return false;
+    }
   }
   
   /**
@@ -1199,14 +1224,10 @@ public class HomePieceOfFurniture implements PieceOfFurniture, Serializable, Sel
    */
   @Override
   public HomePieceOfFurniture clone() {
-    try {
-      HomePieceOfFurniture clone = (HomePieceOfFurniture)super.clone();
-      clone.propertyChangeSupport = new PropertyChangeSupport(clone);
-      clone.level = null;
-      return clone;
-    } catch (CloneNotSupportedException ex) {
-      throw new IllegalStateException("Super class isn't cloneable"); 
-    }
+    HomePieceOfFurniture clone = (HomePieceOfFurniture)super.clone();
+    clone.propertyChangeSupport = new PropertyChangeSupport(clone);
+    clone.level = null;
+    return clone;
   }
 
   /**
