@@ -48,12 +48,12 @@ public abstract class UserPreferences {
    * The properties of user preferences that may change. <code>PropertyChangeListener</code>s added 
    * to user preferences will be notified under a property name equal to the string value of one these properties.
    */
-  public enum Property {LANGUAGE, SUPPORTED_LANGUAGES, UNIT, MAGNETISM_ENABLED, RULERS_VISIBLE, GRID_VISIBLE, 
+  public enum Property {LANGUAGE, SUPPORTED_LANGUAGES, UNIT, MAGNETISM_ENABLED, RULERS_VISIBLE, GRID_VISIBLE, DEFAULT_FONT_NAME, 
                         FURNITURE_VIEWED_FROM_TOP, ROOM_FLOOR_COLORED_OR_TEXTURED, WALL_PATTERN, NEW_WALL_PATTERN,    
-                        NEW_WALL_HEIGHT, NEW_WALL_THICKNESS, NEW_FLOOR_THICKNESS, RECENT_HOMES, IGNORED_ACTION_TIP,
-                        FURNITURE_CATALOG_VIEWED_IN_TREE, NAVIGATION_PANEL_VISIBLE, AERIAL_VIEW_CENTERED_ON_SELECTION_ENABLED, 
-                        CHECK_UPDATES_ENABLED, UPDATES_MINIMUM_DATE, AUTO_SAVE_DELAY_FOR_RECOVERY, AUTO_COMPLETION_STRINGS,
-                        RECENT_COLORS, RECENT_TEXTURES}
+                        NEW_WALL_THICKNESS, NEW_WALL_HEIGHT, NEW_WALL_SIDEBOARD_THICKNESS, NEW_WALL_SIDEBOARD_HEIGHT, NEW_FLOOR_THICKNESS, 
+                        RECENT_HOMES, IGNORED_ACTION_TIP, FURNITURE_CATALOG_VIEWED_IN_TREE, NAVIGATION_PANEL_VISIBLE, 
+                        AERIAL_VIEW_CENTERED_ON_SELECTION_ENABLED, CHECK_UPDATES_ENABLED, UPDATES_MINIMUM_DATE, AUTO_SAVE_DELAY_FOR_RECOVERY, 
+                        AUTO_COMPLETION_STRINGS, RECENT_COLORS, RECENT_TEXTURES}
   
   public static final String FURNITURE_LIBRARY_TYPE = "Furniture library"; 
   public static final String TEXTURES_LIBRARY_TYPE  = "Textures library"; 
@@ -102,12 +102,15 @@ public abstract class UserPreferences {
   private boolean          magnetismEnabled    = true;
   private boolean          rulersVisible       = true;
   private boolean          gridVisible         = true;
+  private String           defaultFontName;
   private boolean          furnitureViewedFromTop;
   private boolean          roomFloorColoredOrTextured;
   private TextureImage     wallPattern;
   private TextureImage     newWallPattern;
   private float            newWallThickness;
   private float            newWallHeight;
+  private float            newWallBaseboardThickness;
+  private float            newWallBaseboardHeight;
   private float            newFloorThickness;
   private List<String>     recentHomes;
   private boolean          checkUpdatesEnabled;
@@ -118,7 +121,7 @@ public abstract class UserPreferences {
   private List<TextureImage> recentTextures;
 
   /**
-   * Creates user preferences.</br> 
+   * Creates user preferences.<br> 
    * Caution: during creation, the default locale will be updated if it doesn't belong to the supported ones. 
    */
   public UserPreferences() {
@@ -317,7 +320,7 @@ public abstract class UserPreferences {
   }
 
   /**
-   * Returns the array of available languages in Sweet Home 3D.
+   * Sets the available languages in Sweet Home 3D.
    * @since 3.4
    */
   protected void setSupportedLanguages(String [] supportedLanguages) {
@@ -338,7 +341,7 @@ public abstract class UserPreferences {
    * <code>resourceClass</code>, then if this file doesn't exist, it searches 
    * the key prefixed by <code>resourceClass</code> name and a dot in a package.properties file 
    * in the folder matching the package of <code>resourceClass</code>. 
-   * @exception IllegalArgumentException if no string for the given key can be found
+   * @throws IllegalArgumentException if no string for the given key can be found
    */
   public String getLocalizedString(Class<?> resourceClass,
                                    String   resourceKey, 
@@ -380,7 +383,7 @@ public abstract class UserPreferences {
    * as a format string, and the returned string will be formatted with these parameters. 
    * This implementation searches the key in a properties file named as 
    * <code>resourceFamily</code>. 
-   * @exception IllegalArgumentException if no string for the given key can be found
+   * @throws IllegalArgumentException if no string for the given key can be found
    * @since 2.3
    */
   public String getLocalizedString(String resourceFamily,
@@ -607,6 +610,28 @@ public abstract class UserPreferences {
           !gridVisible, gridVisible);
     }
   }
+  
+  /**
+   * Returns the name of the font that should be used by default or <code>null</code> 
+   * if the default font should be the default one in the application.
+   * @since 5.0
+   */
+  public String getDefaultFontName() {
+    return this.defaultFontName;
+  }
+
+  /**
+   * Sets the name of the font that should be used by default.
+   * @since 5.0
+   */
+  public void setDefaultFontName(String defaultFontName) {
+    if (defaultFontName != this.defaultFontName
+        && (defaultFontName == null || !defaultFontName.equals(this.defaultFontName))) {
+      String oldName = this.defaultFontName;
+      this.defaultFontName = defaultFontName;
+      this.propertyChangeSupport.firePropertyChange(Property.DEFAULT_FONT_NAME.name(), oldName, defaultFontName);
+    }
+  }
 
   /**
    * Returns <code>true</code> if furniture should be viewed from its top in plan.
@@ -740,6 +765,50 @@ public abstract class UserPreferences {
     }
   }
 
+  /**
+   * Returns default baseboard thickness of new walls in home. 
+   * @since 5.0
+   */
+  public float getNewWallBaseboardThickness() {
+    return this.newWallBaseboardThickness;
+  }
+
+  /**
+   * Sets default baseboard thickness of new walls in home, and notifies
+   * listeners of this change.  
+   * @since 5.0
+   */
+  public void setNewWallBaseboardThickness(float newWallBaseboardThickness) {
+    if (this.newWallBaseboardThickness != newWallBaseboardThickness) {
+      float oldThickness = this.newWallBaseboardThickness;
+      this.newWallBaseboardThickness = newWallBaseboardThickness;
+      this.propertyChangeSupport.firePropertyChange(Property.NEW_WALL_SIDEBOARD_THICKNESS.name(), 
+          oldThickness, newWallBaseboardThickness);
+    }
+  }
+
+  /**
+   * Returns default baseboard height of new home walls. 
+   * @since 5.0
+   */
+  public float getNewWallBaseboardHeight() {
+    return this.newWallBaseboardHeight;
+  }
+
+  /**
+   * Sets default baseboard height of new walls, and notifies
+   * listeners of this change. 
+   * @since 5.0
+   */
+  public void setNewWallBaseboardHeight(float newWallBaseboardHeight) {
+    if (this.newWallBaseboardHeight != newWallBaseboardHeight) {
+      float oldHeight = this.newWallBaseboardHeight;
+      this.newWallBaseboardHeight = newWallBaseboardHeight;
+      this.propertyChangeSupport.firePropertyChange(Property.NEW_WALL_SIDEBOARD_HEIGHT.name(), 
+          oldHeight, newWallBaseboardHeight);
+    }
+  }
+
   /**
    * Returns default thickness of the floor of new levels in home. 
    * @since 3.4
@@ -873,8 +942,8 @@ public abstract class UserPreferences {
   
   /**
    * Returns whether an action tip should be ignored or not. 
-   * <br>This method should be overridden to return the the display information
-   * stored in {@link #setActionTipIgnored(String) setActionTipDisplayed}.
+   * <br>This method should be overridden to return the display information
+   * stored in {@link #setActionTipIgnored(String) setActionTipIgnored}.
    * By default it returns <code>true</code>. 
    */
   public boolean isActionTipIgnored(String actionKey) {
