@@ -99,6 +99,7 @@ public class FileContentManager implements ContentManager {
   private static final String LWS_EXTENSION = ".lws";
   private static final String THREEDS_EXTENSION = ".3ds";
   private static final String DAE_EXTENSION = ".dae";
+  private static final String KMZ_EXTENSION = ".kmz";
   private static final String ZIP_EXTENSION = ".zip";
   private static final FileFilter [] MODEL_FILTERS = {
      OBJ_FILTER [0],
@@ -141,6 +142,19 @@ public class FileContentManager implements ContentManager {
          return "DAE - Collada";
        }
      },
+     new FileFilter() {
+       @Override
+       public boolean accept(File file) {
+         // Accept directories and ZIP files
+         return file.isDirectory()
+                || file.getName().toLowerCase().endsWith(KMZ_EXTENSION);
+       }
+   
+       @Override
+       public String getDescription() {
+         return "KMZ";
+       }
+     },
      new FileFilter() {
        @Override
        public boolean accept(File file) {
@@ -201,7 +215,7 @@ public class FileContentManager implements ContentManager {
       new FileFilter() {
         @Override
         public boolean accept(File file) {
-          // Accept directories and .sh3d files
+          // Accept directories and .bmp files
           return file.isDirectory()
                  || file.getName().toLowerCase().endsWith(BMP_EXTENSION)
                  || file.getName().toLowerCase().endsWith(WBMP_EXTENSION);
@@ -302,6 +316,7 @@ public class FileContentManager implements ContentManager {
   
   private final UserPreferences           preferences;
   private final String                    sweetHome3DFileExtension;
+  private final String                    sweetHome3DFileExtension2;
   private final String                    languageLibraryFileExtension;
   private final String                    furnitureLibraryFileExtension;
   private final String                    texturesLibraryFileExtension;
@@ -313,6 +328,14 @@ public class FileContentManager implements ContentManager {
   public FileContentManager(final UserPreferences preferences) {  
     this.preferences = preferences;
     this.sweetHome3DFileExtension = preferences.getLocalizedString(FileContentManager.class, "homeExtension");
+    String homeExtension2;
+    try {
+      // Get optional second extension
+      homeExtension2 = preferences.getLocalizedString(FileContentManager.class, "homeExtension2");
+    } catch (IllegalArgumentException ex) {
+      homeExtension2 = null;
+    }
+    this.sweetHome3DFileExtension2 = homeExtension2;
     this.languageLibraryFileExtension = preferences.getLocalizedString(FileContentManager.class, "languageLibraryExtension");
     this.furnitureLibraryFileExtension = preferences.getLocalizedString(FileContentManager.class, "furnitureLibraryExtension");
     this.texturesLibraryFileExtension = preferences.getLocalizedString(FileContentManager.class, "texturesLibraryExtension");
@@ -334,9 +357,11 @@ public class FileContentManager implements ContentManager {
         new FileFilter() {
           @Override
           public boolean accept(File file) {
-            // Accept directories and .sh3d files
+            // Accept directories, .sh3d and .sh3x files
             return file.isDirectory()
-                || file.getName().toLowerCase().endsWith(sweetHome3DFileExtension);
+                || file.getName().toLowerCase().endsWith(sweetHome3DFileExtension)
+                || (sweetHome3DFileExtension2 != null 
+                     && file.getName().toLowerCase().endsWith(sweetHome3DFileExtension2));
           }
           
           @Override
@@ -422,11 +447,14 @@ public class FileContentManager implements ContentManager {
 
     // Fill file default extension map
     this.fileExtensions = new HashMap<ContentType, String []>();
-    this.fileExtensions.put(ContentType.SWEET_HOME_3D,     new String [] {sweetHome3DFileExtension});
-    this.fileExtensions.put(ContentType.LANGUAGE_LIBRARY,  new String [] {languageLibraryFileExtension});
-    this.fileExtensions.put(ContentType.FURNITURE_LIBRARY, new String [] {furnitureLibraryFileExtension});
-    this.fileExtensions.put(ContentType.TEXTURES_LIBRARY,  new String [] {texturesLibraryFileExtension});
-    this.fileExtensions.put(ContentType.PLUGIN,            new String [] {pluginFileExtension});
+    String [] sweetHome3DFileExtensions = this.sweetHome3DFileExtension2 != null
+        ? new String [] {this.sweetHome3DFileExtension, this.sweetHome3DFileExtension2}
+        : new String [] {this.sweetHome3DFileExtension};
+    this.fileExtensions.put(ContentType.SWEET_HOME_3D,     sweetHome3DFileExtensions);
+    this.fileExtensions.put(ContentType.LANGUAGE_LIBRARY,  new String [] {this.languageLibraryFileExtension});
+    this.fileExtensions.put(ContentType.FURNITURE_LIBRARY, new String [] {this.furnitureLibraryFileExtension});
+    this.fileExtensions.put(ContentType.TEXTURES_LIBRARY,  new String [] {this.texturesLibraryFileExtension});
+    this.fileExtensions.put(ContentType.PLUGIN,            new String [] {this.pluginFileExtension});
     this.fileExtensions.put(ContentType.PNG,               new String [] {PNG_EXTENSION});
     this.fileExtensions.put(ContentType.JPEG,              new String [] {JPEG_EXTENSION});
     this.fileExtensions.put(ContentType.MOV,               new String [] {MOV_EXTENSION});
@@ -435,7 +463,7 @@ public class FileContentManager implements ContentManager {
     this.fileExtensions.put(ContentType.SVG,               new String [] {SVG_EXTENSION});
     this.fileExtensions.put(ContentType.OBJ,               new String [] {OBJ_EXTENSION});
     this.fileExtensions.put(ContentType.MODEL,             
-        new String [] {OBJ_EXTENSION, LWS_EXTENSION, THREEDS_EXTENSION, DAE_EXTENSION, ZIP_EXTENSION});
+        new String [] {OBJ_EXTENSION, LWS_EXTENSION, THREEDS_EXTENSION, DAE_EXTENSION, ZIP_EXTENSION, KMZ_EXTENSION});
     this.fileExtensions.put(ContentType.IMAGE,             
         new String [] {PNG_EXTENSION, JPEG_EXTENSION, BMP_EXTENSION, WBMP_EXTENSION, GIF_EXTENSION} );
   }
@@ -459,6 +487,10 @@ public class FileContentManager implements ContentManager {
                                     ContentType contentType) {
     switch (contentType) {
       case SWEET_HOME_3D :
+      case FURNITURE_LIBRARY :
+      case TEXTURES_LIBRARY :
+      case LANGUAGE_LIBRARY :
+      case PLUGIN :
         return new File(contentPath).getName();
       default :
         String fileName = new File(contentPath).getName();
@@ -624,7 +656,7 @@ public class FileContentManager implements ContentManager {
         }
       });
 
-      // Update directory
+    // Update directory
     File directory = getLastDirectory(contentType);
     if (directory != null && directory.exists()) {
       if (isDirectory(contentType)) {
