@@ -615,9 +615,9 @@ public class ImportedFurnitureWizardStepsPanel extends JPanel
     this.elevationSpinner = new NullableSpinner(elevationSpinnerModel);
     elevationSpinnerModel.addChangeListener(new ChangeListener() {
         public void stateChanged(ChangeEvent ev) {
-          widthSpinnerModel.removeChangeListener(this);
+          elevationSpinnerModel.removeChangeListener(this);
           controller.setElevation(elevationSpinnerModel.getLength());
-          widthSpinnerModel.addChangeListener(this);
+          elevationSpinnerModel.addChangeListener(this);
         }
       });
     controller.addPropertyChangeListener(ImportedFurnitureWizardController.Property.ELEVATION,
@@ -906,6 +906,18 @@ public class ImportedFurnitureWizardStepsPanel extends JPanel
           // Ignore panel orientation to ensure left button is always at left of panel
         }
       };
+    if (!OperatingSystem.isMacOSX()) {
+      // Make buttons square
+      Dimension preferredSize = this.turnUpButton.getPreferredSize();
+      preferredSize.width = 
+      preferredSize.height = preferredSize.height + 4;
+      this.turnUpButton.setPreferredSize(preferredSize);
+      this.turnLeftButton.setPreferredSize(preferredSize);
+      this.turnRightButton.setPreferredSize(preferredSize);
+      this.turnDownButton.setPreferredSize(preferredSize);
+      this.defaultOrientationButton.setPreferredSize(preferredSize);
+    }
+      
     rotationButtonsPanel.add(this.turnUpButton, new GridBagConstraints(
         1, 0, 1, 1, 0, 0, GridBagConstraints.SOUTH, 
         GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 0, 0));    
@@ -940,55 +952,55 @@ public class ImportedFurnitureWizardStepsPanel extends JPanel
         GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
     attributesPanel.add(this.nameLabel, new GridBagConstraints(
         1, 1, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.NONE, new Insets(0, 0, 10, 5), 0, 0));
+        GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
     attributesPanel.add(this.nameTextField, new GridBagConstraints(
         2, 1, 1, 1, 0, 0, GridBagConstraints.CENTER, 
-        GridBagConstraints.HORIZONTAL, new Insets(0, 0, 10, 0), 0, 0));
+        GridBagConstraints.HORIZONTAL, new Insets(0, 0, 5, 0), 0, 0));
     attributesPanel.add(this.addToCatalogCheckBox, new GridBagConstraints(
         1, 2, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.NONE, new Insets(0, 0, 10, 0), 0, 0));
+        GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 0, 0));
     attributesPanel.add(this.categoryLabel, new GridBagConstraints(
         1, 3, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.NONE, new Insets(0, 0, 10, 5), 0, 0));
+        GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
     attributesPanel.add(this.categoryComboBox, new GridBagConstraints(
         2, 3, 1, 1, 0, 0, GridBagConstraints.CENTER, 
-        GridBagConstraints.HORIZONTAL, new Insets(0, 0, 10, 0), 0, 0));
+        GridBagConstraints.HORIZONTAL, new Insets(0, 0, 5, 0), 0, 0));
     attributesPanel.add(this.widthLabel, new GridBagConstraints(
         1, 4, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.NONE, new Insets(0, 0, 10, 5), 0, 0));
+        GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
     attributesPanel.add(this.widthSpinner, new GridBagConstraints(
         2, 4, 1, 1, 0, 0, GridBagConstraints.CENTER, 
-        GridBagConstraints.HORIZONTAL, new Insets(0, 0, 10, 0), 0, 0));
+        GridBagConstraints.HORIZONTAL, new Insets(0, 0, 5, 0), 0, 0));
     attributesPanel.add(this.depthLabel, new GridBagConstraints(
         1, 5, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.NONE, new Insets(0, 0, 10, 5), 0, 0));
+        GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
     attributesPanel.add(this.depthSpinner, new GridBagConstraints(
         2, 5, 1, 1, 0, 0, GridBagConstraints.CENTER, 
-        GridBagConstraints.HORIZONTAL, new Insets(0, 0, 10, 0), 0, 0));
+        GridBagConstraints.HORIZONTAL, new Insets(0, 0, 5, 0), 0, 0));
     attributesPanel.add(this.heightLabel, new GridBagConstraints(
         1, 6, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.NONE, new Insets(0, 0, 10, 5), 0, 0));
+        GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
     attributesPanel.add(this.heightSpinner, new GridBagConstraints(
         2, 6, 1, 1, 0, 0, GridBagConstraints.CENTER, 
-        GridBagConstraints.HORIZONTAL, new Insets(0, 0, 10, 0), 0, 0));
+        GridBagConstraints.HORIZONTAL, new Insets(0, 0, 5, 0), 0, 0));
     attributesPanel.add(this.keepProportionsCheckBox, new GridBagConstraints(
         1, 7, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.NONE, new Insets(0, 0, 15, 0), 0, 0));
+        GridBagConstraints.NONE, new Insets(0, 0, 10, 0), 0, 0));
     attributesPanel.add(this.elevationLabel, new GridBagConstraints(
         1, 8, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.NONE, new Insets(0, 0, 10, 5), 0, 0));
+        GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
     attributesPanel.add(this.elevationSpinner, new GridBagConstraints(
         2, 8, 1, 1, 0, 0, GridBagConstraints.CENTER, 
-        GridBagConstraints.HORIZONTAL, new Insets(0, 0, 10, 0), 0, 0));
+        GridBagConstraints.HORIZONTAL, new Insets(0, 0, 5, 0), 0, 0));
     attributesPanel.add(this.movableCheckBox, new GridBagConstraints(
         1, 9, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.NONE, new Insets(0, 0, 10, 0), 0, 0));
+        GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 0, 0));
     attributesPanel.add(this.doorOrWindowCheckBox, new GridBagConstraints(
         1, 10, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.NONE, new Insets(0, 0, 10, 0), 0, 0));
+        GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 0, 0));
     attributesPanel.add(this.staircaseCheckBox, new GridBagConstraints(
         1, 11, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
-        GridBagConstraints.NONE, new Insets(0, 0, 10, 0), 0, 0));
+        GridBagConstraints.NONE, new Insets(0, 0, 5, 0), 0, 0));
     attributesPanel.add(this.colorLabel, new GridBagConstraints(
         1, 12, 1, 1, 0, 0, GridBagConstraints.LINE_START, 
         GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
@@ -1527,6 +1539,8 @@ public class ImportedFurnitureWizardStepsPanel extends JPanel
    * Preview component for model orientation. 
    */
   private static class RotationPreviewComponent extends JPanel {
+    private static final int COMPONENT_PREFERRED_WIDTH = Math.round(200 * SwingTools.getResolutionScale());
+    
     private ModelPreviewComponent perspectiveViewComponent3D;
     private JLabel                frontViewLabel;
     private ModelPreviewComponent frontViewComponent3D;
@@ -1595,7 +1609,8 @@ public class ImportedFurnitureWizardStepsPanel extends JPanel
 
     @Override
     public Dimension getPreferredSize() {
-      return new Dimension(200, 204 + this.frontViewLabel.getPreferredSize().height * 2);
+      return new Dimension(COMPONENT_PREFERRED_WIDTH, 
+          COMPONENT_PREFERRED_WIDTH + 4 + this.frontViewLabel.getPreferredSize().height * 2);
     }
 
     /**
@@ -1768,6 +1783,8 @@ public class ImportedFurnitureWizardStepsPanel extends JPanel
    * Preview component for model icon. 
    */
   private static class IconPreviewComponent extends AbstractModelPreviewComponent {
+    private static final int ICON_SIZE = Math.round(128 * SwingTools.getResolutionScale());
+    
     private ImportedFurnitureWizardController controller;
 
     public IconPreviewComponent(ImportedFurnitureWizardController controller) {
@@ -1777,13 +1794,17 @@ public class ImportedFurnitureWizardStepsPanel extends JPanel
       addColorListener(controller);
       addIconYawListener(controller);
 
-      setBackground(UIManager.getColor("window"));
+      Color backgroundColor = UIManager.getColor("window");
+      if (backgroundColor == null) {
+        backgroundColor = Color.LIGHT_GRAY;
+      }
+      setBackground(backgroundColor);
     }
 
     @Override
     public Dimension getPreferredSize() {
       Insets insets = getInsets();
-      return new Dimension(128 + insets.left + insets.right, 128  + insets.top + insets.bottom);
+      return new Dimension(ICON_SIZE + insets.left + insets.right, ICON_SIZE  + insets.top + insets.bottom);
     }
 
     /**
