@@ -28,6 +28,7 @@ import java.awt.Graphics2D;
 import java.awt.GridBagConstraints;
 import java.awt.GridBagLayout;
 import java.awt.Insets;
+import java.awt.RenderingHints;
 import java.awt.TexturePaint;
 import java.awt.datatransfer.DataFlavor;
 import java.awt.datatransfer.Transferable;
@@ -36,6 +37,7 @@ import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.awt.event.ItemEvent;
 import java.awt.event.ItemListener;
+import java.awt.geom.AffineTransform;
 import java.awt.geom.Rectangle2D;
 import java.awt.image.BufferedImage;
 import java.beans.PropertyChangeEvent;
@@ -43,6 +45,7 @@ import java.beans.PropertyChangeListener;
 import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
+import java.lang.reflect.InvocationTargetException;
 import java.net.MalformedURLException;
 import java.net.URL;
 import java.util.Arrays;
@@ -50,6 +53,7 @@ import java.util.Collections;
 import java.util.List;
 import java.util.concurrent.Executor;
 import java.util.concurrent.Executors;
+import java.util.concurrent.atomic.AtomicInteger;
 
 import javax.imageio.ImageIO;
 import javax.swing.ComboBoxEditor;
@@ -89,6 +93,12 @@ import com.eteks.sweethome3d.viewcontroller.View;
  * @author Emmanuel Puybaret
  */
 public class ImportedTextureWizardStepsPanel extends JPanel implements View {
+  private static final int LARGE_IMAGE_PIXEL_COUNT_THRESHOLD = 640 * 640;
+  private static final int IMAGE_PREFERRED_MAX_SIZE = 512;
+  private static final int LARGE_IMAGE_MAX_PIXEL_COUNT = IMAGE_PREFERRED_MAX_SIZE * IMAGE_PREFERRED_MAX_SIZE;
+  
+  private static final int IMAGE_PREFERRED_SIZE = Math.round(150 * SwingTools.getResolutionScale());
+  
   private final ImportedTextureWizardController controller;
   private CardLayout                      cardLayout;
   private JLabel                          imageChoiceOrChangeLabel;
@@ -433,7 +443,7 @@ public class ImportedTextureWizardStepsPanel extends JPanel implements View {
     imageChoiceTopPanel.add(this.imageChoiceOrChangeLabel, new GridBagConstraints(
         0, 0, 2, 1, 0, 0, GridBagConstraints.LINE_START, 
         GridBagConstraints.HORIZONTAL, new Insets(5, 0, 5, 0), 0, 0));
-    this.imageChoicePreviewComponent.setPreferredSize(new Dimension(150, 150));
+    this.imageChoicePreviewComponent.setPreferredSize(new Dimension(IMAGE_PREFERRED_SIZE, IMAGE_PREFERRED_SIZE));
     if (this.findImagesButton != null) {
       imageChoiceTopPanel.add(this.imageChoiceOrChangeButton, new GridBagConstraints(
           0, 1, 1, 1, 1, 0, GridBagConstraints.LINE_END, 
@@ -459,7 +469,7 @@ public class ImportedTextureWizardStepsPanel extends JPanel implements View {
     attributesPanel.add(this.attributesLabel, new GridBagConstraints(
         0, 0, 3, 1, 0, 0, GridBagConstraints.LINE_START, 
         GridBagConstraints.NONE, new Insets(5, 0, 10, 0), 0, 0));
-    this.attributesPreviewComponent.setPreferredSize(new Dimension(150, 150));
+    this.attributesPreviewComponent.setPreferredSize(new Dimension(IMAGE_PREFERRED_SIZE, IMAGE_PREFERRED_SIZE));
     attributesPanel.add(this.attributesPreviewComponent, new GridBagConstraints(
         0, 1, 1, 5, 1, 0, GridBagConstraints.NORTH, 
         GridBagConstraints.NONE, new Insets(0, 0, 5, 5), 0, 0));
@@ -527,7 +537,7 @@ public class ImportedTextureWizardStepsPanel extends JPanel implements View {
           public void run() {
             BufferedImage image = null;
             try {
-              image = readImage(preferences, catalogTexture.getImage());
+              image = readImage(catalogTexture.getImage(), preferences);
             } catch (IOException ex) {
               // image is null
             }
@@ -589,7 +599,15 @@ public class ImportedTextureWizardStepsPanel extends JPanel implements View {
 
           BufferedImage image = null;
           try {
-            image = readImage(preferences, imageContent);
+            // Check image is less than 10 million pixels
+            Dimension size = SwingTools.getImageSizeInPixels(imageContent);
+            if (size.width * (long)size.height > LARGE_IMAGE_PIXEL_COUNT_THRESHOLD) {
+              imageContent = readAndReduceImage(imageContent, size, preferences);
+              if (imageContent == null) {
+                return;
+              }
+            }
+            image = readImage(imageContent, preferences);
           } catch (IOException ex) {
             // image is null
           }
@@ -637,19 +655,80 @@ public class ImportedTextureWizardStepsPanel extends JPanel implements View {
       });
   }
 
+  /**
+   * Informs the user that the image size is larger and returns a reduced size image if he confirms 
+   * that the size should be reduced.
+   * Caution : this method must be thread safe because it's called from image loader executor. 
+   */
+  private Content readAndReduceImage(Content imageContent, 
+                                     final Dimension imageSize, 
+                                     final UserPreferences preferences) throws IOException {
+    try {
+      float factor;
+      float ratio = (float)imageSize.width / imageSize.height;
+      if (ratio < .5f || ratio > 2.) {
+        factor = (float)Math.sqrt((float)LARGE_IMAGE_MAX_PIXEL_COUNT / (imageSize.width * (long)imageSize.height));
+      } else if (ratio < 1f) {
+        factor = (float)IMAGE_PREFERRED_MAX_SIZE / imageSize.height;
+      } else {
+        factor = (float)IMAGE_PREFERRED_MAX_SIZE / imageSize.width;
+      }
+      final int reducedWidth = Math.round(imageSize.width * factor);
+      final int reducedHeight = Math.round(imageSize.height * factor);
+      final AtomicInteger result = new AtomicInteger(JOptionPane.CANCEL_OPTION);
+      EventQueue.invokeAndWait(new Runnable() {
+          public void run() {
+            String title = preferences.getLocalizedString(ImportedTextureWizardStepsPanel.class, "reduceImageSize.title");
+            String confirmMessage = preferences.getLocalizedString(ImportedTextureWizardStepsPanel.class, 
+                "reduceImageSize.message", imageSize.width, imageSize.height, reducedWidth, reducedHeight);
+            String reduceSize = preferences.getLocalizedString(ImportedTextureWizardStepsPanel.class, "reduceImageSize.reduceSize");
+            String keepUnchanged = preferences.getLocalizedString(ImportedTextureWizardStepsPanel.class, "reduceImageSize.keepUnchanged");      
+            String cancel = preferences.getLocalizedString(ImportedTextureWizardStepsPanel.class, "reduceImageSize.cancel");      
+            result.set(JOptionPane.showOptionDialog(SwingUtilities.getRootPane(ImportedTextureWizardStepsPanel.this), 
+                  confirmMessage, title, JOptionPane.YES_NO_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE,
+                  null, new Object [] {reduceSize, keepUnchanged, cancel}, keepUnchanged));
+          }
+        });
+      if (result.get() == JOptionPane.CANCEL_OPTION) {
+        return null;
+      } else if (result.get() == JOptionPane.YES_OPTION) {
+        updatePreviewComponentsWithWaitImage(preferences);
+        
+        InputStream contentStream = imageContent.openStream();
+        BufferedImage image = ImageIO.read(contentStream);
+        contentStream.close();
+        if (image != null) {
+          BufferedImage reducedImage = new BufferedImage(reducedWidth, reducedHeight, image.getType());
+          Graphics2D g2D = (Graphics2D)reducedImage.getGraphics();
+          g2D.setRenderingHint(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
+          g2D.drawImage(image, AffineTransform.getScaleInstance(factor, factor), null);
+          g2D.dispose();
+          
+          File file = OperatingSystem.createTemporaryFile("texture", ".tmp");
+          ImageIO.write(reducedImage, image.getTransparency() == BufferedImage.OPAQUE ? "JPEG" : "PNG", file);
+          return new TemporaryURLContent(file.toURI().toURL());
+        }
+      }
+      return imageContent;
+    } catch (InterruptedException ex) {
+      return imageContent;
+    } catch (InvocationTargetException ex) {
+      ex.printStackTrace();
+      return imageContent;
+    } catch (IOException ex) {
+      updatePreviewComponentsImage(null);
+      throw ex;
+    } 
+  }
+
   /**
    * Reads image from <code>imageContent</code>. 
    * Caution : this method must be thread safe because it's called from image loader executor. 
    */
-  private BufferedImage readImage(UserPreferences preferences,
-                                  Content imageContent) throws IOException {
+  private BufferedImage readImage(Content imageContent,
+                                  UserPreferences preferences) throws IOException {
     try {
-      // Display a waiting image while loading
-      if (waitImage == null) {
-        waitImage = ImageIO.read(ImportedTextureWizardStepsPanel.class.getResource(
-            preferences.getLocalizedString(ImportedTextureWizardStepsPanel.class, "waitIcon")));
-      }
-      updatePreviewComponentsImage(waitImage);
+      updatePreviewComponentsWithWaitImage(preferences);
       
       // Read the image content
       InputStream contentStream = imageContent.openStream();
@@ -668,12 +747,29 @@ public class ImportedTextureWizardStepsPanel extends JPanel implements View {
     } 
   }
   
+  private void updatePreviewComponentsWithWaitImage(UserPreferences preferences) throws IOException {
+    // Display a waiting image while loading
+    if (waitImage == null) {
+      waitImage = ImageIO.read(ImportedTextureWizardStepsPanel.class.getResource(
+          preferences.getLocalizedString(ImportedTextureWizardStepsPanel.class, "waitIcon")));
+    }
+    updatePreviewComponentsImage(waitImage);
+  }
+  
   /**
    * Updates the <code>image</code> displayed by preview components.  
    */
-  private void updatePreviewComponentsImage(BufferedImage image) {
-    this.imageChoicePreviewComponent.setImage(image);
-    this.attributesPreviewComponent.setImage(image);
+  private void updatePreviewComponentsImage(final BufferedImage image) {
+    if (EventQueue.isDispatchThread()) {
+      this.imageChoicePreviewComponent.setImage(image);
+      this.attributesPreviewComponent.setImage(image);
+    } else {
+      EventQueue.invokeLater(new Runnable() {
+          public void run() {
+            updatePreviewComponentsImage(image);
+          }
+        });
+    }
   }
 
   /**
@@ -723,24 +819,23 @@ public class ImportedTextureWizardStepsPanel extends JPanel implements View {
    */
   private void updateAttributesPreviewImage() {
     BufferedImage attributesPreviewImage = this.attributesPreviewComponent.getImage();
-    final int imageSize = 150;
     if (attributesPreviewImage == null
         || attributesPreviewImage == this.imageChoicePreviewComponent.getImage()) {
-      attributesPreviewImage = new BufferedImage(imageSize, imageSize, BufferedImage.TYPE_INT_RGB);
+      attributesPreviewImage = new BufferedImage(IMAGE_PREFERRED_SIZE, IMAGE_PREFERRED_SIZE, BufferedImage.TYPE_INT_RGB);
       this.attributesPreviewComponent.setImage(attributesPreviewImage);
     }
     // Fill image with a white background
     Graphics2D g2D = (Graphics2D)attributesPreviewImage.getGraphics();
     g2D.setPaint(Color.WHITE);
-    g2D.fillRect(0, 0, imageSize, imageSize);    
+    g2D.fillRect(0, 0, IMAGE_PREFERRED_SIZE, IMAGE_PREFERRED_SIZE);    
     BufferedImage textureImage = this.imageChoicePreviewComponent.getImage();
     if (textureImage != null) {
       // Draw the texture image as if it will be shown on a 250 x 250 cm wall
       g2D.setPaint(new TexturePaint(textureImage, 
           new Rectangle2D.Float(0, 0, 
-              this.controller.getWidth() / 250 * imageSize,
-              this.controller.getHeight() / 250 * imageSize)));
-      g2D.fillRect(0, 0, imageSize, imageSize);
+              this.controller.getWidth() / 250 * IMAGE_PREFERRED_SIZE,
+              this.controller.getHeight() / 250 * IMAGE_PREFERRED_SIZE)));
+      g2D.fillRect(0, 0, IMAGE_PREFERRED_SIZE, IMAGE_PREFERRED_SIZE);
     }
     g2D.dispose();
     this.attributesPreviewComponent.repaint();
