@@ -28,6 +28,7 @@ import com.eteks.sweethome3d.model.FurnitureCatalog;
 import com.eteks.sweethome3d.model.Home;
 import com.eteks.sweethome3d.model.UserPreferences;
 import com.eteks.sweethome3d.viewcontroller.BackgroundImageWizardController;
+import com.eteks.sweethome3d.viewcontroller.BaseboardChoiceController;
 import com.eteks.sweethome3d.viewcontroller.CompassController;
 import com.eteks.sweethome3d.viewcontroller.DialogView;
 import com.eteks.sweethome3d.viewcontroller.FurnitureCatalogController;
@@ -51,6 +52,7 @@ import com.eteks.sweethome3d.viewcontroller.PhotoController;
 import com.eteks.sweethome3d.viewcontroller.PhotosController;
 import com.eteks.sweethome3d.viewcontroller.PlanController;
 import com.eteks.sweethome3d.viewcontroller.PlanView;
+import com.eteks.sweethome3d.viewcontroller.PolylineController;
 import com.eteks.sweethome3d.viewcontroller.PrintPreviewController;
 import com.eteks.sweethome3d.viewcontroller.RoomController;
 import com.eteks.sweethome3d.viewcontroller.TextureChoiceController;
@@ -69,6 +71,10 @@ import com.eteks.sweethome3d.viewcontroller.WizardController;
  * @author Emmanuel Puybaret
  */
 public class SwingViewFactory implements ViewFactory {
+  public SwingViewFactory() {
+    SwingTools.updateComponentDefaults();
+  }
+  
   /**
    * Returns a new view that displays furniture <code>catalog</code>.
    */
@@ -211,6 +217,14 @@ public class SwingViewFactory implements ViewFactory {
     return new RoomPanel(preferences, roomController);
   }
   
+  /**
+   * Returns a new view that edits polyline values.
+   */
+  public DialogView createPolylineView(UserPreferences preferences,
+                                       PolylineController polylineController) {
+    return new PolylinePanel(preferences, polylineController);
+  }
+
   /**
    * Returns a new view that edits label values.
    */
@@ -252,6 +266,14 @@ public class SwingViewFactory implements ViewFactory {
     return new TextureChoiceComponent(preferences, textureChoiceController);
   }
 
+  /**
+   * Returns a new view that edits the baseboard of its controller.  
+   */
+  public View createBaseboardChoiceView(UserPreferences preferences,
+                                        BaseboardChoiceController baseboardChoiceController) {
+    return new BaseboardChoiceComponent(preferences, baseboardChoiceController);
+  }
+
   /**
    * Returns a new view that edits the materials of its controller.  
    */
