@@ -37,6 +37,7 @@ import com.eteks.sweethome3d.model.Elevatable;
 import com.eteks.sweethome3d.model.Home;
 import com.eteks.sweethome3d.model.HomeEnvironment;
 import com.eteks.sweethome3d.model.HomePieceOfFurniture;
+import com.eteks.sweethome3d.model.Label;
 import com.eteks.sweethome3d.model.Level;
 import com.eteks.sweethome3d.model.ObserverCamera;
 import com.eteks.sweethome3d.model.Room;
@@ -113,7 +114,7 @@ public class HomeController3D implements Controller {
     if (selectedLevel != null) {
       selectedLevel.addPropertyChangeListener(levelElevationChangeListener);
     }
-    this.home.addPropertyChangeListener(Home.Property.SELECTED_LEVEL, new PropertyChangeListener() {
+    home.addPropertyChangeListener(Home.Property.SELECTED_LEVEL, new PropertyChangeListener() {
         public void propertyChange(PropertyChangeEvent ev) {
           Level oldSelectedLevel = (Level)ev.getOldValue();
           Level selectedLevel = home.getSelectedLevel();
@@ -146,8 +147,8 @@ public class HomeController3D implements Controller {
            }
          }
        };
-     this.home.addPropertyChangeListener(Home.Property.SELECTED_LEVEL, selectedLevelListener);     
-     this.home.getEnvironment().addPropertyChangeListener(HomeEnvironment.Property.ALL_LEVELS_VISIBLE, selectedLevelListener);
+    home.addPropertyChangeListener(Home.Property.SELECTED_LEVEL, selectedLevelListener);     
+    home.getEnvironment().addPropertyChangeListener(HomeEnvironment.Property.ALL_LEVELS_VISIBLE, selectedLevelListener);
   }
 
   private float getObserverCameraMinimumElevation(final Home home) {
@@ -280,6 +281,8 @@ public class HomeController3D implements Controller {
   
   /**
    * Moves home camera of <code>delta</code>.
+   * @param delta  the value in cm that the camera should move forward 
+   *               (with a negative delta) or backward (with a positive delta)
    */
   public void moveCamera(float delta) {
     this.cameraState.moveCamera(delta);
@@ -287,6 +290,8 @@ public class HomeController3D implements Controller {
 
   /**
    * Moves home camera sideways of <code>delta</code>.
+   * @param delta  the value in cm that the camera should move left 
+   *               (with a negative delta) or right (with a positive delta)
    * @since 4.4 
    */
   public void moveCameraSideways(float delta) {
@@ -295,6 +300,8 @@ public class HomeController3D implements Controller {
 
   /**
    * Elevates home camera of <code>delta</code>.
+   * @param delta the value in cm that the camera should move down 
+   *              (with a negative delta) or up (with a positive delta)
    */
   public void elevateCamera(float delta) {
     this.cameraState.elevateCamera(delta);
@@ -302,6 +309,7 @@ public class HomeController3D implements Controller {
 
   /**
    * Rotates home camera yaw angle of <code>delta</code> radians.
+   * @param delta  the value in rad that the camera should turn around yaw axis
    */
   public void rotateCameraYaw(float delta) {
     this.cameraState.rotateCameraYaw(delta);
@@ -309,6 +317,7 @@ public class HomeController3D implements Controller {
 
   /**
    * Rotates home camera pitch angle of <code>delta</code> radians.
+   * @param delta  the value in rad that the camera should turn around pitch axis
    */
   public void rotateCameraPitch(float delta) {
     this.cameraState.rotateCameraPitch(delta);
@@ -340,6 +349,7 @@ public class HomeController3D implements Controller {
 
     public void moveCamera(float delta) {
     }
+    
     public void moveCameraSideways(float delta) {
     }
 
@@ -362,8 +372,8 @@ public class HomeController3D implements Controller {
    * Top camera controller state. 
    */
   private class TopCameraState extends CameraControllerState {
-    private final float MIN_WIDTH  = 1000;
-    private final float MIN_DEPTH  = 1000;
+    private final float MIN_WIDTH  = 100;
+    private final float MIN_DEPTH  = MIN_WIDTH;
     private final float MIN_HEIGHT = 20;
     
     private Camera      topCamera;
@@ -420,6 +430,16 @@ public class HomeController3D implements Controller {
           updateCameraFromHomeBounds(false);
         }
       };
+    private CollectionListener<Label> labelsListener = new CollectionListener<Label>() {
+        public void collectionChanged(CollectionEvent<Label> ev) {
+          if (ev.getType() == CollectionEvent.Type.ADD) {
+            ev.getItem().addPropertyChangeListener(objectChangeListener);
+          } else if (ev.getType() == CollectionEvent.Type.DELETE) {
+            ev.getItem().removePropertyChangeListener(objectChangeListener);
+          } 
+          updateCameraFromHomeBounds(false);
+        }
+      };
     private SelectionListener selectionListener = new SelectionListener() {
         public void selectionChanged(SelectionEvent ev) {
           updateCameraFromHomeBounds(false);
@@ -452,6 +472,10 @@ public class HomeController3D implements Controller {
         room.addPropertyChangeListener(this.objectChangeListener);
       }
       home.addRoomsListener(this.roomsListener);
+      for (Label label : home.getLabels()) {
+        label.addPropertyChangeListener(this.objectChangeListener);
+      }
+      home.addLabelsListener(this.labelsListener);
       home.addSelectionListener(this.selectionListener);
     }
     
@@ -468,10 +492,10 @@ public class HomeController3D implements Controller {
      */
     private void updateCameraFromHomeBounds(boolean firstPieceOfFurnitureAddedToEmptyHome) {
       if (this.aerialViewBoundsLowerPoint == null) {
-        updateAerialViewBounds(this.aerialViewCenteredOnSelectionEnabled);
+        updateAerialViewBoundsFromHomeBounds(this.aerialViewCenteredOnSelectionEnabled);
       }
       float distanceToCenter = getCameraToAerialViewCenterDistance();
-      updateAerialViewBounds(this.aerialViewCenteredOnSelectionEnabled);
+      updateAerialViewBoundsFromHomeBounds(this.aerialViewCenteredOnSelectionEnabled);
       updateCameraIntervalToAerialViewCenter();
       placeCameraAt(distanceToCenter, firstPieceOfFurnitureAddedToEmptyHome);
     }
@@ -489,7 +513,7 @@ public class HomeController3D implements Controller {
      * Sets the bounds that includes walls, furniture and rooms, or only selected items 
      * if <code>centerOnSelection</code> is <code>true</code>.
      */
-    private void updateAerialViewBounds(boolean centerOnSelection) {
+    private void updateAerialViewBoundsFromHomeBounds(boolean centerOnSelection) {
       this.aerialViewBoundsLowerPoint = 
       this.aerialViewBoundsUpperPoint = null;
       List<Selectable> selectedItems = Collections.emptyList();
@@ -499,7 +523,9 @@ public class HomeController3D implements Controller {
           if (item instanceof Elevatable 
               && isItemAtVisibleLevel((Elevatable)item)
               && (!(item instanceof HomePieceOfFurniture)
-                  || ((HomePieceOfFurniture)item).isVisible())) {
+                  || ((HomePieceOfFurniture)item).isVisible())
+              && (!(item instanceof Label)
+                  || ((Label)item).getPitch() != null)) {
             selectedItems.add(item);
           }
         }
@@ -578,11 +604,30 @@ public class HomeController3D implements Controller {
         }
       }
       
+      for (Label label : selectionEmpty
+                             ? home.getLabels()
+                             : Home.getLabelsSubList(selectedItems)) {
+        if (label.getPitch() != null && isItemAtVisibleLevel(label)) {
+          float minZ;
+          float maxZ;
+          if (selectionEmpty) {
+            minZ = Math.max(0, label.getGroundElevation());
+            maxZ = Math.max(MIN_HEIGHT, label.getGroundElevation());
+          } else {
+            minZ = 
+            maxZ = label.getGroundElevation();
+          }
+          for (float [] point : label.getPoints()) {
+            updateAerialViewBounds(point [0], point [1], minZ, maxZ);
+          }
+        }
+      }
+      
       if (this.aerialViewBoundsLowerPoint == null) {
         this.aerialViewBoundsLowerPoint = new float [] {0, 0, 0};
         this.aerialViewBoundsUpperPoint = new float [] {MIN_WIDTH, MIN_DEPTH, MIN_HEIGHT};
       } else if (containsVisibleWalls && selectionEmpty) {
-        // If home contains walls, ensure bounds are always minimum 10 meters wide centered in middle of 3D view
+        // If home contains walls, ensure bounds are always minimum 1 meter wide centered in middle of 3D view
         if (MIN_WIDTH > this.aerialViewBoundsUpperPoint [0] - this.aerialViewBoundsLowerPoint [0]) {
           this.aerialViewBoundsLowerPoint [0] = (this.aerialViewBoundsLowerPoint [0] + this.aerialViewBoundsUpperPoint [0]) / 2 - MIN_WIDTH / 2;
           this.aerialViewBoundsUpperPoint [0] = this.aerialViewBoundsLowerPoint [0] + MIN_WIDTH;
@@ -619,7 +664,7 @@ public class HomeController3D implements Controller {
      * Returns <code>true</code> if the given <code>item</code> is at a visible level.
      */
     private boolean isItemAtVisibleLevel(Elevatable item) {
-      return item.getLevel() == null || item.getLevel().isVisible();
+      return item.getLevel() == null || item.getLevel().isViewableAndVisible();
     }
     
     /**
@@ -716,6 +761,10 @@ public class HomeController3D implements Controller {
         room.removePropertyChangeListener(this.objectChangeListener);
       }
       home.removeRoomsListener(this.roomsListener);
+      for (Label label : home.getLabels()) {
+        label.removePropertyChangeListener(this.objectChangeListener);
+      }
+      home.removeLabelsListener(this.labelsListener);
       for (Level level : home.getLevels()) {
         level.removePropertyChangeListener(this.objectChangeListener);
       }
@@ -829,9 +878,9 @@ public class HomeController3D implements Controller {
     @Override
     public void rotateCameraPitch(float delta) {
       float newPitch = this.observerCamera.getPitch() + delta; 
-      // Check new angle is between -60 and 75  
-      newPitch = Math.max(newPitch, -(float)Math.PI / 3);
-      newPitch = Math.min(newPitch, (float)Math.PI / 36 * 15);
+      // Check new angle is between -90 and 90  
+      newPitch = Math.max(newPitch, -(float)Math.PI / 2);
+      newPitch = Math.min(newPitch, (float)Math.PI / 2);
       this.observerCamera.setPitch(newPitch); 
       // Select observer camera for user feedback
       home.setSelectedItems(Arrays.asList(new Selectable [] {this.observerCamera}));
@@ -853,8 +902,8 @@ public class HomeController3D implements Controller {
         selectedItems.remove(this.observerCamera);
         home.setSelectedItems(selectedItems);
       }
-      for (Level room : home.getLevels()) {
-        room.removePropertyChangeListener(this.levelElevationChangeListener);
+      for (Level level : home.getLevels()) {
+        level.removePropertyChangeListener(this.levelElevationChangeListener);
       }
       home.removeLevelsListener(this.levelsListener);
       this.observerCamera = null;
