@@ -316,7 +316,7 @@ public class ImportedFurnitureWizardController extends WizardController
   /**
    * Returns the furniture choice step state.
    */
-  protected ImportedFurnitureWizardStepState getFurnitureModelStepStatee() {
+  protected ImportedFurnitureWizardStepState getFurnitureModelStepState() {
     return this.furnitureModelStepState;
   }
 
@@ -372,14 +372,14 @@ public class ImportedFurnitureWizardController extends WizardController
   }
 
   /**
-   * Adds the property change <code>listener</code> in parameter to this home.
+   * Adds the property change <code>listener</code> in parameter to this controller.
    */
   public void addPropertyChangeListener(Property property, PropertyChangeListener listener) {
     this.propertyChangeSupport.addPropertyChangeListener(property.name(), listener);
   }
 
   /**
-   * Removes the property change <code>listener</code> in parameter from this home.
+   * Removes the property change <code>listener</code> in parameter from this controller.
    */
   public void removePropertyChangeListener(Property property, PropertyChangeListener listener) {
     this.propertyChangeSupport.removePropertyChangeListener(property.name(), listener);
@@ -789,7 +789,7 @@ public class ImportedFurnitureWizardController extends WizardController
     
     @Override
     public void goBackToPreviousStep() {
-      setStepState(getFurnitureModelStepStatee());
+      setStepState(getFurnitureModelStepState());
     }
     
     @Override
