@@ -41,8 +41,10 @@ import javax.swing.undo.CannotUndoException;
 import javax.swing.undo.UndoableEdit;
 import javax.swing.undo.UndoableEditSupport;
 
+import com.eteks.sweethome3d.model.Baseboard;
 import com.eteks.sweethome3d.model.Home;
 import com.eteks.sweethome3d.model.HomeTexture;
+import com.eteks.sweethome3d.model.Level;
 import com.eteks.sweethome3d.model.Room;
 import com.eteks.sweethome3d.model.Selectable;
 import com.eteks.sweethome3d.model.UserPreferences;
@@ -58,12 +60,12 @@ public class RoomController implements Controller {
    */
   public enum Property {NAME, AREA_VISIBLE, FLOOR_VISIBLE, FLOOR_COLOR, FLOOR_PAINT, FLOOR_SHININESS,
       CEILING_VISIBLE, CEILING_COLOR, CEILING_PAINT, CEILING_SHININESS,
-      SPLIT_SURROUNDING_WALLS, WALL_SIDES_COLOR, WALL_SIDES_PAINT, WALL_SIDES_SHININESS}
+      SPLIT_SURROUNDING_WALLS, WALL_SIDES_COLOR, WALL_SIDES_PAINT, WALL_SIDES_SHININESS, WALL_SIDES_BASEBOARD}
   
   /**
    * The possible values for {@linkplain #getFloorPaint() room paint type}.
    */
-  public enum RoomPaint {COLORED, TEXTURED} 
+  public enum RoomPaint {DEFAULT, COLORED, TEXTURED} 
 
   private final Home                  home;
   private final UserPreferences       preferences;
@@ -73,6 +75,7 @@ public class RoomController implements Controller {
   private TextureChoiceController     floorTextureController;
   private TextureChoiceController     ceilingTextureController;
   private TextureChoiceController     wallSidesTextureController;
+  private BaseboardChoiceController   wallSidesBaseboardController;
   private final PropertyChangeSupport propertyChangeSupport;
   private DialogView                  roomView;
 
@@ -168,6 +171,19 @@ public class RoomController implements Controller {
     return this.wallSidesTextureController;
   }
 
+  /**
+   * Returns the controller of the wall sides baseboard.
+   * @since 5.0
+   */
+  public BaseboardChoiceController getWallSidesBaseboardController() {
+    // Create sub controller lazily only once it's needed
+    if (this.wallSidesBaseboardController == null) {
+      this.wallSidesBaseboardController = new BaseboardChoiceController(
+          this.preferences, this.viewFactory, this.contentManager);
+    }
+    return this.wallSidesBaseboardController;
+  }
+
   /**
    * Returns the view associated with this controller.
    */
@@ -204,7 +220,7 @@ public class RoomController implements Controller {
    * Returns <code>true</code> if the given <code>property</code> is editable.
    * Depending on whether a property is editable or not, the view associated to this controller
    * may render it differently.
-   * The implementation of this method always returns <code>true</code> except for <code>WALLS</code> properties. 
+   * The implementation of this method always returns <code>true</code> except for <code>WALL</code> properties. 
    */
   public boolean isPropertyEditable(Property property) {
     switch (property) {
@@ -212,6 +228,7 @@ public class RoomController implements Controller {
       case WALL_SIDES_COLOR :
       case WALL_SIDES_PAINT :
       case WALL_SIDES_SHININESS :
+      case WALL_SIDES_BASEBOARD :
         return this.wallSidesEditable;
       default :
         return true;
@@ -292,10 +309,22 @@ public class RoomController implements Controller {
       }
       getFloorTextureController().setTexture(floorTexture);
       
+      boolean defaultColorsAndTextures = true;
+      for (int i = 0; i < selectedRooms.size(); i++) {
+        Room room = selectedRooms.get(i);
+        if (room.getFloorColor() != null
+            || room.getFloorTexture() != null) {
+          defaultColorsAndTextures = false;
+          break;
+        }
+      }
+      
       if (floorColor != null) {
         setFloorPaint(RoomPaint.COLORED);
       } else if (floorTexture != null) {
         setFloorPaint(RoomPaint.TEXTURED);
+      } else if (defaultColorsAndTextures) {
+        setFloorPaint(RoomPaint.DEFAULT);
       } else {
         setFloorPaint(null);
       }
@@ -344,10 +373,22 @@ public class RoomController implements Controller {
       }
       getCeilingTextureController().setTexture(ceilingTexture);
       
+      defaultColorsAndTextures = true;
+      for (int i = 0; i < selectedRooms.size(); i++) {
+        Room room = selectedRooms.get(i);
+        if (room.getCeilingColor() != null
+            || room.getCeilingTexture() != null) {
+          defaultColorsAndTextures = false;
+          break;
+        }
+      }
+      
       if (ceilingColor != null) {
         setCeilingPaint(RoomPaint.COLORED);
       } else if (ceilingTexture != null) {
         setCeilingPaint(RoomPaint.TEXTURED);
+      } else if (defaultColorsAndTextures) {
+        setCeilingPaint(RoomPaint.DEFAULT);
       } else {
         setCeilingPaint(null);
       }
@@ -371,6 +412,12 @@ public class RoomController implements Controller {
       setWallSidesColor(null);
       setWallSidesPaint(null);
       setWallSidesShininess(null);
+      getWallSidesBaseboardController().setVisible(null);
+      getWallSidesBaseboardController().setThickness(null);
+      getWallSidesBaseboardController().setHeight(null);
+      getWallSidesBaseboardController().setColor(null);
+      getWallSidesBaseboardController().getTextureController().setTexture(null);
+      getWallSidesBaseboardController().setPaint(null);
     } else {
       this.wallSidesEditable = true;
       this.splitSurroundingWallsNeeded = splitWalls(wallSides, null, null, null);
@@ -411,6 +458,30 @@ public class RoomController implements Controller {
       }
       getWallSidesTextureController().setTexture(wallSidesTexture);
       
+      boolean defaultColorsAndTextures = true;
+      for (int i = 1; i < wallSides.size(); i++) {
+        WallSide wallSide = wallSides.get(i);
+        if ((wallSide.getSide() == WallSide.LEFT_SIDE
+                ? wallSide.getWall().getLeftSideColor()
+                : wallSide.getWall().getRightSideColor()) != null
+            || (wallSide.getSide() == WallSide.LEFT_SIDE
+                    ? wallSide.getWall().getLeftSideTexture()
+                    : wallSide.getWall().getRightSideTexture()) != null) {
+          defaultColorsAndTextures = false;
+          break;
+        }
+      }
+      
+      if (wallSidesColor != null) {
+        setWallSidesPaint(RoomPaint.COLORED);
+      } else if (wallSidesTexture != null) {
+        setWallSidesPaint(RoomPaint.TEXTURED);
+      } else if (defaultColorsAndTextures) {
+        setWallSidesPaint(RoomPaint.DEFAULT);
+      } else {
+        setWallSidesPaint(null);
+      }
+      
       // Search the common floor shininess among rooms
       Float wallSidesShininess = firstWallSide.getSide() == WallSide.LEFT_SIDE
           ? firstWallSide.getWall().getLeftSideShininess()
@@ -427,6 +498,133 @@ public class RoomController implements Controller {
         }
       }
       setWallSidesShininess(wallSidesShininess);
+
+      Baseboard firstWallSideBaseboard = firstWallSide.getSide() == WallSide.LEFT_SIDE
+          ? firstWallSide.getWall().getLeftSideBaseboard()
+          : firstWallSide.getWall().getRightSideBaseboard();
+      Boolean wallSidesBaseboardVisible = firstWallSideBaseboard != null;
+      for (int i = 1; i < wallSides.size(); i++) {
+        WallSide wallSide = wallSides.get(i);
+        if (wallSidesBaseboardVisible 
+            != (wallSide.getSide() == WallSide.LEFT_SIDE
+                  ? wallSide.getWall().getLeftSideBaseboard() != null
+                  : wallSide.getWall().getRightSideBaseboard() != null)) {
+          wallSidesBaseboardVisible = null;
+          break;
+        }
+      }
+      getWallSidesBaseboardController().setVisible(wallSidesBaseboardVisible);
+
+      // Search the common baseboard thickness  among walls
+      Float wallSidesBaseboardThickness = firstWallSideBaseboard != null
+          ? firstWallSideBaseboard.getThickness()
+          : this.preferences.getNewWallBaseboardThickness();
+      for (int i = 1; i < wallSides.size(); i++) {
+        WallSide wallSide = wallSides.get(i);
+        Baseboard baseboard = wallSide.getSide() == WallSide.LEFT_SIDE
+            ? wallSide.getWall().getLeftSideBaseboard()
+            : wallSide.getWall().getRightSideBaseboard();
+        if (!wallSidesBaseboardThickness.equals(baseboard != null
+                ? baseboard.getThickness()
+                : this.preferences.getNewWallBaseboardThickness())) {
+          wallSidesBaseboardThickness = null;
+          break;
+        }
+      }
+      getWallSidesBaseboardController().setThickness(wallSidesBaseboardThickness);
+      
+      // Search the common baseboard height among walls
+      Float wallSidesBaseboardHeight = firstWallSideBaseboard != null
+          ? firstWallSideBaseboard.getHeight()
+          : this.preferences.getNewWallBaseboardHeight();
+      for (int i = 1; i < wallSides.size(); i++) {
+        WallSide wallSide = wallSides.get(i);
+        Baseboard baseboard = wallSide.getSide() == WallSide.LEFT_SIDE
+            ? wallSide.getWall().getLeftSideBaseboard()
+            : wallSide.getWall().getRightSideBaseboard();
+        if (!wallSidesBaseboardHeight.equals(baseboard != null
+                ? baseboard.getHeight()
+                : this.preferences.getNewWallBaseboardHeight())) {
+          wallSidesBaseboardHeight = null;
+          break;
+        }
+      }
+      getWallSidesBaseboardController().setHeight(wallSidesBaseboardHeight);
+
+      // Search the maximum height among walls
+      float maxBaseboardHeight = firstWallSide.getWall().isTrapezoidal()
+          ? Math.max(firstWallSide.getWall().getHeight(), firstWallSide.getWall().getHeightAtEnd())
+          : firstWallSide.getWall().getHeight();
+      for (int i = 1; i < wallSides.size(); i++) {
+        Wall wall = wallSides.get(i).getWall();
+        maxBaseboardHeight = Math.max(maxBaseboardHeight, 
+            wall.isTrapezoidal()
+                ? Math.max(wall.getHeight(), wall.getHeightAtEnd())
+                : wall.getHeight());
+      }
+      getWallSidesBaseboardController().setMaxHeight(maxBaseboardHeight);
+
+      // Search the common baseboard color among walls
+      Integer wallSidesBaseboardColor = firstWallSideBaseboard != null
+          ? firstWallSideBaseboard.getColor()
+          : null;
+      if (wallSidesBaseboardColor != null) {
+        for (int i = 1; i < wallSides.size(); i++) {
+          WallSide wallSide = wallSides.get(i);
+          Baseboard baseboard = wallSide.getSide() == WallSide.LEFT_SIDE
+              ? wallSide.getWall().getLeftSideBaseboard()
+              : wallSide.getWall().getRightSideBaseboard();
+          if (baseboard == null
+              || !wallSidesBaseboardColor.equals(baseboard.getColor())) {
+            wallSidesBaseboardColor = null;
+            break;
+          }
+        }
+      }
+      getWallSidesBaseboardController().setColor(wallSidesBaseboardColor);
+      
+      // Search the common right baseboard texture among walls
+      HomeTexture wallSidesBaseboardTexture = firstWallSideBaseboard != null
+          ? firstWallSideBaseboard.getTexture()
+          : null;
+      if (wallSidesBaseboardTexture != null) {
+        for (int i = 1; i < wallSides.size(); i++) {
+          WallSide wallSide = wallSides.get(i);
+          Baseboard baseboard = wallSide.getSide() == WallSide.LEFT_SIDE
+              ? wallSide.getWall().getLeftSideBaseboard()
+              : wallSide.getWall().getRightSideBaseboard();
+          if (baseboard == null
+              || !wallSidesBaseboardTexture.equals(baseboard.getTexture())) {
+            wallSidesBaseboardTexture = null;
+            break;
+          }
+        }
+      } 
+      getWallSidesBaseboardController().getTextureController().setTexture(wallSidesBaseboardTexture);
+      
+      defaultColorsAndTextures = true;
+      for (int i = 0; i < wallSides.size(); i++) {
+        WallSide wallSide = wallSides.get(i);
+        Baseboard baseboard = wallSide.getSide() == WallSide.LEFT_SIDE
+            ? wallSide.getWall().getLeftSideBaseboard()
+            : wallSide.getWall().getRightSideBaseboard();
+        if (baseboard != null
+            && (baseboard.getColor() != null
+                || baseboard.getTexture() != null)) {
+          defaultColorsAndTextures = false;
+          break;
+        }
+      }
+      
+      if (wallSidesBaseboardColor != null) {
+        getWallSidesBaseboardController().setPaint(BaseboardChoiceController.BaseboardPaint.COLORED);
+      } else if (wallSidesBaseboardTexture != null) {
+        getWallSidesBaseboardController().setPaint(BaseboardChoiceController.BaseboardPaint.TEXTURED);
+      } else if (defaultColorsAndTextures) {
+        getWallSidesBaseboardController().setPaint(BaseboardChoiceController.BaseboardPaint.DEFAULT);
+      } else {
+        getWallSidesBaseboardController().setPaint(null);
+      }      
     }      
   }
   
@@ -436,15 +634,7 @@ public class RoomController implements Controller {
   private List<WallSide> getRoomsWallSides(List<Room> rooms, List<WallSide> defaultWallSides) {
     List<WallSide> wallSides = new ArrayList<WallSide>();
     for (Room room : rooms) {
-      float [][] points = room.getPoints();
-      GeneralPath roomShape = new GeneralPath();
-      roomShape.moveTo(points [0][0], points [0][1]);
-      for (int i = 1; i < points.length; i++) {
-        roomShape.lineTo(points [i][0], points [i][1]);
-      }
-      roomShape.closePath();
-      Area roomArea = new Area(roomShape);
-
+      Area roomArea = new Area(getPath(room.getPoints(), true));
       if (defaultWallSides != null) {
         for (WallSide wallSide : defaultWallSides) {
           if (isRoomItersectingWallSide(wallSide.getWall().getPoints(), wallSide.getSide(), roomArea)) {
@@ -453,7 +643,8 @@ public class RoomController implements Controller {
         }
       } else {
         for (Wall wall : this.home.getWalls()) {
-          if (wall.isAtLevel(this.home.getSelectedLevel())) {
+          if ((wall.getLevel() == null || wall.getLevel().isViewable())
+              && wall.isAtLevel(this.home.getSelectedLevel())) {
             float [][] wallPoints = wall.getPoints();
             if (isRoomItersectingWallSide(wallPoints, WallSide.LEFT_SIDE, roomArea)) {
               wallSides.add(new WallSide(wall, WallSide.LEFT_SIDE));
@@ -846,15 +1037,17 @@ public class RoomController implements Controller {
       String name = getName();
       Boolean areaVisible = getAreaVisible();
       Boolean floorVisible = getFloorVisible();
-      Integer floorColor = getFloorPaint() == RoomPaint.COLORED 
+      RoomPaint floorPaint = getFloorPaint();
+      Integer floorColor = floorPaint == RoomPaint.COLORED 
           ? getFloorColor() : null;
-      HomeTexture floorTexture = getFloorPaint() == RoomPaint.TEXTURED
+      HomeTexture floorTexture = floorPaint == RoomPaint.TEXTURED
           ? getFloorTextureController().getTexture() : null;
       Float floorShininess = getFloorShininess();
       Boolean ceilingVisible = getCeilingVisible();
-      Integer ceilingColor = getCeilingPaint() == RoomPaint.COLORED
+      RoomPaint ceilingPaint = getCeilingPaint();
+      Integer ceilingColor = ceilingPaint == RoomPaint.COLORED
           ? getCeilingColor() : null;
-      HomeTexture ceilingTexture = getCeilingPaint() == RoomPaint.TEXTURED
+      HomeTexture ceilingTexture = ceilingPaint == RoomPaint.TEXTURED
           ? getCeilingTextureController().getTexture() : null;
       Float ceilingShininess = getCeilingShininess();
       Integer wallSidesColor = getWallSidesPaint() == RoomPaint.COLORED 
@@ -862,6 +1055,15 @@ public class RoomController implements Controller {
       HomeTexture wallSidesTexture = getWallSidesPaint() == RoomPaint.TEXTURED
           ? getWallSidesTextureController().getTexture() : null;
       Float wallSidesShininess = getWallSidesShininess();
+      Boolean wallSidesBaseboardVisible = getWallSidesBaseboardController().getVisible();
+      Float wallSidesBaseboardThickness = getWallSidesBaseboardController().getThickness();
+      Float wallSidesBaseboardHeight = getWallSidesBaseboardController().getHeight();
+      BaseboardChoiceController.BaseboardPaint wallSidesBaseboardPaint = getWallSidesBaseboardController().getPaint();
+      Integer wallSidesBaseboardColor = wallSidesBaseboardPaint == BaseboardChoiceController.BaseboardPaint.COLORED 
+          ? getWallSidesBaseboardController().getColor() : null;
+      HomeTexture wallSidesBaseboardTexture = wallSidesBaseboardPaint == BaseboardChoiceController.BaseboardPaint.TEXTURED
+          ? getWallSidesBaseboardController().getTextureController().getTexture()
+          : null;
       List<WallSide> selectedRoomsWallSides = getRoomsWallSides(selectedRooms, null);
       
       // Create an array of modified rooms with their current properties values
@@ -888,15 +1090,21 @@ public class RoomController implements Controller {
         modifiedWallSides [i] = new ModifiedWallSide(selectedRoomsWallSides.get(i));
       }
       doModifyRoomsAndWallSides(home, modifiedRooms, name, areaVisible, 
-          floorVisible, floorColor, floorTexture, floorShininess, 
-          ceilingVisible, ceilingColor, ceilingTexture, ceilingShininess,
-          modifiedWallSides, wallSidesColor, wallSidesTexture, wallSidesShininess, null, null);
+          floorVisible, floorPaint, floorColor, floorTexture, floorShininess, 
+          ceilingVisible, ceilingPaint, ceilingColor, ceilingTexture, ceilingShininess,
+          modifiedWallSides, this.preferences.getNewWallBaseboardThickness(), this.preferences.getNewWallBaseboardHeight(),
+          wallSidesColor, wallSidesTexture, wallSidesShininess, 
+          wallSidesBaseboardVisible, wallSidesBaseboardThickness, wallSidesBaseboardHeight, 
+          wallSidesBaseboardPaint, wallSidesBaseboardColor, wallSidesBaseboardTexture, null, null);
       if (this.undoSupport != null) {
         UndoableEdit undoableEdit = new RoomsAndWallSidesModificationUndoableEdit(
             this.home, this.preferences, oldSelection, newSelection, modifiedRooms, name, areaVisible, 
-            floorColor, floorTexture, floorVisible, floorShininess,
-            ceilingColor, ceilingTexture, ceilingVisible, ceilingShininess,
-            modifiedWallSides, wallSidesColor, wallSidesTexture, wallSidesShininess,
+            floorVisible, floorPaint, floorColor, floorTexture, floorShininess,
+            ceilingVisible, ceilingPaint, ceilingColor, ceilingTexture, ceilingShininess,
+            modifiedWallSides, this.preferences.getNewWallBaseboardThickness(), this.preferences.getNewWallBaseboardHeight(),
+            wallSidesColor, wallSidesTexture, wallSidesShininess,
+            wallSidesBaseboardVisible, wallSidesBaseboardThickness, wallSidesBaseboardHeight, 
+            wallSidesBaseboardPaint, wallSidesBaseboardColor, wallSidesBaseboardTexture,
             deletedWalls.toArray(new ModifiedWall [deletedWalls.size()]), 
             addedWalls.toArray(new ModifiedWall [addedWalls.size()]));
         this.undoSupport.postEdit(undoableEdit);
@@ -926,7 +1134,8 @@ public class RoomController implements Controller {
           it.hasNext() && splitWallSide == null; ) {
         WallSide wallSide = (WallSide)it.next();
         Wall wall = wallSide.getWall();
-        if (wall.getArcExtent() == null) { // Ignore round walls
+        Float arcExtent = wall.getArcExtent();
+        if (arcExtent == null || arcExtent.floatValue() == 0) { // Ignore round walls
           Area wallArea = new Area(getPath(wall.getPoints(), true));
           for (WallSide intersectedWallSide : wallSides) {
             Wall intersectedWall = intersectedWallSide.getWall();
@@ -942,6 +1151,9 @@ public class RoomController implements Controller {
                   // Clone new walls to copy their characteristics 
                   firstWall = wall.clone();
                   secondWall = wall.clone();
+                  // Store wall level to add new walls at the same level as the split wall
+                  firstWall.setLevel(wall.getLevel());
+                  secondWall.setLevel(wall.getLevel());
                   
                   // Change split walls end and start point
                   firstWall.setXEnd(intersection [0]);
@@ -1051,9 +1263,11 @@ public class RoomController implements Controller {
     if (deletedWalls == null) { 
       return false; 
     } else {
-      for (Wall newWall : newWalls) {      
+      for (Wall newWall : newWalls) {     
+        ModifiedWall addedWall = new ModifiedWall(newWall);
+        addedWalls.add(addedWall);
         this.home.addWall(newWall);
-        addedWalls.add(new ModifiedWall(newWall));
+        newWall.setLevel(addedWall.getLevel());
       }
       for (ModifiedWall deletedWall : deletedWalls) {
         this.home.deleteWall(deletedWall.getWall());
@@ -1092,18 +1306,28 @@ public class RoomController implements Controller {
     private final ModifiedRoom []     modifiedRooms;
     private final String              name;
     private final Boolean             areaVisible;
+    private final Boolean             floorVisible;
+    private final RoomPaint           floorPaint;
     private final Integer             floorColor;
     private final HomeTexture         floorTexture;
-    private final Boolean             floorVisible;
     private final Float               floorShininess;
+    private final Boolean             ceilingVisible;
+    private final RoomPaint           ceilingPaint;
     private final Integer             ceilingColor;
     private final HomeTexture         ceilingTexture;
-    private final Boolean             ceilingVisible;
     private final Float               ceilingShininess;
     private final ModifiedWallSide [] modifiedWallSides;
+    private final float               newWallBaseboardHeight;
+    private final float               newWallBaseboardThickness;
     private final Integer             wallSidesColor;
     private final HomeTexture         wallSidesTexture;
     private final Float               wallSidesShininess;
+    private final Boolean             wallSidesBaseboardVisible;
+    private final Float               wallSidesBaseboardThickness;
+    private final Float               wallSidesBaseboardHeight;
+    private final BaseboardChoiceController.BaseboardPaint wallSidesBaseboardPaint;
+    private final Integer             wallSidesBaseboardColor;
+    private final HomeTexture         wallSidesBaseboardTexture;
     private final ModifiedWall []     deletedWalls;
     private final ModifiedWall []     addedWalls;
 
@@ -1114,18 +1338,28 @@ public class RoomController implements Controller {
                                           ModifiedRoom [] modifiedRooms,
                                           String name,
                                           Boolean areaVisible,
+                                          Boolean floorVisible,
+                                          RoomPaint floorPaint,
                                           Integer floorColor,
                                           HomeTexture floorTexture,
-                                          Boolean floorVisible,
                                           Float floorShininess,
+                                          Boolean ceilingVisible,
+                                          RoomPaint ceilingPaint,
                                           Integer ceilingColor,
                                           HomeTexture ceilingTexture,
-                                          Boolean ceilingVisible,
                                           Float ceilingShininess,
                                           ModifiedWallSide [] modifiedWallSides,
+                                          float newWallBaseboardThickness, 
+                                          float newWallBaseboardHeight,
                                           Integer wallSidesColor,
                                           HomeTexture wallSidesTexture,
                                           Float wallSidesShininess, 
+                                          Boolean wallSidesBaseboardVisible, 
+                                          Float wallSidesBaseboardThickness, 
+                                          Float wallSidesBaseboardHeight, 
+                                          BaseboardChoiceController.BaseboardPaint wallSidesBaseboardPaint, 
+                                          Integer wallSidesBaseboardColor, 
+                                          HomeTexture wallSidesBaseboardTexture,
                                           ModifiedWall [] deletedWalls, 
                                           ModifiedWall [] addedWalls) {
       this.home = home;
@@ -1135,18 +1369,28 @@ public class RoomController implements Controller {
       this.modifiedRooms = modifiedRooms;
       this.name = name;
       this.areaVisible = areaVisible;
+      this.floorVisible = floorVisible;
+      this.floorPaint = floorPaint;
       this.floorColor = floorColor;
       this.floorTexture = floorTexture;
-      this.floorVisible = floorVisible;
       this.floorShininess = floorShininess;
+      this.ceilingVisible = ceilingVisible;
+      this.ceilingPaint = ceilingPaint;
       this.ceilingColor = ceilingColor;
       this.ceilingTexture = ceilingTexture;
-      this.ceilingVisible = ceilingVisible;
       this.ceilingShininess = ceilingShininess;
       this.modifiedWallSides = modifiedWallSides;
+      this.newWallBaseboardThickness = newWallBaseboardThickness;
+      this.newWallBaseboardHeight = newWallBaseboardHeight;
       this.wallSidesColor = wallSidesColor;
       this.wallSidesTexture = wallSidesTexture;
       this.wallSidesShininess = wallSidesShininess;
+      this.wallSidesBaseboardVisible = wallSidesBaseboardVisible;
+      this.wallSidesBaseboardThickness = wallSidesBaseboardThickness;
+      this.wallSidesBaseboardHeight = wallSidesBaseboardHeight;
+      this.wallSidesBaseboardPaint = wallSidesBaseboardPaint;
+      this.wallSidesBaseboardColor = wallSidesBaseboardColor;
+      this.wallSidesBaseboardTexture = wallSidesBaseboardTexture;
       this.deletedWalls = deletedWalls;
       this.addedWalls = addedWalls;
     }
@@ -1163,9 +1407,12 @@ public class RoomController implements Controller {
       super.redo();
       doModifyRoomsAndWallSides(this.home,
           this.modifiedRooms, this.name, this.areaVisible, 
-          this.floorVisible, this.floorColor, this.floorTexture, this.floorShininess, 
-          this.ceilingVisible, this.ceilingColor, this.ceilingTexture, this.ceilingShininess,
-          this.modifiedWallSides, this.wallSidesColor, this.wallSidesTexture, this.wallSidesShininess,
+          this.floorVisible, this.floorPaint, this.floorColor, this.floorTexture, this.floorShininess, 
+          this.ceilingVisible, this.ceilingPaint, this.ceilingColor, this.ceilingTexture, this.ceilingShininess,
+          this.modifiedWallSides, newWallBaseboardThickness, this.newWallBaseboardHeight, 
+          this.wallSidesColor, this.wallSidesTexture, this.wallSidesShininess,
+          this.wallSidesBaseboardVisible, this.wallSidesBaseboardThickness, this.wallSidesBaseboardHeight, 
+          this.wallSidesBaseboardPaint, this.wallSidesBaseboardColor, this.wallSidesBaseboardTexture,
           this.deletedWalls, this.addedWalls); 
       this.home.setSelectedItems(this.newSelection); 
     }
@@ -1180,17 +1427,21 @@ public class RoomController implements Controller {
    * Modifies rooms and walls properties with the values in parameter.
    */
   private static void doModifyRoomsAndWallSides(Home home, ModifiedRoom [] modifiedRooms, 
-                             String name, Boolean areaVisible, 
-                             Boolean floorVisible, Integer floorColor, HomeTexture floorTexture, Float floorShininess,
-                             Boolean ceilingVisible, Integer ceilingColor, HomeTexture ceilingTexture, Float ceilingShininess,
-                             ModifiedWallSide [] modifiedWallSides, 
-                             Integer wallSidesColor, HomeTexture wallSidesTexture, Float wallSidesShininess, 
-                             ModifiedWall [] deletedWalls, 
-                             ModifiedWall [] addedWalls) {
+                                                String name, Boolean areaVisible, 
+                                                Boolean floorVisible, RoomPaint floorPaint, Integer floorColor, HomeTexture floorTexture, Float floorShininess,
+                                                Boolean ceilingVisible, RoomPaint ceilingPaint, Integer ceilingColor, HomeTexture ceilingTexture, Float ceilingShininess,
+                                                ModifiedWallSide [] modifiedWallSides, 
+                                                float newWallBaseboardThickness, float newWallBaseboardHeight,
+                                                Integer wallSidesColor, HomeTexture wallSidesTexture, Float wallSidesShininess, 
+                                                Boolean wallSidesBaseboardVisible, Float wallSidesBaseboardThickness, Float wallSidesBaseboardHeight, 
+                                                BaseboardChoiceController.BaseboardPaint wallSidesBaseboardPaint, Integer wallSidesBaseboardColor, HomeTexture wallSidesBaseboardTexture,
+                                                ModifiedWall [] deletedWalls, 
+                                                ModifiedWall [] addedWalls) {
     if (deletedWalls != null) {
       for (ModifiedWall newWall : addedWalls) {
-        newWall.reset();
+        newWall.resetJoinedWalls();
         home.addWall(newWall.getWall());
+        newWall.getWall().setLevel(newWall.getLevel());
       }
       for (ModifiedWall deletedWall : deletedWalls) {
         home.deleteWall(deletedWall.getWall());
@@ -1207,12 +1458,21 @@ public class RoomController implements Controller {
       if (floorVisible != null) {
         room.setFloorVisible(floorVisible);
       }
-      if (floorTexture != null) {
-        room.setFloorTexture(floorTexture);
-        room.setFloorColor(null);
-      } else if (floorColor != null) {
-        room.setFloorColor(floorColor);
-        room.setFloorTexture(null);
+      if (floorPaint != null) {
+        switch (floorPaint) {
+          case DEFAULT :
+            room.setFloorColor(null);
+            room.setFloorTexture(null);
+            break;
+          case COLORED :
+            room.setFloorColor(floorColor);
+            room.setFloorTexture(null);
+            break;
+          case TEXTURED :
+            room.setFloorColor(null);
+            room.setFloorTexture(floorTexture);
+            break;
+        }
       }
       if (floorShininess != null) {
         room.setFloorShininess(floorShininess);
@@ -1220,12 +1480,21 @@ public class RoomController implements Controller {
       if (ceilingVisible != null) {
         room.setCeilingVisible(ceilingVisible);
       }
-      if (ceilingTexture != null) {
-        room.setCeilingTexture(ceilingTexture);
-        room.setCeilingColor(null);
-      } else if (ceilingColor != null) {
-        room.setCeilingColor(ceilingColor);
-        room.setCeilingTexture(null);
+      if (ceilingPaint != null) {
+        switch (ceilingPaint) {
+          case DEFAULT :
+            room.setCeilingColor(null);
+            room.setCeilingTexture(null);
+            break;
+          case COLORED :
+            room.setCeilingColor(ceilingColor);
+            room.setCeilingTexture(null);
+            break;
+          case TEXTURED :
+            room.setCeilingColor(null);
+            room.setCeilingTexture(ceilingTexture);
+            break;
+        }
       }
       if (ceilingShininess != null) {
         room.setCeilingShininess(ceilingShininess);
@@ -1233,29 +1502,90 @@ public class RoomController implements Controller {
     }
     for (ModifiedWallSide modifiedWallSide : modifiedWallSides) {
       WallSide wallSide = modifiedWallSide.getWallSide();
+      Wall wall = wallSide.getWall();
       if (wallSidesColor != null) {
         if (wallSide.getSide() == WallSide.LEFT_SIDE) {
-          wallSide.getWall().setLeftSideColor(wallSidesColor);
+          wall.setLeftSideColor(wallSidesColor);
         } else {
-          wallSide.getWall().setRightSideColor(wallSidesColor);
+          wall.setRightSideColor(wallSidesColor);
         }
       }
       
       if (wallSidesTexture != null || wallSidesColor != null) {
         if (wallSide.getSide() == WallSide.LEFT_SIDE) {
-          wallSide.getWall().setLeftSideTexture(wallSidesTexture);
+          wall.setLeftSideTexture(wallSidesTexture);
         } else {
-          wallSide.getWall().setRightSideTexture(wallSidesTexture);
+          wall.setRightSideTexture(wallSidesTexture);
         }
       }
 
       if (wallSidesShininess != null) {
         if (wallSide.getSide() == WallSide.LEFT_SIDE) {
-          wallSide.getWall().setLeftSideShininess(wallSidesShininess);
+          wall.setLeftSideShininess(wallSidesShininess);
         } else {
-          wallSide.getWall().setRightSideShininess(wallSidesShininess);
+          wall.setRightSideShininess(wallSidesShininess);
         }
       }
+
+      if (wallSidesBaseboardVisible == Boolean.FALSE) {
+        if (wallSide.getSide() == WallSide.LEFT_SIDE) {
+          wall.setLeftSideBaseboard(null);
+        } else {
+          wall.setRightSideBaseboard(null);
+        }
+      } else {
+        Baseboard baseboard = wallSide.getSide() == WallSide.LEFT_SIDE
+            ? wall.getLeftSideBaseboard()
+            : wall.getRightSideBaseboard();
+        if (wallSidesBaseboardVisible == Boolean.TRUE 
+            || baseboard != null) {
+          float baseboardThickness = baseboard != null
+              ? baseboard.getThickness()
+              : newWallBaseboardThickness;
+          float baseboardHeight = baseboard != null
+              ? baseboard.getHeight()
+              : newWallBaseboardHeight;
+          Integer baseboardColor = baseboard != null
+              ? baseboard.getColor()
+              : null;
+          HomeTexture baseboardTexture = baseboard != null
+              ? baseboard.getTexture()
+              : null;
+          if (wallSidesBaseboardPaint != null) {
+            switch (wallSidesBaseboardPaint) {
+              case DEFAULT :
+                baseboardColor = null;
+                baseboardTexture = null;
+                break;
+              case COLORED :
+                if (wallSidesBaseboardColor != null) {
+                  baseboardColor = wallSidesBaseboardColor;
+                }
+                baseboardTexture = null;
+                break;
+              case TEXTURED :
+                baseboardColor = null;
+                if (wallSidesBaseboardTexture != null) {
+                  baseboardTexture = wallSidesBaseboardTexture;
+                }
+                break;
+            }
+          }
+          baseboard = Baseboard.getInstance(
+              wallSidesBaseboardThickness != null
+                  ? wallSidesBaseboardThickness
+                  : baseboardThickness, 
+              wallSidesBaseboardHeight != null
+                  ? wallSidesBaseboardHeight
+                  : baseboardHeight, 
+              baseboardColor, baseboardTexture);
+          if (wallSide.getSide() == WallSide.LEFT_SIDE) {
+            wall.setLeftSideBaseboard(baseboard);
+          } else {
+            wall.setRightSideBaseboard(baseboard);
+          }
+        } 
+      }
     }
   }
 
@@ -1277,8 +1607,9 @@ public class RoomController implements Controller {
       home.deleteWall(newWall.getWall());
     }
     for (ModifiedWall deletedWall : deletedWalls) {
-      deletedWall.reset();
+      deletedWall.resetJoinedWalls();
       home.addWall(deletedWall.getWall());
+      deletedWall.getWall().setLevel(deletedWall.getLevel());
     }
   }
   
@@ -1387,6 +1718,7 @@ public class RoomController implements Controller {
    */
   private class ModifiedWall {
     private Wall          wall;
+    private final Level   level;
     private final Wall    wallAtStart;
     private final Wall    wallAtEnd;
     private final boolean joinedAtEndOfWallAtStart;
@@ -1394,6 +1726,7 @@ public class RoomController implements Controller {
     
     public ModifiedWall(Wall wall) {
       this.wall = wall;
+      this.level = wall.getLevel();
       this.wallAtStart = wall.getWallAtStart();
       this.joinedAtEndOfWallAtStart =
           this.wallAtStart != null
@@ -1408,7 +1741,11 @@ public class RoomController implements Controller {
       return this.wall;
     }
     
-    public void reset() {
+    public Level getLevel() {
+      return this.level;
+    }
+    
+    public void resetJoinedWalls() {
       if (this.wallAtStart != null) {
         this.wall.setWallAtStart(this.wallAtStart);
         if (this.joinedAtEndOfWallAtStart) {
@@ -1436,17 +1773,21 @@ public class RoomController implements Controller {
     private final Integer     wallColor;
     private final HomeTexture wallTexture;
     private final Float       wallShininess;
+    private final Baseboard   wallBaseboard;
 
     public ModifiedWallSide(WallSide wallSide) {
       this.wallSide = wallSide;
+      Wall wall = wallSide.getWall();
       if (wallSide.getSide() == WallSide.LEFT_SIDE) {
-        this.wallColor = wallSide.getWall().getLeftSideColor();
-        this.wallTexture = wallSide.getWall().getLeftSideTexture();
-        this.wallShininess = wallSide.getWall().getLeftSideShininess();
+        this.wallColor = wall.getLeftSideColor();
+        this.wallTexture = wall.getLeftSideTexture();
+        this.wallShininess = wall.getLeftSideShininess();
+        this.wallBaseboard = wall.getLeftSideBaseboard();
       } else {
-        this.wallColor = wallSide.getWall().getRightSideColor();
-        this.wallTexture = wallSide.getWall().getRightSideTexture();
-        this.wallShininess = wallSide.getWall().getRightSideShininess();
+        this.wallColor = wall.getRightSideColor();
+        this.wallTexture = wall.getRightSideTexture();
+        this.wallShininess = wall.getRightSideShininess();
+        this.wallBaseboard = wall.getRightSideBaseboard();
       }
     }
 
@@ -1455,16 +1796,18 @@ public class RoomController implements Controller {
     }
     
     public void reset() {
+      Wall wall = this.wallSide.getWall();
       if (this.wallSide.getSide() == WallSide.LEFT_SIDE) {
-        this.wallSide.getWall().setLeftSideColor(this.wallColor);
-        this.wallSide.getWall().setLeftSideTexture(this.wallTexture);
-        this.wallSide.getWall().setLeftSideShininess(this.wallShininess);
+        wall.setLeftSideColor(this.wallColor);
+        wall.setLeftSideTexture(this.wallTexture);
+        wall.setLeftSideShininess(this.wallShininess);
+        wall.setLeftSideBaseboard(this.wallBaseboard);
       } else {
-        this.wallSide.getWall().setRightSideColor(this.wallColor);
-        this.wallSide.getWall().setRightSideTexture(this.wallTexture);
-        this.wallSide.getWall().setRightSideShininess(this.wallShininess);
+        wall.setRightSideColor(this.wallColor);
+        wall.setRightSideTexture(this.wallTexture);
+        wall.setRightSideShininess(this.wallShininess);
+        wall.setRightSideBaseboard(this.wallBaseboard);
       }
-      Wall wall = wallSide.getWall();
       Wall wallAtStart = wallSide.getWallAtStart();
       if (wallAtStart != null) {
         wall.setWallAtStart(wallAtStart);
