@@ -64,6 +64,7 @@ import com.eteks.sweethome3d.viewcontroller.ObserverCameraController;
 import com.eteks.sweethome3d.viewcontroller.PageSetupController;
 import com.eteks.sweethome3d.viewcontroller.PhotoController;
 import com.eteks.sweethome3d.viewcontroller.PlanController;
+import com.eteks.sweethome3d.viewcontroller.PolylineController;
 import com.eteks.sweethome3d.viewcontroller.PrintPreviewController;
 import com.eteks.sweethome3d.viewcontroller.RoomController;
 import com.eteks.sweethome3d.viewcontroller.TextureChoiceController;
@@ -228,6 +229,14 @@ public class UserPreferencesPanelTest extends TestCase {
         LengthUnit.INCH.getFormatWithUnit().format(LengthUnit.inchToCentimeter(11 + 12)));
     assertEquals("Wrong conversion", "1'11\u215b\"", 
         LengthUnit.INCH.getFormatWithUnit().format(LengthUnit.inchToCentimeter(11 + 12 + 0.125f)));
+    assertEquals("Wrong conversion", "-1'11\u215b\"", 
+        LengthUnit.INCH.getFormatWithUnit().format(LengthUnit.inchToCentimeter(-(11 + 12 + 0.125f))));
+    assertEquals("Wrong conversion", "-0'11\u215b\"", 
+        LengthUnit.INCH.getFormatWithUnit().format(LengthUnit.inchToCentimeter(-(11 + 0.125f))));
+    assertEquals("Wrong conversion", "-0'0\u215b\"", 
+        LengthUnit.INCH.getFormatWithUnit().format(LengthUnit.inchToCentimeter(-0.125f)));
+    assertEquals("Wrong conversion", "-1'", 
+        LengthUnit.INCH.getFormatWithUnit().format(LengthUnit.inchToCentimeter(-12)));
     
     // Test formats with unit
     assertEquals("Wrong conversion", "102 cm", LengthUnit.CENTIMETER.getFormatWithUnit().format(102));
@@ -263,8 +272,34 @@ public class UserPreferencesPanelTest extends TestCase {
         ((Number)LengthUnit.INCH.getFormat().parseObject("11,125")).floatValue(), 1E-10f);
     TestUtilities.assertEqualsWithinEpsilon("Wrong conversion",  LengthUnit.inchToCentimeter(12 + 11 + 3 * 0.125f),
         ((Number)LengthUnit.INCH.getFormat().parseObject("1'11\u215c\"")).floatValue(), 1E-10f);
+    TestUtilities.assertEqualsWithinEpsilon("Wrong conversion",  LengthUnit.inchToCentimeter(12 + 11 + 3 * 0.125f),
+        ((Number)LengthUnit.INCH.getFormat().parseObject("1'11 3/8\"")).floatValue(), 1E-10f);
     TestUtilities.assertEqualsWithinEpsilon("Wrong conversion",  LengthUnit.inchToCentimeter(12 + 11 + 3 * 0.125f),
         ((Number)LengthUnit.INCH.getFormat().parseObject("1' 11 \u215c")).floatValue(), 1E-10f);
+    TestUtilities.assertEqualsWithinEpsilon("Wrong conversion",  LengthUnit.inchToCentimeter(-(12 + 11 + 3 * 0.125f)),
+        ((Number)LengthUnit.INCH.getFormat().parseObject("-1' 11 \u215c")).floatValue(), 1E-10f);
+    TestUtilities.assertEqualsWithinEpsilon("Wrong conversion",  LengthUnit.inchToCentimeter(-(12 + 11 + 3 * 0.125f)),
+        ((Number)LengthUnit.INCH.getFormat().parseObject("-1'11\u215c")).floatValue(), 1E-10f);
+    TestUtilities.assertEqualsWithinEpsilon("Wrong conversion",  LengthUnit.inchToCentimeter(-(12 + 11 + 3 * 0.125f)),
+        ((Number)LengthUnit.INCH.getFormat().parseObject("-1'11 3/8")).floatValue(), 1E-10f);
+    TestUtilities.assertEqualsWithinEpsilon("Wrong conversion",  LengthUnit.inchToCentimeter(-(12 + 3 * 0.125f)),
+        ((Number)LengthUnit.INCH.getFormat().parseObject("-1'0\u215c")).floatValue(), 1E-10f);
+    TestUtilities.assertEqualsWithinEpsilon("Wrong conversion",  LengthUnit.inchToCentimeter(-(12 + 3 * 0.125f)),
+        ((Number)LengthUnit.INCH.getFormat().parseObject("-1'0 3/8")).floatValue(), 1E-10f);
+    TestUtilities.assertEqualsWithinEpsilon("Wrong conversion",  LengthUnit.inchToCentimeter(-(11 + 3 * 0.125f)),
+        ((Number)LengthUnit.INCH.getFormat().parseObject("-0'11\u215c")).floatValue(), 1E-4f);
+    TestUtilities.assertEqualsWithinEpsilon("Wrong conversion",  LengthUnit.inchToCentimeter(-(11 + 3 * 0.125f)),
+        ((Number)LengthUnit.INCH.getFormat().parseObject("-0'11 3/8")).floatValue(), 1E-4f);
+    TestUtilities.assertEqualsWithinEpsilon("Wrong conversion",  LengthUnit.inchToCentimeter(-(3 * 0.125f)),
+        ((Number)LengthUnit.INCH.getFormat().parseObject("-0'0\u215c")).floatValue(), 1E-10f);
+    TestUtilities.assertEqualsWithinEpsilon("Wrong conversion",  LengthUnit.inchToCentimeter(-(3 * 0.125f)),
+        ((Number)LengthUnit.INCH.getFormat().parseObject("-0' 0 \u215c\"")).floatValue(), 1E-10f);
+    TestUtilities.assertEqualsWithinEpsilon("Wrong conversion",  LengthUnit.inchToCentimeter(-0.125f),
+        ((Number)LengthUnit.INCH.getFormat().parseObject("-0'0,125")).floatValue(), 1E-10f);
+    TestUtilities.assertEqualsWithinEpsilon("Wrong conversion",  LengthUnit.inchToCentimeter(-0.125f),
+        ((Number)LengthUnit.INCH.getFormat().parseObject("-0,125")).floatValue(), 1E-10f);
+    TestUtilities.assertEqualsWithinEpsilon("Wrong conversion",  0,
+        ((Number)LengthUnit.INCH.getFormat().parseObject("-0'0")).floatValue(), 1E-10f);
     try {
       LengthUnit.INCH.getFormat().parseObject("'");
       fail("' not a number");
@@ -292,6 +327,9 @@ public class UserPreferencesPanelTest extends TestCase {
     ParsePosition parsePosition = new ParsePosition(0);
     LengthUnit.INCH.getFormat().parseObject("10'2A", parsePosition);
     assertEquals("Wrong parse position", "10'2A".indexOf('A'), parsePosition.getIndex());
+    parsePosition = new ParsePosition(0);
+    LengthUnit.INCH.getFormat().parseObject("0,03/8", parsePosition);
+    assertEquals("Wrong parse position", "0,03/8".indexOf('/'), parsePosition.getIndex());
   }
   
   /**
@@ -335,6 +373,7 @@ public class UserPreferencesPanelTest extends TestCase {
       new LevelController(home, preferences, viewFactory, undoableEditSupport).getView();
       new WallController(home, preferences, viewFactory, contentManager, undoableEditSupport).getView();
       new RoomController(home, preferences, viewFactory, contentManager, undoableEditSupport).getView();
+      new PolylineController(home, preferences, viewFactory, contentManager, undoableEditSupport).getView();
       new LabelController(home, preferences, viewFactory, undoableEditSupport).getView();
       new CompassController(home, preferences, viewFactory, undoableEditSupport).getView();
       new ObserverCameraController(home, preferences, viewFactory).getView();
